; *************************************************************************************************
; *
; *	Title:	EVENT.ASM
; *	Copyright (c) March 1994, Ryu Consulting
; *	Written by Rahner James
; *
; *	This file contains the driver for the event manager
; *
; *************************************************************************************************

	include	driver.inc

Driver_Name		db	"Event manager", 0	; Name of the driver


; *************************************************************************************************
; *
; *				Data relative to this driver
; *
; *************************************************************************************************

	even
Jump_Table label word
	dw	init_driver
	dw	stop_driver

	db	50 dup('Hay')
	db	"Needle"
	db	50 dup('Hay')
Local_Stack label word
Old_SP	dw	0
Old_SS	dw	0


; *************************************************************************************************
; *
; *	STRING_OUT
; *	Outputs an ASCIIz string to the console
; *
; *	Given:
; *		SP -> string to output
; *
; *	Returns:
; *		SI -> 0 terminator
; *
; *************************************************************************************************
string_out proc

	pop	si
@@:	mov	dl, cs:[si]
	inc	si

	or	dl, dl
	jz	short done

	mov	ah, 6
	int	21h
	jmp	@B

done:	jmp	si

string_out endp


; *************************************************************************************************
; *
; *	INIT_DRIVER
; *	Initialize this driver
; *
; *	Given:
; *		AX = 0
; *		SS = CS
; *		DS = ES = global data segment
; *
; *	Returns:
; *		Driver initialized, no registers (except CS,SS,SP) preserved
; *
; *************************************************************************************************
init_driver proc

ifdef DEBUG
	STROUT	"Initializing event manager ..."
	CRLF
endif
	cli
	mov	dword ptr ds:ET_S.total_free_events, MAX_EVENT_PACKETS	; Set the maximum number of event packets
;	mov	ET_S.first_free_event, 0				; First is at the beginning, set by previous mov
	mov	dword ptr ds:ET_S.total_queued_events, 0		; None queued
;	mov	ds:ET_S.first_queued_event, 0				; Start at the beginning, set by previous mov
	mov	ds:ET_S.count_missed_events, 0				; None missed yet

	mov	word ptr ds:ET_S.vector_get_packet, offset get_event_envelope
	mov	word ptr ds:ET_S.vector_get_packet+2, cs
	sti

	ret

init_driver endp


; *************************************************************************************************
; *
; *	STOP_DRIVER
; *	Stops this driver
; *
; *	Given:
; *		AX = 1
; *		SS = CS
; *		DS = ES = global data segment
; *
; *	Returns:
; *		Driver stopped, all INTs returned, no registers (except CS,SS,SP) preserved
; *
; *************************************************************************************************
stop_driver proc

ifdef DEBUG
	STROUT	"Stopping event manager ..."
	CRLF
endif
	cli
	mov	dword ptr ds:ET_S.total_free_events, MAX_EVENT_PACKETS	; Set the maximum number of event packets
;	mov	ET_S.first_free_event, 0				; First is at the beginning, set by previous mov
	mov	dword ptr ds:ET_S.total_queued_events, 0		; None queued
;	mov	ds:ET_S.first_queued_event, 0				; Start at the beginning, set by previous mov
	mov	ds:ET_S.count_missed_events, 0				; None missed yet

	mov	word ptr ds:ET_S.vector_get_packet, ET_S.set_carry	; Make the get packet function -> STC/RETF
	mov	word ptr ds:ET_S.vector_get_packet+2, ds
	sti

	ret

stop_driver endp


; *************************************************************************************************
; *
; *	GET_EVENT_ENVELOPE
; *	Gets a pointer to the next available empty packet
; *
; *	Given:
; *		DS = global data segment
; *
; *	Returns:
; *		CARRY set if there were no packets available
; *		EDI -> packet that is ready to be filled
; *		Interrupts enabled
; *		EVT.time set to current time, all other fields 0'ed
; *		EAX = current time, all other registers preserved
; *
; *************************************************************************************************
get_event_envelope proc far uses eax

	cli
	xor	eax, eax
	cmp	ds:ET_S.total_free_events, ax		; See if we have any to fill
	jz	short derr				; Quit if we are full

	dec	ds:ET_S.total_free_events		; One less event to be filled
	movzx	edi, byte ptr ds:ET_S.first_free_event	; Assumes 256 event packets
	inc	ds:ET_S.first_free_event		; Bump the event index
	sti

	shl	di, 4					; EDI = offset of the packet
	add	di, ET_S.event_packets			; EDI -> correct event packet, also clears CARRY

	mov	[edi], eax				; Clear the packet
	mov	[edi+4], eax
	mov	[edi+8], eax
	mov	[edi+12], eax
	mov	eax, ds:ET_S.tick_count
	mov	[edi+2], eax

done:	ret

derr:	inc	ds:ET_S.count_missed_events	; Bump the number of missed events
	stc					; Set CARRY
	sti					; Re-enable ints
	jmp	done

get_event_envelope endp


; *************************************************************************************************
; *
; *	START_OF_CODE
; *	This is the entry point from the protected-mode client
; *
; *	Given:
; *		AX = task to perform,
; *			0 = initialization
; *			1 = disengage
; *
; *	Returns:
; *		Jumps to next in line after completion
; *		All registers can be destroyed except EAX, SS, SP
; *
; *************************************************************************************************
start_of_code:
	push	ax
	mov	cs:Old_SS, ss
	mov	cs:Old_SP, sp

	cmp	ax, 2			; See if too big
	jnc	short @F

	cld
	sti
	mov	ds, word ptr cs:Global_Data_Ptr+2
	mov	es, word ptr cs:Global_Data_Ptr+2
	mov	bx, cs
	mov	ss, bx
	mov	sp, offset Local_Stack

	mov	bx, ax
	add	bx, bx			; BX *= 2
	call	cs:Jump_Table[bx]

@@:	lss	sp, dword ptr cs:Old_SP
	pop	ax
	jmp	dword ptr cs:[Next_Driver]


	end

