; *************************************************************************************************
; *
; *	Title:	EVENT.ASM
; *	Copyright (c) January 1993, N, Inc.
; *	Written by Rahner James
; *
; *	This file contains all the scheduler functions
; *
; *************************************************************************************************

_EVENT_ASM_	equ	1
	include	mouse.inc

MAX_EVENTS	equ	25

HELP_ENABLED	equ	(1 shl 0)	; Set if help is enabled
IN_HELP	equ	(1 shl 1)	; Set if help currently being accessed


.data

	public	_Mouse_X, _Mouse_Y, _Mouse_Button, _Mouse_Motion_Vector

	even
Help_Function_Ptr	dd	0
Help_Flags		dw	0	; b0:enabled, b1:in_help
Help_Key		dw	0

Base_Segment		dw	0	; Base segment of the allocated packets
End_Segment		dw	0	; Segment beyond the last one
Total_Free		dw	0	; Number of free packets
First_Free		dw	0	; -> first available packet
Total_Queued		dw	0	; Number of queued packets
First_Queued		dw	0	; -> first queued event

_Mouse_Motion_Vector	dd	0	; Function to call when mouse has moved
_Mouse_X		dw	0	; Current mouse X coordinate
_Mouse_Y		dw	0	; Current mouse Y coordinate

Event_Mask		dw	7Eh	; Event mask to use for the event processing
_Mouse_Button		db	0	; Current mouse button status
Keyboard_Semaphore	db	0	; Set to !0 when getting keyboard input

System_Ticks		dd	0	; Current system count
First_Timer_Event	dd	0	; -> first timer event
Total_Timer_Events	dw	0	; Number of timer events
Timer_Semaphore	db	0,0
Old_SS			dw	0
Old_SP			dw	0
			dw	256 dup(0)
Our_Stack		label	word

.code
Kbd_Intercept_Vector	dd	0	; -> old keyboard intercept vector
Old_Timer_Vector	dd	0
Our_Data		dw	@Data
if @Codesize
	extern	ipx_listen_packet:far
else
	extern	ipx_listen_packet:near
endif

; *************************************************************************************************
; *
; *	int START_EVENTS( int EVENT_PACKETS )
; *	Initializes all event packets
; *	Starts up the scheduler and allocates it's memory
; *
; *	Given:
; *		EVENT_PACKETS = number of events that can be scheduled
; *
; *	Returns:
; *		0 if it was started OK
; *		-86 if not enough memory
; *
; *************************************************************************************************
if @Datasize
start_events proc uses ds, event_packets:word
else
start_events proc, event_packets:word
endif

if @Datasize
	mov	ds, Our_Data
endif
	cmp	Base_Segment, 0		; See if we have done this before
	jz	@F			; Skip if we have
	xor	ax, ax
	jmp	short done_start_events

@@:	mov	ax, event_packets
	or	ax, ax			; See if we should allocated some
	jnz	@F
	mov	ax, MAX_EVENTS

@@:	mov	Total_Free, ax		; Set the number we have here
	mov	Total_Queued, 0

	mov	bx, ax			; BX = number of paragraphs to allocate
	mov	ah, 48h			; AH = DOS Allocate Memory Block command
	int	21h
	jnc	@F			; Skip if no error encountered

	mov	ax, -86			; Return a memory error here
	jmp	done10_start_events

; *
; * Initialize the event circular list
; *
@@:	mov	Base_Segment, ax	; Set the pointers to the start
	mov	First_Free, ax
	mov	First_Queued, ax
	add	ax, Total_Free
	mov	End_Segment, ax

; *
; * Set the timer vector
; *
	xor	ax, ax
	mov	es, ax
	mov	dx, cs
	mov	ax, offset timer_isr
	cli
	xchg	es:[1ch*4], ax
	xchg	es:[1ch*4+2], dx
	mov	word ptr Old_Timer_Vector, ax
	mov	word ptr Old_Timer_Vector+2, dx
	sti

; *
; * See if the INT-15 vector is supported
; *
	mov	ah, 0c0h		; Get environment table pointer
	int	15h
	test	byte ptr es:[bx+5], 1 shl 4
	jz	short done_start_events

	xor	ax, ax
	mov	es, ax
	mov	dx, cs
	mov	ax, offset keyboard_intercept
	cli
	xchg	es:[15h*4], ax
	xchg	es:[15h*4+2], dx
	mov	word ptr Kbd_Intercept_Vector, ax
	mov	word ptr Kbd_Intercept_Vector+2, dx
	sti

done_start_events:
	mov	ax, cs
	mov	es, ax
	mov	dx, offset esr_mouse
	mov	cx, Event_Mask
	and	cx, 7fh
	MOUSE	12

	xor	ax, ax

	cli
	mov	word ptr System_Ticks, ax	; Current system count
	mov	word ptr System_Ticks+2, ax
	mov	word ptr First_Timer_Event, ax	; -> first timer event
	mov	word ptr First_Timer_Event+2, ax
	mov	Total_Timer_Events, ax		; Number of timer events
	mov	Timer_Semaphore, al
	sti

done10_start_events:
	ret

start_events endp


; *************************************************************************************************
; *
; *	void STOP_EVENTS( void )
; *	Stops all event processing
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		Events and INT vectors stopped
; *
; *************************************************************************************************
if @Datasize
stop_events proc uses ds
	mov	ds, Our_Data
else
stop_events proc
endif

	xor	ax, ax
	cmp	Base_Segment, ax	; See if we have done this before
	jz	done_stop_events	; Skip if we have

; *
; * Reset the timer vector
; *
	mov	es, ax
	cli

	mov	dx, word ptr Old_Timer_Vector
	mov	es:[1ch*4], dx
	mov	dx, word ptr Old_Timer_Vector+2
	mov	es:[1ch*4+2], dx

	cmp	word ptr Kbd_Intercept_Vector+2, ax	; See if it's 0
	jz	short @F
	mov	dx, word ptr Kbd_Intercept_Vector
	mov	es:[15h*4], dx
	mov	dx, word ptr Kbd_Intercept_Vector+2
	mov	es:[15h*4+2], dx

@@:	sti

	MOUSE	21h

	mov	es, Base_Segment
	mov	Base_Segment, 0
	mov	ah, 49h			; DOS Release Memory Block command
	int	21h

done_stop_events:
	ret

stop_events endp


; *************************************************************************************************
; *
; *	GET_KEY
; *	Gets the next keystroke
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		AH = keyboard scan code
; *		AL = ASCII value
; *		ZERO set if no key available
; *
; *************************************************************************************************
get_key proc near private

	mov	ah, 11h			; BIOS Get Keyboard Status command
	int	16h
	jz	short done		; Quit if none

; *
; * Key was waiting, let's see if it's extended or not
; *
	cmp	ah, 85h			; See if F11
	je	do_extended
	cmp	ah, 86h			; See if F12
	je	do_extended

	mov	ah, 0			; BIOS Read Character from Keyboard
get10_key:
	int	16h
	or	ah, ah			; This should always be NZ

done:	ret

do_extended:
	mov	ah, 10h
	jmp	get10_key

get_key endp


; *************************************************************************************************
; *
; *	GET_EVENT_ENVELOPE
; *	Gets the next available empty event envelope
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		ES:DI -> envelope to fill, time value filled in
; *		CARRY set if none available and AX = -87
; *
; *************************************************************************************************
get_event_envelope proc near private

if @Datasize
	mov	ds, Our_Data
endif
	pushf
	cli
	cmp	Base_Segment, 0		; See if we have allocated memory for the event manager
	jz	short derr		; Quit if we haven't
	cmp	Total_Free, 0		; See if we have any to fill
	jz	short derr		; Quit if we are full

	dec	Total_Free
	inc	Total_Queued
	mov	ax, First_Free
	mov	bx, ax

	inc	ax
	cmp	End_Segment, ax		; See if we went over the top
	ja	@F
	mov	ax, Base_Segment
@@:	mov	First_Free, ax

	mov	dx, word ptr System_Ticks+2	; DX:AX = current time
	mov	ax, word ptr System_Ticks
	popf

	mov	es, bx				; ES:[SI] -> first free buffer
	xor	di, di				; DI = 0, also clears CARRY for good return
	mov	word ptr es:[di].MOUSE_EVT.time+2, dx
	mov	word ptr es:[di].MOUSE_EVT.time, ax

	ret

derr:	mov	ax, -87
	popf
	stc
	ret

get_event_envelope endp


; *************************************************************************************************
; *
; *	int PUT_EVENT( EVT FAR *EVENT )
; *	Places an event packet into the event process queue
; *
; *	Given:
; *		EP -> event packet to enqueue
; *
; *	Returns:
; *		0 if it was enqueued
; *		-87 if we are full
; *
; *************************************************************************************************
put_event proc uses ds di si, ep:dword

put10_event::
if NULL_CHECK
	mov	si, word ptr ep		; Make sure we didn't get a NULL pointer
	mov	ax, word ptr ep+2
	or	ax, si
	jz	short done_put_event
endif

	call	get_event_envelope	; Get the next available event entry
	jc	short done_put_event

	lds	si, ep			; DS:SI -> new event packet
	cld
	movsw				; Move the etype and emask fields
	add	di, 4			; Skip the timer field
	add	si, 4
	mov	cx, 10/2
	rep	movsw
	xor	ax, ax			; AX = 0 for good return

done_put_event:
	ret

put_event endp


; *************************************************************************************************
; *
; *	int PUT_MOUSE_EVENT( EVT FAR *EVENT )
; *	Places a mouse event packet into the event process queue
; *
; *	Given:
; *		EP -> event packet to enqueue
; *
; *	Returns:
; *		0 if it was enqueued
; *		-87 if we are full
; *
; *************************************************************************************************
put_mouse_event proc uses ds di si, ep:dword

	les	di, ep
if NULL_CHECK
	mov	ax, es
	or	ax, di
	jz	short @F
endif
	mov	es:[di].MOUSE_EVT.etype, 1	; Make it a mouse event
	jmp	put10_event

@@:	ret

put_mouse_event endp


; *************************************************************************************************
; *
; *	int PUT_KBD_EVENT( EVT FAR *EVENT )
; *	Places a keyboard event packet into the event process queue
; *
; *	Given:
; *		EP -> event packet to enqueue
; *
; *	Returns:
; *		0 if it was enqueued
; *		-87 if we are full
; *
; *************************************************************************************************
put_kbd_event proc uses ds di si, ep:dword

	les	di, ep
if NULL_CHECK
	mov	ax, es
	or	ax, di
	jz	short @F
endif
	mov	es:[di].MOUSE_EVT.etype, 2	; Make it a keyboard event
	jmp	put10_event

@@:	ret

put_kbd_event endp


; *************************************************************************************************
; *
; *	int PUT_CALLBACK_EVENT( void (FAR *FUNCTION)(), void FAR *BUFFER, unsigned short VALUE )
; *	Places a callback event packet into the event process queue
; *
; *	Given:
; *		FUNCTION -> function to call
; *		BUFFER -> buffer to use
; *		VALUE = value to set value to
; *
; *	Returns:
; *		0 if it was enqueued
; *		-87 if we are full
; *
; *************************************************************************************************
put_callback_event proc uses ds di, function:dword, buffer:dword, value:word

	call	get_event_envelope	; Get the next available event entry
	jc	short @F

	mov	word ptr es:[di].CALLBACK_EVT.etype, 6
	mov	ax, word ptr function
	mov	word ptr es:[di].CALLBACK_EVT.function, ax
	mov	ax, word ptr function+2
	mov	word ptr es:[di].CALLBACK_EVT.function+2, ax
	mov	ax, word ptr buffer
	mov	word ptr es:[di].CALLBACK_EVT.vector, ax
	mov	ax, word ptr buffer+2
	mov	word ptr es:[di].CALLBACK_EVT.vector+2, ax
	mov	ax, value
	mov	es:[di].CALLBACK_EVT.value, ax
	xor	ax, ax

@@:	ret

put_callback_event endp


; *************************************************************************************************
; *
; *	KEYBOARD_INTERCEPT
; *	Called whenever INT-15 is called
; *
; *	Given:
; *		AH = INT-15h paramter
; *
; *	Returns:
; *		Only if event buffer is full
; *
; *************************************************************************************************
keyboard_intercept proc far private

	cmp	ah, 4fh			; See if this is the one
	jne	short not_us
	or	al, al			; Don't need break keys and specials
	jz	short not_us

	sti
	push	ds
	push	es
	push	di
	push	dx
	push	cx
	push	bx
	push	ax

	mov	ds, Our_Data
	call	get_event_envelope	; See if we can pack it in
	jc	short @F

	mov	word ptr es:[di].MOUSE_EVT.etype, 2
	mov	word ptr es:[di].MOUSE_EVT.ascii, 0

	mov	bx, es
	xor	ax, ax
	mov	es, ax
	mov	al, es:[417h]		; AL = current keyboard state
	mov	es, bx
	mov	es:[di].MOUSE_EVT.state, ax

	pop	ax
	pop	bx
	pop	cx
	pop	dx
	pop	di
	pop	es
	pop	ds

not_us:
	jmp	dword ptr [Kbd_Intercept_Vector]

@@:	clc				; Clear CARRY
	pop	ax
	pop	bx
	pop	cx
	pop	dx
	pop	di
	pop	es
	pop	ds
	ret	2

keyboard_intercept endp


; *************************************************************************************************
; *
; *	PUT_SERIAL_EVENT
; *	Places a serial event into the event process queue
; *
; *	Given:
; *		AH = line status
; *		AL = modem status
; *		CH = serial event type, 0=byte in, 1=modem change, 2=error condition
; *		CL = COM number
; *
; *	Returns:
; *		0 if it was enqueued
; *		-87 if we are full
; *
; *************************************************************************************************
put_serial_event proc uses ds es bx dx di

	push	ax
	push	cx
	call	get_event_envelope	; Get the next available event entry
	pop	cx
	pop	ax
	jc	short done

	mov	es:[di].SERIAL_EVT.etype, 4
	mov	es:[di].SERIAL_EVT.emask, ch
	mov	es:[di].SERIAL_EVT.status, ax
	mov	es:[di].SERIAL_EVT.com_number, cl	; COM number of the event

done:	ret

put_serial_event endp


; *************************************************************************************************
; *
; *	PUT_NETWORK_EVENT
; *	Places a network event into the event process queue
; *
; *	Given:
; *		ES:SI -> ECB to enqueue
; *
; *	Returns:
; *		DX:BX -> network event packet
; *		CARRY set if there was a problem
; *		-87 if we are full
; *
; *************************************************************************************************
put_network_event proc uses ds bx di

	mov	al, es:[si].ECB_S.completionCode
	push	ax
	push	es
	push	si
	call	get_event_envelope	; Get the next available event entry
	pop	si
	pop	ax
	pop	bx			; Restore the completion code
	jc	short done

	mov	es:[di].NETWORK_EVT.etype, 5
	mov	es:[di].NETWORK_EVT.emask, 0
	mov	word ptr es:[di].NETWORK_EVT.ecb, si
	mov	word ptr es:[di].NETWORK_EVT.ecb+2, ax
	mov	es:[di].NETWORK_EVT.completionCode, bl
	mov	es:[di].NETWORK_EVT.connection, -1
	mov	dx, es
	mov	es, ax
	mov	bx, di			; DX:BX -> event packet already enqueued

done:	ret

put_network_event endp


; *************************************************************************************************
; *
; *	int GET_EVENT( EVT FAR *EP )
; *	Fills an event entry with an event or no-dice
; *
; *	Given:
; *		EP -> event structure to fill
; *
; *	Returns:
; *		0 if event filled
; *		-4020 (grERR_EQEMPTY from graphics lib)
; *
; *************************************************************************************************
if @Datasize
get_event proc uses ds si di, ep:dword
local	rv:word

	mov	ds, Our_Data
else
get_event proc uses di si, ep:dword
local	rv:word
endif

	mov	rv, 0
	les	di, ep			; ES:DI -> destination event packet
if NULL_CHECK
	mov	ax, es
	or	ax, di
	jz	short derr_get_event	; Quit if NULL pointer
endif
	xor	ax, ax				; AX = 0

	mov	es:[di].MOUSE_EVT.etype, al	; Set the type to none, in case that's what it is

	cmp	Base_Segment, ax
	jz	short derr_get_event
	cmp	Total_Queued, ax
	jnz	get10_event		; Skip if we have an event
	cmp	word ptr Kbd_Intercept_Vector+2, ax
	jnz	short derr_get_event

; *
; * Check the keyboard if event queue is empty
; *
	inc	al			; AL = 1
	xchg	Keyboard_Semaphore, al	; See if we can do this
	or	al, al
	jz	get03_event		; Skip if we can

derr_get_event:
	mov	rv, -4020
	jmp	done_get_event

get03_event:
	call	get_key
	jnz	short @F		; Skip if there was a key

; *
; * No key waiting, just set the current state
; *
	push	es
	xor	ax, ax
	mov	Keyboard_Semaphore, al
	mov	es, ax
	mov	al, es:[417h]		; AL = current keyboard state
	pop	es
	mov	es:[di].MOUSE_EVT.state, ax
	jmp	derr_get_event

; *
; * Put the keyboard event into the receiving buffer
; *
@@:	mov	word ptr es:[di].MOUSE_EVT.etype, 2
	mov	word ptr es:[di].MOUSE_EVT.ascii, ax
	mov	ah, 2			; BIOS Get Keyboard Flags command
	int	16h
	mov	ah, _Mouse_Button
	mov	es:[di].MOUSE_EVT.state, ax
	mov	ax, _Mouse_X
	mov	es:[di].MOUSE_EVT.x, ax
	mov	ax, _Mouse_Y
	mov	es:[di].MOUSE_EVT.y, ax

	cli
	mov	ax, word ptr System_Ticks
	mov	word ptr es:[di].MOUSE_EVT.time, ax
	mov	ax, word ptr System_Ticks+2
	mov	word ptr es:[di].MOUSE_EVT.time+2, ax

get05_event:
	xor	ax, ax
	mov	Keyboard_Semaphore, al
	jmp	short done_get_event

; *
; * Take the next event from the queue
; *
get10_event:
	mov	ds, First_Queued	; DS:SI -> first queued buffer
	xor	si, si

	cmp	[si].MOUSE_EVT.etype, 2	; Keyboard type
	jne	short @F

	mov	rv, -4020
	mov	[si].MOUSE_EVT.etype, 0
	call	get_key
	jz	short @F
	mov	[si].MOUSE_EVT.etype, 2
	mov	rv, 0
	mov	word ptr [si].MOUSE_EVT.ascii, ax

@@:	mov	cx, 16/2
	cld
	rep	movsw

	mov	ds, Our_Data
@@:	inc	Total_Free
	dec	Total_Queued
	mov	ax, First_Queued
	inc	ax
	cmp	End_Segment, ax		; See if we went over the top
	ja	@F
	mov	ax, Base_Segment
@@:	mov	First_Queued, ax

done_get_event:
	sti

	mov	ax, rv
	ret

get_event endp


; *************************************************************************************************
; *
; *	int CLEAR_EVENTS( void )
; *	Clears all pending events
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		0 if everything was cleared OK
; *
; *************************************************************************************************
if @Datasize
clear_events proc uses ds
	mov	ds, Our_Data
else
clear_events proc
endif
	cmp	Base_Segment, 0
	jz	short done_clear_events

	pushf

; *
; * Check the event queue for pending values
; *
clear10_events:
	cli
	cmp	Total_Queued, 0		; See if there are any events queued up
	jz	clear20_events		; Quit if they are all clear

	sti
; *
; * Clear the event from the queue
; *
	mov	ax, First_Queued	; AX = first in the queue of packets
	mov	es, ax
	inc	ax
	cmp	ax, End_Segment
	jc	@F
	mov	ax, Base_Segment
@@:	mov	First_Queued, ax
	dec	Total_Queued		; One less in the queue

	test	es:MOUSE_EVT.etype, 5	; See if it's a network event
	jne	clear10_events

	push	word ptr es:NETWORK_EVT.ecb+2	; Send the segment to the ECB
	push	word ptr es:NETWORK_EVT.ecb
	call	ipx_listen_packet
	add	sp, 4
	jmp	clear10_events

; *
; * Initialize the event queue
; *
clear20_events:
	mov	ax, Base_Segment
	mov	First_Free, ax
	mov	First_Queued, ax

	mov	ax, Total_Queued
	add	Total_Free, ax
	xor	ax, ax
	mov	Total_Queued, ax

	popf
done_clear_events:
	ret

clear_events endp


; *************************************************************************************************
; *
; *	int WAIT_EVENT( EVT FAR *EP )
; *	Waits for an event to occur
; *
; *	Given:
; *		EP -> event structure to fill
; *
; *	Returns:
; *		0 if event filled
; *		-4020 (grERR_EQEMPTY from graphics lib)
; *
; *************************************************************************************************
if @Datasize
wait_event proc uses ds di, ep:dword
	mov	ds, Our_Data
else
wait_event proc uses di, ep:dword
endif

	mov	ax, -1
	cmp	Base_Segment, 0
	jz	short derr_wait_event
	les	di, ep
	push	es
	push	di

@@:	call	get_event
	or	ax, ax
	jnz	@B
	add	sp, 4

derr_wait_event:
	ret

wait_event endp


; *************************************************************************************************
; *
; *	int WAIT_TEVENT( EVT FAR *EP, unsigned int TIMEOUT )
; *	Waits for an event to occur with a timeout
; *
; *	Given:
; *		EP -> event structure to fill
; *		TIMEOUT = number of system ticks to wait for the event
; *
; *	Returns:
; *		0 if event filled
; *		-1 if function not initialized
; *		-2 if timeout occurred
; *
; *************************************************************************************************
if @Datasize
wait_tevent proc uses ds di, ep:dword, timeout:word
local	end_time:dword
	mov	ds, Our_Data
else
wait_tevent proc uses di, ep:dword, timeout:word
local	end_time:dword
endif

	cli
	mov	ax, word ptr System_Ticks	; DX:AX = current time
	mov	dx, word ptr System_Ticks+2
	add	ax, timeout			; DX:AX = timeout time
	adc	dx, 0
	mov	word ptr end_time, ax
	mov	word ptr end_time+2, dx
	sti

	mov	ax, -1
	cmp	Base_Segment, 0
	jz	short derr_wait_event

	les	di, ep			; ES:DI -> event block
	push	es
	push	di

@@:	call	get_event			; See if there is an event
	or	ax, ax
	jz	@F				; Skip if there was

	cli
	mov	ax, word ptr System_Ticks	; See if we timed out
	mov	dx, word ptr System_Ticks+2
	sti
	sub	ax, word ptr end_time		; Do a destructive comparison
	sbb	dx, word ptr end_time+2
	jc	@B				; Loop if no timeout yet

; *
; * Wait timed out
; *
	les	di, ep				; ES:DI -> event block
	mov	es:[di].MOUSE_EVT.etype, 0	; Time event
	mov	es:[di].MOUSE_EVT.state, 0
	mov	es:[di].MOUSE_EVT.x, 0
	mov	es:[di].MOUSE_EVT.y, 0
	cli
	mov	ax, word ptr System_Ticks	; See if we timed out
	mov	dx, word ptr System_Ticks+2
	sti
	mov	word ptr es:[di].MOUSE_EVT.time, ax
	mov	word ptr es:[di].MOUSE_EVT.time+2, dx
	mov	ax, -2

@@:	add	sp, 4

derr_wait_event:
	ret

wait_tevent endp


; *************************************************************************************************
; *
; *	void FAR ESR_MOUSE
; *	Processes a mouse event
; *
; *	Given:
; *		AX = condition mask
; *		BX = button state
; *		CX = X cursor coordinate
; *		DX = Y cursor coordinate
; *
; *	Returns:
; *		All registers unchanged
; *
; *************************************************************************************************
esr_mouse proc far private uses ds es

	mov	ds, Our_Data

	mov	_Mouse_X, cx
	mov	_Mouse_Y, dx
	mov	_Mouse_Button, bl
	and	al, byte ptr Event_Mask
	and	al, 0FEh		; See if it's just a position change
	jz	short esr20_mouse

	pushf
	cli
	cmp	Total_Free, 0		; See if we have any to fill
	jz	short esr10_mouse	; Quit if we are full

	mov	si, ax
	mov	ah, bl			; AH = button state
	xor	bx, bx			; BX = 0
	mov	es, bx			; ES -> base segment
	mov	al, es:[417h]		; AL = keyboard flags
	mov	es, First_Free		; ES:0 -> first free buffer

	mov	word ptr es:MOUSE_EVT.ascii, bx
	mov	es:MOUSE_EVT.state, ax
	mov	ax, si
	mov	es:MOUSE_EVT.emask, ah
	mov	es:MOUSE_EVT.etype, 1

	mov	es:MOUSE_EVT.x, cx
	mov	es:MOUSE_EVT.y, dx
	mov	ax, word ptr System_Ticks
	mov	word ptr es:MOUSE_EVT.time, ax
	mov	ax, word ptr System_Ticks+2
	mov	word ptr es:MOUSE_EVT.time+2, ax

	dec	Total_Free
	inc	Total_Queued
	mov	ax, First_Free
	inc	ax
	cmp	End_Segment, ax		; See if we went over the top
	ja	@F
	mov	ax, Base_Segment
@@:	mov	First_Free, ax
esr10_mouse:
	popf

done_esr_mouse:
	ret

esr20_mouse:
	cmp	word ptr _Mouse_Motion_Vector+2, 0
	jz	done_esr_mouse
	call	dword ptr [_Mouse_Motion_Vector]
	jmp	done_esr_mouse

esr_mouse endp


; *************************************************************************************************
; *
; *	void INIT_HELP( void (*FUNCTION)( void ), us KEY )
; *	Initializes the help function
; *
; *	Given:
; *		FUNCTION -> function to call when help key pressed
; *		KEY = help key
; *
; *	Returns:
; *		nothing
; *
; *************************************************************************************************
if @Datasize
init_help proc uses ds, function:dword, key:word
	mov	ds, Our_Data
else
init_help proc, function:dword, key:word
endif

	mov	ax, key
	mov	Help_Key, ax
	inc	ah
	jnz	@F
	xchg	ah, al
	mov	Help_Key, ax
@@:	mov	ax, word ptr function
	mov	bx, word ptr function+2
	mov	word ptr Help_Function_Ptr, ax
	mov	word ptr Help_Function_Ptr+2, bx

	and	Help_Flags, not HELP_ENABLED
	or	ax, bx
	jz	@F
	or	Help_Flags, HELP_ENABLED

@@:	ret

init_help endp


; *************************************************************************************************
; *
; *	void ENABLE_HELP( void )
; *	Enables the help key
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		nothing
; *
; *************************************************************************************************
if @Datasize
enable_help proc uses ds
	mov	ds, Our_Data
else
enable_help proc
endif

	and	Help_Flags, not HELP_ENABLED
	mov	ax, word ptr Help_Function_Ptr
	or	ax, word ptr Help_Function_Ptr+2

	jz	@F
	or	Help_Flags, HELP_ENABLED

@@:	ret

enable_help endp


; *************************************************************************************************
; *
; *	void DISABLE_HELP( void )
; *	Disables the help key
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		nothing
; *
; *************************************************************************************************
if @Datasize
disable_help proc uses ds
	mov	ds, Our_Data
else
disable_help proc
endif

	and	Help_Flags, not HELP_ENABLED
	ret

disable_help endp


; *************************************************************************************************
; *
; *	unsigned short SET_EVENT_MASK( unsigned short EMASK )
; *	Sets the current event mask
; *
; *	Given:
; *		EMASK = mask for the events coming in
; *
; *	Returns:
; *		AX = old event mask
; *
; *************************************************************************************************
set_event_mask proc, emask:word

	mov	es, Our_Data
	mov	ax, emask
	xchg	es:Event_Mask, ax

	ret

set_event_mask endp


; *************************************************************************************************
; *
; *	int GET_EVENT_STATUS( void )
; *	Gets the current number of pending events
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		AX = number of events waiting
; *
; *************************************************************************************************
get_event_status proc

if @Datasize
	mov	es, Our_Data
	mov	ax, es:Total_Queued
else
	mov	ax, Total_Queued
endif

	ret

get_event_status endp


; *************************************************************************************************
; *
; *	void FAR TIMER_ISR
; *	Processes a system tick
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		All registers unchanged
; *
; *************************************************************************************************
timer_isr proc far private

	push	ds				; Save the registers we are gunna use
	mov	ds, Our_Data

	add	word ptr System_Ticks, 1
	adc	word ptr System_Ticks+2, 0

	cmp	Total_Timer_Events, 0		; Number of timer events
	jnz	short @F

timer05_isr:
	pop	ds
	jmp	dword ptr [Old_Timer_Vector]

done_timer_isr:
	popa					; Restore the saved registers
	pop	es
	mov	ss, Old_SS
	mov	sp, Old_SP

	mov	Timer_Semaphore, 0		; Clear the blocking semaphire

	jmp	timer05_isr

; *
; * Process the timers in the list
; *
@@:	cmp	Timer_Semaphore, 0		; See if we are already here
	jnz	short timer05_isr
	mov	Timer_Semaphore, 1		; Set the blocking semaphore

	mov	Old_SS, ss
	mov	Old_SP, sp
	mov	ss, Our_Data
	mov	sp, offset Our_Stack

	sti

	push	es
	pusha

	les	di, First_Timer_Event		; -> first timer event
	mov	dx, seg First_Timer_Event	; DX:BX -> last timer event structure
	mov	bx, offset First_Timer_Event	; This expects the next pointer to be the first field
timer10_isr:
	mov	ax, es				; See if we are at the end of the line
	or	ax, di
	jz	short done_timer_isr		; Quit if hit NULL pointer

; *
; * Process the timer event in ES:DI
; *
	test	es:[di].EVT_TIMER.flags, TMR_ACTIVE	; See if timer is currently being updated or deleted
	jz	short pre_timer100			; Skip this update if it is

	sub	word ptr es:[di].EVT_TIMER.count, 1	; Decrement the counter
	sbb	word ptr es:[di].EVT_TIMER.count+2, 0
	jae	short pre_timer100			; Continue loop if we haven't reached the end yet

; *
; * See if we call the function
; *
	test	es:[di].EVT_TIMER.flags, TMR_CALL_FUNCTION
	jz	short timer20_isr

	mov	ax, word ptr es:[di].EVT_TIMER.function
	or	ax, word ptr es:[di].EVT_TIMER.function+2
	jz	short timer30_isr
	push	es
	call	es:[di].EVT_TIMER.function
	pop	es
	jmp	short timer30_isr
pre_timer100:
	jmp	timer100_isr

; *
; * Put timeout message into the event queue
; *
timer20_isr:
	push	es			; Save ES:DI
	push	di
	call	get_event_envelope	; Get the next available event entry
	jc	short timer25_isr

	pop	si			; DS:SI -> timer structure
	pop	ds
	push	ds			; Save it again
	push	si

	mov	es:[di].TMR_EVT.etype, 3
	mov	ax, [si].EVT_TIMER.flags
	mov	es:[di].TMR_EVT.emask, al
	mov	ax, [si].EVT_TIMER.value
	mov	es:[di].TMR_EVT.value, ax
	mov	ax, word ptr [si].EVT_TIMER.vector
	mov	word ptr es:[di].TMR_EVT.vector, ax
	mov	ax, word ptr [si].EVT_TIMER.vector+2
	mov	word ptr es:[di].TMR_EVT.vector+2, ax

	mov	word ptr es:[di].TMR_EVT.timer, si
	mov	word ptr es:[di].TMR_EVT.timer+2, ds

	mov	ds, Our_Data

timer25_isr:
	pop	di			; Restore ES:DI
	pop	es

; *
; * See if retriggerable
; *
timer30_isr:
	test	es:[di].EVT_TIMER.flags, TMR_SET_BYTE
	jz	short timer35_isr

	mov	bx, word ptr es:[di].EVT_TIMER.vector
	mov	dx, word ptr es:[di].EVT_TIMER.vector+2
	mov	cx, dx
	or	cx, bx
	jz	short timer35_isr
	mov	cx, es
	mov	es, dx
	mov	byte ptr es:[bx], 1
	mov	es, cx

; *
; * See if retriggerable
; *
timer35_isr:
	test	es:[di].EVT_TIMER.flags, TMR_RETRIGGER
	jnz	short timer40_isr	; Skip to reloading the counter

; *
; * Eliminate the timer event from the chain
; *
	mov	ax, word ptr es:[di].EVT_TIMER.next	; CX:AX -> next structure
	mov	cx, word ptr es:[di].EVT_TIMER.next+2
	mov	si, es
	mov	es, dx
	mov	word ptr es:[bx].EVT_TIMER.next, ax
	mov	word ptr es:[bx].EVT_TIMER.next+2, cx
	mov	es, si
	jmp	short timer100_isr

; *
; * Reload the count
; *
timer40_isr:
	mov	ax, word ptr es:[di].EVT_TIMER.counter_reload
	mov	word ptr es:[di].EVT_TIMER.count, ax
	mov	ax, word ptr es:[di].EVT_TIMER.counter_reload+2
	mov	word ptr es:[di].EVT_TIMER.count+2, ax

; *
; * Get the next timer structure and loop
; *
timer100_isr:
	les	di, es:[di].EVT_TIMER.next
	jmp	timer10_isr

timer_isr endp


; *************************************************************************************************
; *
; *	TIME_HDR_T FAR *TMR_CREATE( unsigned int TIMER_SIZE, char DYNAMIC )
; *	Allocates and sets up a timer
; *
; *	Given:
; *		TIMER_SIZE = number of bytes beyond the timer header required
; *		DYNAMIC = 0 if a static timer, !0 if dynamic
; *
; *	Returns:
; *		Pointer to allocated block or NULL if no memory is available
; *
; *************************************************************************************************
tmr_create proc uses di, timer_size:word, dynamic:byte
local	rv:word

if @Datasize
	push	ds
	mov	ds, Our_Data
endif
	mov	ax, timer_size		; AX = number of bytes to add on
	xor	dx, dx
	mov	rv, dx			; Assume NULL pointer return value

	add	ax, (size EVT_TIMER) + 15
	jc	short done_create

	mov	cx, 16
	div	cx			; AX = number of paragraphs to allocate
	mov	bx, ax			; BX = number of paragraphs to allocate
	mul	cx
	push	ax

	mov	ah, 48h			; AH = DOS Allocate Memory command
	int	21h
	pop	cx			; CX = size of the buffer
	jc	short done_create	; Quit if not enough memory

; *
; * Setup the timer structure
; *
	mov	es, ax			; ES:DI -> timer structure
	mov	rv, ax
	xor	di, di
	mov	ax, di			; AX = 0
	shr	cx, 1			; Make it words
	cld
	rep	stosw

	cmp	dynamic, 0		; See if we need to set the dynamic bit
	jz	done_create

	mov	es:EVT_TIMER.flags, TMR_DYNAMIC

done_create:
if @Datasize
	pop	ds
endif
	mov	dx, rv
	xor	ax, ax
	ret

tmr_create endp


; *************************************************************************************************
; *
; *	TIME_HDR_T FAR *TMR_CREATE_EXTENDED( ul COUNT, void FAR (*FUNCTION)(), uc FAR *BUFFER, us VALUE, us FLAGS )
; *	Allocates, initializes and starts an extended timer
; *
; *	Given:
; *		COUNT = number of system ticks to wait
; *		FUNCTION -> function call when triggered
; *		BUFFER -> byte to set to 1 when triggered, will be cleared on startup
; *		VALUE = value to place in the timer structure
; *		FLAGS = value to set the FLAGS field
; *
; *	Returns:
; *		Pointer to allocated block or NULL if no memory is available
; *
; *************************************************************************************************
tmr_create_extended proc, count:dword, function:dword, buffer:dword, value:word, flags:word

; *
; * Allocate the timer
; *
	mov	ax, flags
	and	ax, TMR_DYNAMIC		; Mask off all but the dynamic flag
	push	ax
	push	word ptr 0
	call	tmr_create
	add	sp, 4

	mov	es, dx
	mov	bx, ax
	or	ax, dx			; See if there was a NULL return
	jz	short done

	mov	ax, word ptr function
	mov	dx, word ptr function+2
	mov	word ptr es:[bx].EVT_TIMER.function, ax
	mov	word ptr es:[bx].EVT_TIMER.function+2, dx
	mov	ax, word ptr buffer
	mov	dx, word ptr buffer+2
	mov	word ptr es:[bx].EVT_TIMER.vector, ax
	mov	word ptr es:[bx].EVT_TIMER.vector+2, dx
	mov	ax, word ptr value
	mov	word ptr es:[bx].EVT_TIMER.value, ax
	mov	ax, flags
	and	ax, TMR_RETRIGGER+TMR_DYNAMIC		; Mask off all but the dynamic flag
	mov	word ptr es:[bx].EVT_TIMER.flags, ax

	push	word ptr count+2
	push	word ptr count
	push	es
	push	bx
	call	tmr_start
	pop	ax			; DX:AX -> return pointer
	pop	dx
	add	sp, 4

done:	ret

tmr_create_extended endp


; *************************************************************************************************
; *
; *	int TMR_START( TIME_HDR_T FAR *TP, long COUNT )
; *	Adds a timer structure to the timer queue
; *
; *	Given:
; *		TP -> timer structure to add
; *		COUNT = number of system ticks to wait
; *
; *	Returns:
; *		0 if the structure was added
; *
; *************************************************************************************************
tmr_start proc, tp:dword, count:dword

if @Datasize
	push	ds
	mov	ds, Our_Data
endif
	les	bx, tp			; ES:BX -> new timer structure

if NULL_CHECK
	mov	ax, es			; See if we were passed a NULL pointer
	or	ax, bx
	jz	short done
endif

; *
; * Search for the timer structure
; *
	mov	dx, es			; DX:CX = our timer pointer
	mov	cx, bx
	mov	es, Our_Data
	mov	bx, offset First_Timer_Event

	cli
; *
; * Look for the timer in the middle of the list
; *
tmr20_start:
	cmp	word ptr es:[bx].EVT_TIMER.next, cx	; See if it's pointing to us
	jne	short tmr30_start
	cmp	word ptr es:[bx].EVT_TIMER.next+2, dx
	jne	short tmr30_start

; *
; * Update our counter
; *
	les	bx, tp					; ES:BX -> timer structure
	mov	ax, word ptr count
	mov	word ptr es:[bx].EVT_TIMER.count, ax
	mov	ax, word ptr count+2
	mov	word ptr es:[bx].EVT_TIMER.count+2, ax
	jmp	short done

tmr30_start:
	les	bx, es:[bx].EVT_TIMER.next		; Assumes 'next' field is the first entry
	mov	ax, es					; See if we reached the end
	or	ax, bx
	jnz	short tmr20_start

; *
; * Setup the timer structure
; *
	les	bx, tp					; ES:BX -> timer structure
	mov	ax, word ptr count			; Initialize the counters
	mov	cx, word ptr count+2
	mov	word ptr es:[bx].EVT_TIMER.count, ax
	mov	word ptr es:[bx].EVT_TIMER.count+2, cx
	test	es:[bx].EVT_TIMER.flags, TMR_RETRIGGER
	jz	short tmr40_start
	mov	word ptr es:[bx].EVT_TIMER.counter_reload, ax
	mov	word ptr es:[bx].EVT_TIMER.counter_reload+2, cx

tmr40_start:
	or	es:[bx].EVT_TIMER.flags, TMR_ACTIVE
	and	es:[bx].EVT_TIMER.flags, not TMR_DISABLED

; *
; * Insert it at the head of the timer list
; *
	mov	ax, word ptr First_Timer_Event		; -> first timer event
	mov	word ptr es:[bx].EVT_TIMER.next, ax
	mov	ax, word ptr First_Timer_Event+2
	mov	word ptr es:[bx].EVT_TIMER.next+2, ax

	mov	word ptr First_Timer_Event, bx		; Make us the first timer structure
	mov	word ptr First_Timer_Event+2, es

	inc	Total_Timer_Events			; Number of timer events

done:
	sti
if @Datasize
	pop	ds
endif
	xor	ax, ax

	ret

tmr_start endp


; *************************************************************************************************
; *
; *	int TMR_STOP( TIME_HDR_T FAR *TP )
; *	Stops the timer from further processing
; *
; *	Given:
; *		TP -> timer structure to stop
; *
; *	Returns:
; *		0 if the structure was stopped
; *
; *************************************************************************************************
tmr_stop proc, tp:dword

if @Datasize
	push	ds
	mov	ds, Our_Data
endif
	les	bx, tp			; ES:BX -> new timer structure
	mov	ax, es			; See if we were passed a NULL pointer
	or	ax, bx
	jnz	short @F
	jmp	done

@@:	and	es:[bx].EVT_TIMER.flags, not TMR_ACTIVE
	or	es:[bx].EVT_TIMER.flags, TMR_DISABLED+TMR_SIGNAL_STOP

; *
; * Search for the timer structure
; *
	mov	dx, es			; DX:CX = our timer pointer
	mov	cx, bx
	cmp	word ptr First_Timer_Event, cx		; See if we are the first
	jnz	short tmr10_stop
	cmp	word ptr First_Timer_Event+2, dx
	jnz	short tmr10_stop			; Branch if not

; *
; * Remove us from the beginning
; *
	cli
	mov	ax, word ptr es:[bx].EVT_TIMER.next
	mov	word ptr First_Timer_Event, ax
	mov	ax, word ptr es:[bx].EVT_TIMER.next+2
	mov	word ptr First_Timer_Event+2, ax
	sti
	jmp	short tmr40_stop

; *
; * Remove us from the middle
; *
tmr10_stop:
	les	bx, First_Timer_Event			; ES:BX -> first event
tmr20_stop:
	mov	ax, es					; See if we reached the end
	or	ax, bx
	jz	short done

	cmp	word ptr es:[bx].EVT_TIMER.next, cx	; See if it's pointing to us
	jne	short tmr30_stop
	cmp	word ptr es:[bx].EVT_TIMER.next+2, dx
	jne	short tmr30_stop

; *
; * Here if we are found
; *
	mov	ax, es					; AX:CX -> previous structure, ES:BX -> our structure
	xchg	cx, bx
	mov	dx, word ptr es:[bx].EVT_TIMER.next+2	; DX:BX -> next structure
	mov	bx, word ptr es:[bx].EVT_TIMER.next
	mov	es, ax					; ES:BX -> previous structure
	xchg	bx, cx					; DX:CX -> next structure
	cli
	mov	word ptr es:[bx].EVT_TIMER.next+2, dx	; Fill in the new next pointer
	mov	word ptr es:[bx].EVT_TIMER.next, cx
	sti
	jmp	short tmr40_stop

tmr30_stop:
	les	bx, es:[bx].EVT_TIMER.next
	jmp	tmr20_stop

tmr40_stop:
	dec	Total_Timer_Events			; One less timer event

; *
; * See if we need to send a message
; *
	les	bx, tp					; ES:BX -> thing
	test	es:[bx].EVT_TIMER.flags, TMR_SIGNAL_STOP
	jz	short done

	push	di
	push	si

	push	es			; Save ES:BX
	push	bx
	call	get_event_envelope	; Get the next available event entry
	pop	si			; DS:SI -> timer structure
	pop	ds
	jc	short @F

	mov	es:[di].TMR_EVT.etype, 3
	mov	ax, [si].EVT_TIMER.flags
	mov	es:[di].TMR_EVT.emask, al

	mov	ax, [si].EVT_TIMER.value
	mov	es:[di].TMR_EVT.value, ax
	mov	ax, word ptr [si].EVT_TIMER.vector
	mov	word ptr es:[di].TMR_EVT.vector, ax
	mov	ax, word ptr [si].EVT_TIMER.vector+2
	mov	word ptr es:[di].TMR_EVT.vector+2, ax

	mov	word ptr es:[di].TMR_EVT.timer, si
	mov	word ptr es:[di].TMR_EVT.timer+2, es

@@:	pop	si
	pop	di
	mov	ds, Our_Data

done:
if @Datasize
	pop	ds
endif
	xor	ax, ax

	ret

tmr_stop endp


; *************************************************************************************************
; *
; *	int TMR_KILL( TIME_HDR_T FAR *TP )
; *	Stops the timer from further processing and deallocates the structure if it's dynamic
; *
; *	Given:
; *		TP -> timer structure to stop
; *
; *	Returns:
; *		0 if the structure was stopped
; *
; *************************************************************************************************
tmr_kill proc, tp:dword

	mov	ax, word ptr tp		; See if NULL pointer
	or	ax, word ptr tp+2
	jz	done10

	push	word ptr tp+2		; Stop the timer
	push	word ptr tp
	call	tmr_stop
	add	sp, 4

; *
; * See if it's dynamic, to see if we free it
; *
	les	bx, tp			; ES:BX -> timer event
	test	es:[bx].EVT_TIMER.flags, TMR_DYNAMIC
	jz	short done		; No need to go further

	mov	ah, 49h			; AH = DOS Release Allocated Memory command
	int	21h

done:
	xor	ax, ax
done10:
	ret

tmr_kill endp

	end

