/**************************************************************************************************
*
*		Title:	EVENT.H
*		Copyright (c) January 1993, Ryu Consulting
*		Written by Rahner James
*
*		This is the header file that contains all the information and structures for event
*		processing.
*
**************************************************************************************************/

#define	_MOUSE_H_

#ifndef FAR
	#ifdef __386__
		#define	HUGE
		#define	FAR
	#else
		#define	HUGE	huge
		#define	FAR		far
	#endif
#endif
#ifdef __WATCOMC__
	#ifdef __SMALL__
		#pragma library ("wevents.lib");
	#else
		#pragma library ("weventl.lib");
	#endif
#endif
#pragma	pack(1)
#ifndef NULL
	#define	NULL	((void *)0)
#endif
#ifndef	FNULL
	#define	FNULL	((void FAR *)0)
#endif


/**************************************************************************************************
*
*										Definitions
*
**************************************************************************************************/

#define	MAX_CONNECTIONS		20			// Maximum number of connection supported

#define	IPX_OPEN			0			// Open IPX connection
#define	SPX_OPEN			1			// Open SPX connection
#define	SPX_LISTEN			0			// Listen for SPX connection
#define	SPX_ESTABLISH		2			// Establish SPX connection

#define	EVT_NOEVENTS		0			// Event has not happened
#define	EVT_MOUSE			1			// Event is a mouse event
#define	EVT_KEYBOARD		2			// Event triggered by the keyboard
#define	EVT_TIMER			3			// Event triggered by the timer
#define	EVT_SERIAL			4			// Event triggered by the COM port
#define	EVT_NETWORK			5			// Event triggered by a network event
#define	EVT_CALLBACK		6			// Event is a callback event


/**************************************************************************************************
*
*									Mouse Event Definitions
*
**************************************************************************************************/

#define	MOUSE_LEFT			1			// Mask for the left button
#define	MOUSE_RIGHT			2			// Mask for the right button
#define	MOUSE_MIDDLE		4			// Middle button on a Logitech

#define	EVT_MOUSEMOVE		1
#define	EVT_LPRESS			2
#define	EVT_LRELEASE		4
#define	EVT_RPRESS			8
#define	EVT_RRELEASE		0x10
#define	EVT_MPRESS			0x20
#define	EVT_MRELEASE		0x40


/**************************************************************************************************
*
*									Network Structures
*
**************************************************************************************************/

typedef struct NODE_S
{
	unsigned long	network;
	unsigned char	node[6];
	unsigned short	socket;
} NODE_T;


typedef struct IPX_STATUS_S
{
	volatile unsigned char	inUseFlag;		// State of this ECB
	volatile unsigned char	completionCode;	// Completion code this ECB, only valid when inUseFLag == 0
} IPX_STATUS_T;


typedef struct CONN_S
{
	unsigned short	next;				// offset of next connection
	unsigned short	index;				// Index into the Connection_Table for this entry
	volatile union
	{
		unsigned int	flags;			// B0:0=IPX/1=SPX, B1:0=listen SPX/1=establish SPX
										// B2:1=connection established, B3:1=connection is bad or was stopped
		struct
		{
			unsigned int	spx : 1;		// SPX connection
			unsigned int	establish : 1;	// Set if this side was the establishing connection
			unsigned int	connected : 1;	// Set if this connection has been established, valid for SPX only
			unsigned int	stopped : 1;	// Connection is bad or has been canceled
			unsigned int	ncp_connection : 1;
			unsigned int	server : 1;
			unsigned int	reserved : 2;
			unsigned int	packet_type : 7;// Used to fill in the transmissions
			unsigned int	active : 1;		// Set if this connection entry is open
		};
	};

	unsigned long	network;
	unsigned char	node[6];
	unsigned short	tsocket;
	unsigned short	lsocket;
	unsigned char	immediate[6];
	unsigned int	connection;			// Connection number if this is an SPX connection
	unsigned int	transport_time;		// Number of system ticks for a round trip
	unsigned int	listener_memory;	// Segment used to store the listeners
	unsigned int	listener_count;		// Number of listeners
} CONN_T;

typedef struct IPX_S
{
	unsigned short		checkSum;			// Checksum number, unused by this driver, stored high-low
	unsigned short		length;				// Length of the complete IPX packet, set by the IPX driver, stored high-low
	unsigned char		transportControl;	// Number of bridges that this packet has passed, set to 0 by the IPX driver
	unsigned char		packetType;			// Packet type number, SPX driver sets this to 5, application sets IPX to 0 or 4
	NODE_T				destination;		// Internetwork address of the destination for the packet, filled by the application
	NODE_T				source;				// Internetwork address of the destination for the packet, filled by the driver
} IPX_T;

typedef struct SPX_S
{
	unsigned short		checkSum;			// Checksum number, unused by this driver, stored high-low
	unsigned short		length;				// Length of the complete IPX packet, set by the IPX driver, stored high-low
	unsigned char		transportControl;	// Number of bridges that this packet has passed, set to 0 by the IPX driver
	unsigned char		packetType;			// Packet type number, SPX driver sets this to 5, application sets IPX to 0 or 4
	NODE_T				destination;		// Internetwork address of the destination for the packet, filled by the application
	NODE_T				source;				// Internetwork address of the destination for the packet, filled by the driver

	unsigned char		connectionControl;	// Flags used by SPX, see SYSTEM_PACKET, ACKNOWLEDGEMENT_REQUIRED, END_OF_MESSAGE
	unsigned char		dataStreamType;		// Can be used by the IPX application, SPX uses 0xFE and 0xFF
	unsigned short		sourceConnectionID;	// Connection ID of the source of the packet, stored high-low
	unsigned short		destConnectionID;	// Connection ID of the destination of the packet set upon xmit by app., stored high-low
	unsigned short		sequenceNumber;		// Sequence number of the packet, used by SPX, stored high-low
	unsigned short		acknowledgeNumber;	// Sequence number of the next packet that SPX expects to receive, stored high-low
	unsigned short		allocationNumber;	// Number of listening packets available at the receiver + acknowledgeNumber, stored high-low
} SPX_T;


typedef struct ECB_HEADER_S
{
	struct ECB_S far	*linkAddress;	// Used as a linked list pointer while ECB is in the IPX/SPX domain
	void 		(far	*ESRAddress)();	// -> function to call when this ECB has completed, AL=identifier,ESI->ECB
	volatile unsigned char	inUseFlag;		// State of this ECB
	unsigned char		completionCode;	// Completion code this ECB, only valid when inUseFLag == 0
	unsigned short		socketNumber;	// Socket number of the source or destination

	unsigned char		reserved[16];	// Used by the IPX driver

	unsigned char		immediateAddress[6];	// Physical destination of this ECB
	unsigned short		fragmentCount;			// Number of fragments to follow
	struct
	{
		void FAR *		ptr;
		unsigned short	size;
	} frag[1];
} ECB_HEADER_T;


typedef struct ECB_S
{
	struct ECB_S far	*linkAddress;	// Used as a linked list pointer while ECB is in the IPX/SPX domain
	void 		(far	*ESRAddress)();	// -> function to call when this ECB has completed, AL=identifier,ESI->ECB
	volatile unsigned char		inUseFlag;		// State of this ECB
	unsigned char		completionCode;	// Completion code this ECB, only valid when inUseFLag == 0
	unsigned short		socketNumber;	// Socket number of the source or destination

	unsigned char		reserved[16];	// Used by the IPX driver

	unsigned char		immediateAddress[6];	// Physical destination of this ECB
	unsigned short		fragmentCount;			// Number of fragments to follow

	SPX_T		far		*buffer1;			// -> SPX header
	unsigned short		buffer1_size;		// Size of SPX header, 30 for IPX, 42 for SPX
	void		far		*buffer2;			// -> buffer
	unsigned short		buffer2_size;		// Size of buffer

	unsigned short far	*completion_ptr;	// -> word to fill with the inUseFlag and completionCode

	unsigned short		checkSum;			// Checksum number, unused by this driver, stored high-low
	unsigned short		length;				// Length of the complete IPX packet, set by the IPX driver, stored high-low
	unsigned char		transportControl;	// Number of bridges that this packet has passed, set to 0 by the IPX driver
	unsigned char		packetType;			// Packet type number, SPX driver sets this to 5, application sets IPX to 0 or 4
	NODE_T				destination;		// Internetwork address of the destination for the packet, filled by the application
	NODE_T				source;				// Internetwork address of the destination for the packet, filled by the driver

	unsigned char		connectionControl;	// Flags used by SPX, see SYSTEM_PACKET, ACKNOWLEDGEMENT_REQUIRED, END_OF_MESSAGE
	unsigned char		dataStreamType;		// Can be used by the IPX application, SPX uses 0xFE and 0xFF
	unsigned short		sourceConnectionID;	// Connection ID of the source of the packet, stored high-low
	unsigned short		destConnectionID;	// Connection ID of the destination of the packet set upon xmit by app., stored high-low
	unsigned short		sequenceNumber;		// Sequence number of the packet, used by SPX, stored high-low
	unsigned short		acknowledgeNumber;	// Sequence number of the next packet that SPX expects to receive, stored high-low
	unsigned short		allocationNumber;	// Number of listening packets available at the receiver + acknowledgeNumber, stored high-low
} ECB_T;


typedef struct SAP_S
{
	unsigned short	type;
	unsigned char	name[48];
	unsigned long	network;
	unsigned char	node[6];
	unsigned short	socket;
	unsigned short	intermediate_networks;
} SAP_T;


/**************************************************************************************************
*
*										Structures
*
**************************************************************************************************/

typedef struct TIME_HDR_S
{
	void FAR *		next;
	unsigned long	count;
	struct
	{
		unsigned short	retrigger 		: 1;	// Counter will be retriggered after the event processor is signalled
		unsigned short	disable_signal	: 1;	// Timer was disabled by disable call, set on final call to event processor
		unsigned short	dynamic			: 1;	// Timer structure was allocated and should be freed once processed
		unsigned short	signal_stop		: 1;	// Unused right now
		unsigned short	reserved		: 11;	// Unused right now
		unsigned short	active			: 1;	// Timer is not updated while this is set, so counters can be changed w/o CLI
	};
	unsigned long		counter_reload;			// Value to reload the counter with after it is triggered

	void		(FAR	*function)();			// -> function associated with the timer
	unsigned char FAR	*vector;				// -> data buffer associated with the timer
	unsigned short		value;					// Value associated with the timer event

} TIME_HDR_T;


typedef struct TIME_RETRIGGER_S
{
	void FAR *		next;
	unsigned long	count;
	struct
	{
		unsigned short	retrigger 		: 1;	// Counter will be retriggered after the event processor is signalled
		unsigned short	disable_signal	: 1;	// Timer was disabled by disable call, set on final call to event processor
		unsigned short	dynamic			: 1;	// Timer structure was allocated and should be freed once processed
		unsigned short	signal_stop		: 1;	// Event manager signalled if this timer is stopped
		unsigned short	reserved		: 11;	// Unused right now
		unsigned short	active			: 1;	// Timer is not updated while this is set, so counters can be changed w/o CLI
	};
	unsigned long	counter_reload;				// Value to reload the counter with after it is triggered
} TIME_RETRIGGER_T;


typedef struct MOUSE_STATUS_S
{
	int		status;						// Current button status
	int		press_count;				// Number of presses
	int		press_x, press_y;			// X,Y position of the last press
	int		release_count;				// Number of releases
	int		release_x, release_y;		// X,Y position of the last release
} MOUSE_STATUS_T;


typedef struct MOUSE_EVENT_S
{
	unsigned char	type;				// Type of event, MOUSE == type 1, KEYBOARD == type 2
	union
	{
		unsigned char	mask;			// Event mask
		struct
		{
			unsigned char mmove : 1;
			unsigned char mlpress : 1;
			unsigned char mlrelease : 1;
			unsigned char mrpress : 1;
			unsigned char mrrelease : 1;
			unsigned char mmpress : 1;
			unsigned char mmrelease : 1;
		};
	};
	unsigned long	time;				// System tick time of the event

	union
	{
		unsigned short	state;			// Keyboard flags, mouse button status
		struct
		{
			unsigned short rshift : 1;
			unsigned short lshift : 1;
			unsigned short ctl : 1;
			unsigned short alt : 1;
			unsigned short scroll_lock : 1;
			unsigned short num_lock : 1;
			unsigned short caps_lock : 1;
			unsigned short insert : 1;
			unsigned short left : 1;
			unsigned short right : 1;
			unsigned short middle : 1;
		};
	};

	unsigned char	ascii;				// ASCII character from keyboard
	unsigned char	scan;				// Scancode from the keyboard
	int 		x;
	int			y;
} MOUSE_EVENT_T;


typedef struct TMR_EVENT_S
{
	unsigned char	type;				// Type of event, TIMER = type 3
	union
	{
		unsigned char	mask;			// Event mask
		struct
		{
			unsigned char timeout : 1;
			unsigned char retriggerable : 1;
			unsigned char canceled : 1;
			unsigned char changed : 1;
		};
	};
	unsigned long	time;				// System tick time of the event

	TIME_HDR_T FAR	*timer;				// -> function associated with the timer event
	void FAR 		*vector;			// -> data area
	unsigned short	value;				// Value associated with the timer event

} TMR_EVENT_T;


typedef struct SERIAL_EVENT_S
{
	unsigned char	type;				// Type of event, SERIAL = type 4
	unsigned char	reason;				// 0=serial input has reached threshold, 1=modem change, 2=error
	unsigned long	time;				// System tick time of the event

	union
	{
		unsigned short	status;			// Status at the time of the event
		struct
		{
			unsigned short	dCTS : 1;		// Change in CTS
			unsigned short	dDSR : 1;		// Change in DSR
			unsigned short	dRI : 1;		// Change in RI
			unsigned short	dDCD : 1;		// Change in DCD
			unsigned short	CTS : 1;		// Change in CTS
			unsigned short	DSR : 1;		// Data set ready asserted
			unsigned short	RI : 1;			// Ring indicator asserted
			unsigned short	DCD : 1;		// Data Carrier Detect asserted

			unsigned short	data_ready : 1;			// Data is in the input buffer waiting to be read
			unsigned short	overwrite : 1;			// A received byte was overwritten
			unsigned short	parity : 1;				// Parity error occurred
			unsigned short	frame_error : 1;		// A framing synch error occurred
			unsigned short	break_signal : 1;		// A break signal was received
			unsigned short	xmit_empty : 1;			// Transmit register is empty
			unsigned short	xmit_shirt_empty : 1;	// Transmitter shift register is empty
			unsigned short	general_error : 1;		// General error occurred
		};
	};

	unsigned char	com_number;			// COM number that generated the event

} SERIAL_EVENT_T;


typedef struct NETWORK_EVENT_S
{
	unsigned char	type;			// Type of event, NETWORK = type 5
	union
	{
		unsigned char	reason;		// Reason for the event
		struct
		{
			unsigned char received_packet : 1;
			unsigned char transmitted_packet : 1;
			unsigned char aes_packet : 1;			// Triggered by an AES event
			unsigned char new_connection : 1;		// New connection, connection field filled, from SPX connection listen

			unsigned char disconnect : 1;			// Remote disconnected, connection number filled
			unsigned char establish_connection : 1;	// SPX connection established
			unsigned char reserved : 1;
			unsigned char error : 1;				// Some sort of error here
		};
	};
	unsigned long	time;			// System tick time of the event

	signed short	connection;		// Connection number associated with the event
	ECB_T FAR *		ecb;			// -> ECB that caused the event
	unsigned char	completionCode;	// Completion code from the triggering ECB
} NETWORK_EVENT_T;


typedef struct CALLBACK_EVENT_S
{
	unsigned char	type;				// Type of event, CALLBACK = type 6
	union
	{
		unsigned char	mask;			// Event mask
		struct
		{
			unsigned char nfunction : 1;	// Set if the function called is a near function
			unsigned char vfunction : 1;	// Set if the function called does not require parameters
		};
	};
	unsigned long	time;				// System tick time of the event

	union
	{
		void	(FAR *function)();		// -> function that will be called
		void	(*near_function)();		// Near function called if near_function is set
	};
	void	FAR *vector;				// -> data area
	unsigned short	value;				// Value associated with the timer event

} CALLBACK_EVENT_T;


typedef struct EVENT_S
{
	union
	{
		struct
		{
			unsigned char	type;
			unsigned char	mask;
			unsigned long	time;
		};

		MOUSE_EVENT_T		mouse;
		MOUSE_EVENT_T		kbd;
		TMR_EVENT_T			timer;
		SERIAL_EVENT_T		serial;
		NETWORK_EVENT_T		network;
		CALLBACK_EVENT_T	callback;
	};
} EVENT_T;


/**************************************************************************************************
*
*										Data Prototypes
*
**************************************************************************************************/

extern	int	_Mouse_Button_Count;							// Number of mouse buttons, must be !0 if mouse functions will work
extern	int	_Mouse_Motion_X, _Mouse_Motion_Y;				// Mouse motion values retunred by function calls
extern	int	_Mouse_X, _Mouse_Y;								// Current location of the mouse, as of the last event
extern	char _Mouse_Button;									// Current state of the mouse buttons
extern	void (FAR *_Mouse_Motion_Vector)(void);				// -> function that will be called when the mouse is moved
extern	int (*_Serial_Input_Vector)(void);					// -> function that will be called when serial input is available

extern	int (FAR* _Event_Vector_Table[])(EVENT_T FAR *);	// Table of event function entries
extern	int (FAR **_Callback_Event_Vector)(EVENT_T FAR *);	// Used for a vectored callback value
extern	int _Callback_Table_Size;							// Number of entries in the callback table

extern	CONN_T			Connection_Table[];
extern	unsigned char	SPX_Retry_Count, Socket_Life, Watchdog_Flag;
extern	NODE_T			_Our_Address;
extern	unsigned char	_Our_Node[6];

int Cn;


/**************************************************************************************************
*
*									Function Prototypes
*
**************************************************************************************************/

extern	int cdecl				clear_events( void );
extern	void cdecl				disable_help( void );
extern	void cdecl				enable_help( void );
extern	int cdecl				get_event( EVENT_T far * );
extern	void cdecl				init_help( void (FAR *)(void), unsigned short );
extern	void cdecl				mouse_arrow( void );
extern	void cdecl				mouse_cross( void );
extern	void cdecl				mouse_finger( void );
extern	int cdecl				mouse_get_motion( void );
extern	void FAR * cdecl		mouse_get_pointer( int FAR *, int FAR * );
extern	int cdecl				mouse_get_status( MOUSE_STATUS_T FAR *, int );
extern	int cdecl				mouse_getxy( int *, int * );
extern	int cdecl				mouse_gotoxy( int, int );
extern	int cdecl				mouse_hide( void );
extern	void cdecl				mouse_hourglass( void );
extern	int cdecl				mouse_init( void );
extern	void cdecl				mouse_pointing_hand( void );
extern	int cdecl				mouse_reset( void );
extern	int cdecl				mouse_set_bounds( int, int, int, int );
extern	int cdecl				mouse_set_gcursor( unsigned short FAR *, int, int );
extern	int cdecl				mouse_set_ratio( int, int );
extern	int cdecl				mouse_set_speed( int );
extern	int cdecl				mouse_set_tcursor( int, int, int );
extern	int cdecl				mouse_set_vector( void (FAR *)(), int );
extern	int cdecl				mouse_show( void );
extern	void cdecl				mouse_stop_hand( void );
extern	void cdecl				mouse_up_arrow( void );
extern	int cdecl				mouse_update_region( int, int, int, int );
extern	int cdecl				mouse_wait( int, int );				// Mask, result
extern	int cdecl				process_events( void );
extern	int cdecl				put_callback_event( void (FAR *)(), void FAR *, unsigned short );
extern	int cdecl				put_event( EVENT_T far * );
extern	int cdecl				put_kbd_event( EVENT_T far * );
extern	int cdecl				put_mouse_event( EVENT_T far * );
extern	int cdecl				start_events( int );
extern	void cdecl				stop_events( void );
extern	int cdecl				wait_event( EVENT_T far * );
extern	unsigned short cdecl	set_event_mask( unsigned short );
extern	TIME_HDR_T FAR * cdecl	tmr_create( unsigned int, char );
extern	TIME_HDR_T FAR * cdecl	tmr_create_extended( unsigned long, void (FAR *)(), void FAR *, us, us );
extern	int cdecl				tmr_start( TIME_HDR_T FAR *, long );
extern	int cdecl				tmr_stop( TIME_HDR_T FAR * );
extern	int cdecl				tmr_kill( TIME_HDR_T FAR * );


/**************************************************************************************************
*
*									Network Prototypes
*
**************************************************************************************************/

extern	int	cdecl				ipx_init( int );								// Number of talkers
extern	int	cdecl				ipx_open_connection( NODE_T FAR *, int, unsigned int, unsigned int );
extern	int	cdecl				ipx_write_packet( int, void FAR *, int, IPX_STATUS_T FAR * );
extern	int	cdecl				ipx_listen_packet( ECB_T FAR * );
extern	int cdecl				ipx_close_socket( unsigned short );
extern	int cdecl				ipx_stop( void );
extern	CONN_T FAR * cdecl		ipx_get_connection( int );
extern	void cdecl				atoaddr( char FAR *, char FAR * );
extern	char FAR * cdecl		addrtoa( char FAR *, char FAR * );
extern	unsigned short cdecl	to_intel( unsigned short );
extern	unsigned long cdecl		to4_intel( unsigned long );

extern	int	cdecl				open_semaphore( char FAR *, char, unsigned long FAR * );
extern	int	cdecl				close_semaphore( unsigned long );
extern	int	cdecl				wait_semaphore( unsigned long, unsigned short );
extern	int	cdecl				signal_semaphore( unsigned long );
extern	int	cdecl				get_semaphore_count( unsigned long );
extern	int	cdecl				get_semaphore_value( unsigned long );

extern	ECB_HEADER_T FAR *	cdecl ipx_alloc_ecb( int );
extern	void FAR *			cdecl ipx_malloc( unsigned int );
extern	ECB_HEADER_T FAR *	cdecl ipx_alloc_ecbipx( int );
extern	ECB_HEADER_T FAR *	cdecl ipx_alloc_ecbspx( int );
extern	ECB_HEADER_T FAR *	cdecl ipx_alloc_ipx_buffer( int );
extern	ECB_HEADER_T FAR *	cdecl ipx_alloc_spx_buffer( int );
extern	void				cdecl ipx_free_ecb( ECB_HEADER_T FAR * );
extern	void				cdecl ipx_free( void FAR * );
extern	int					cdecl sap_attach_to_server( SAP_T FAR * );

extern	int					cdecl ipx_call( int, void FAR *, unsigned short, unsigned short );
extern	int					cdecl ipx_get_local( void FAR *, void FAR * );

extern	void 				cdecl __copy_address( void FAR *, void FAR * );		// Copies ten bytes
extern	void 				cdecl __copy_node( void FAR *, void FAR * );		// Copies six bytes
extern	int					cdecl __cmp_address( void FAR *, void FAR * );		// Compares ten bytes
extern	int					cdecl __cmp_node( void FAR *, void FAR * );			// Compares six bytes

extern	void				cdecl __set_broadcast( void FAR * );
extern	int					cdecl farcmpw( void FAR *, void FAR *, unsigned int );
extern	void FAR *			cdecl farmove( void FAR *, void FAR *, unsigned int );
extern	unsigned int		cdecl farnlen( void FAR *, unsigned int );
extern	void FAR *			cdecl farset( void FAR *, char, unsigned int );
extern	int					cdecl isbroadcast( char FAR * );



int sap_find_server( unsigned short, SAP_T FAR *, int );
int init_server( unsigned short, char FAR *, unsigned short, char );
int sap_advertise_server( unsigned short, char FAR *, unsigned short );
int sap_stop_server( char FAR * );


/**************************************************************************************************
*
*										Macros
*
**************************************************************************************************/

#define	tmr_disable(x)	x->active = 0
#define	tmr_enable(x)	x->active = 1
