; *************************************************************************************************
; *
; *	Title:	FARSET.ASM
; *	Copyright (c) January 1991, Ryu Consulting, 916/722-1939
; *	Written by Rahner James
; *
; *	This file contains the functions for far manipulations
; *
; *************************************************************************************************

_FARSET_ASM_	equ	1

ifdef	LARGEMODEL
	.model	large,c
else
	.model	small,c
endif


.code
; *************************************************************************************************
; *
; *	void FAR *FARSET( void FAR *BUFFER, char VALUE, unsigned int COUNT )
; *	Sets a buffer to bytes of VALUE
; *
; *	Given:
; *		BUFFER -> buffer to set
; *		VALUE = byte to set the buffer to
; *		COUNT = number of bytes to do
; *
; *	Returns:
; *		-> buffer
; *
; *	Note:
; *		This function assumes that the buffers in question will start on even boundaries
; *		because they all deal with allocated far memory buffers
; *		It also assumes that the buffer resides within one 64K segment
; *
; *************************************************************************************************
farset proc uses di, buffer:dword, value:byte, count:word

	cld
	les	di, buffer		; ES:DI -> buffer start
	mov	al, value		; AX = a word of bytes
	mov	ah, al
	mov	cx, count		; CX = number of bytes to set
	shr	cx, 1			; CX = number of words to set
	rep	stosw
	adc	cx, cx
	rep	stosb

	mov	ax, word ptr buffer	; DX:AX -> beginning of the buffer for the return
	mov	dx, es
	ret

farset endp


; *************************************************************************************************
; *
; *	void FAR *FARMOVE( void FAR *DEST, void FAR *SOURCE, unsigned int COUNT )
; *	Moves a number of bytes from one fram memory buffer to another
; *
; *	Given:
; *		DEST -> buffer to move the bytes to
; *		SOURCE -> buffer to copy the bytes from
; *		COUNT = number of bytes to do
; *
; *	Returns:
; *		-> destination buffer
; *
; *	Note:
; *		This function assumes that the buffers in question will start on even boundaries.
; *		It also assumes that both buffers reside within single 64K segments and
; *		do not overlap in any way
; *
; *************************************************************************************************
farmove proc uses di si ds, dest:dword, source:dword, count:word

	cld
	les	di, dest		; ES:DI -> destination buffer
	lds	si, source		; DS:SI -> source buffer
	mov	ax, di			; AX = offset of return value
	mov	cx, count		; CX = number of bytes to move
	shr	cx, 1			; CX = number of words to move
	rep	movsw
	adc	cx, cx
	rep	movsb

	mov	dx, es
	ret

farmove endp


; *************************************************************************************************
; *
; *	int FARCMPW( void FAR *BUFFER1, void FAR *BUFFER2, unsigned int COUNT )
; *	Compares one buffer to another, a word at a time
; *
; *	Given:
; *		BUFFER1 -> first buffer to compare
; *		BUFFER2 -> second buffer to compare
; *		COUNT = number of bytes to compare
; *
; *	Returns:
; *		0 if same
; *
; *	Note:
; *		This function assumes that the buffers in question will start on even boundaries.
; *		It also assumes that both buffers reside within single 64K segments and are
; *		comprised of an even number of bytes
; *
; *************************************************************************************************
farcmpw proc uses di si ds, buffer1:dword, buffer2:dword, count:word

	cld
	xor	ax, ax
	les	di, buffer1		; ES:DI -> first buffer
	lds	si, buffer2		; DS:SI -> second buffer
	mov	cx, count		; CX = number of bytes to move
	shr	cx, 1			; CX = number of words to move
	repe	cmpsw
	je	@F
	adc	ax, ax			; AX = 1 if buffer1 > buffer2, otherwise 0
	add	ax, ax			; AX = 2 if buffer1 > buffer2, otherwise 0
	dec	ax			; AX = 1 if buffer1 > buffer2, otherwise -1

@@:	ret

farcmpw endp


; *************************************************************************************************
; *
; *	int FARNLEN( void FAR *BUFFER, unsigned int COUNT )
; *	Returns the end of a string up to a particular length
; *
; *	Given:
; *		BUFFER -> string to measure
; *		COUNT = maximum number of bytes in length
; *
; *	Returns:
; *		Number of byte in the string
; *
; *	Note:
; *		This function assumes that the buffers in question will start on even boundaries.
; *		It also assumes that both buffers reside within single 64K segments and are
; *		comprised of an even number of bytes
; *
; *************************************************************************************************
farnlen proc uses di, buffer:dword, count:word

	cld
	xor	ax, ax
	les	di, buffer		; ES:DI -> string to measure
	mov	cx, count		; CX = number of bytes to check
	mov	bx, cx
	repne	scasb
	cmc
	sbb	bx, cx
	mov	ax, bx
	ret

farnlen endp


; *************************************************************************************************
; *
; *	void ATOADDR( char FAR *ADDRESS, char FAR *STRING )
; *	Converts an ASCII string into an IPX/SPX node address
; *
; *	Given:
; *		ADDRESS -> IPX/SPX 6-byte node address 
; *		STRING -> ASCII string to convert, assumed to be 6 bytes or end with a 0
; *
; *	Returns:
; *		ASCII string converted into a node address
; *
; *	Note:
; *		This function was written for those of you who have a need for speed.
; *		This does not range checking.  You must be sure of the values provided
; *		otherwise you will get bizarro binary
; *
; *************************************************************************************************
atoaddr proc uses di si ds, address:dword, string:dword

	cld
	les	di, address		; ES:DI -> IPX/SPX node address buffer
	lds	si, string		; DS:SI -> ASCII representation of an IPX/SPX node address
	mov	cx, 604h		; CH = loop counter, CL = shifter
atoaddr10:
	lodsw				; Get two digits at a time
	sub	al, '0'			; Convert to binary
	cmp	al, 10			; See if < 10
	jc	@F			; Skip if we are < 10
	and	al, 7			; Get rid of the upper
	add	al, 9			; Dwell on the lower, no matter if it is upper or lower case

@@:	sub	ah, '0'			; Convert to upper nibble
	cmp	ah, 10			; See if < 10
	jc	@F			; Skip if we are < 10
	and	ah, 7			; Get rid of the upper
	add	ah, 9			; Dwell on the lower, no matter if it is upper or lower case

@@:	shl	al, cl			; Move lower byte bits into upper nibble
	or	al, ah			; Bring on Mr. Low Nibble
	stosb				; Store them together, like peas with a STOSB
	dec	ch
	jnz	atoaddr10		; Loop if this is a National Socialist

	ret

atoaddr endp


; *************************************************************************************************
; *
; *	char FAR *ADDRTOA( char FAR *STRING, char FAR *ADDRESS )
; *	Converts an IPX/SPX node address into an ASCII string
; *
; *	Given:
; *		STRING -> ASCII string to convert, assumed to be 6 bytes or end with a 0
; *		ADDRESS -> IPX/SPX 6-byte node address 
; *
; *	Returns:
; *		Node address converted to ASCII string
; *
; *	Note:
; *		This function was written for those of you who have a need for speed.
; *		This does not range checking.  You must be sure of the values provided
; *		otherwise you will get bizarro binary
; *
; *	Uses:
; *		This function uses the following table:
ascii_table		db	'0123456789ABCDEF'
; *		for looking up the ASCII hexadecimal conversion letters
; *
; *************************************************************************************************
addrtoa proc uses di si ds, string:dword, address:dword

	cld
	les	di, string		; ES:DI -> ASCII representation of an IPX/SPX node address
	lds	si, address		; DS:SI -> IPX/SPX node address buffer
	mov	cx, 604h		; CH = loop counter, CL = shifter
	mov	bx, offset ascii_table	; BX -> lookup table
@@:	xor	ah, ah			; AH = 0
	lodsb				; Get two binary digits at a time
	ror	ax, cl			; Put LSNibble in AH, old MSNibble -> LSNibble
	xlat	cs:ascii_table		; Translate to ASCII in AL
	stosb
	mov	al, ah
	shr	al, cl
	xlat	cs:ascii_table		; Translate to ASCII in AL
	stosb
	dec	ch			; One less byte for mankind
	jnz	@B

	mov	byte ptr es:[di], 0	; Put in Arnold
	mov	dx, es
	mov	ax, word ptr string

	ret

addrtoa endp


; *************************************************************************************************
; *
; *	ul TO4_INTEL( ul NUMBER )
; *	Converts a 32-bit value from the dreaded Motorola format to Intel
; *
; *	Given:
; *		NUMBER = 4-byte number to convert
; *
; *	Returns:
; *		32-bit Intel repesentation of the sequence
; *
; *************************************************************************************************
to4_intel proc, number:dword

	mov	ax, word ptr number+2	; AX = LSWord, in the incorrect place
	mov	dx, word ptr number	; DX = MSWord
	xchg	dh, dl
	xchg	ah, al

	ret

to4_intel endp


; *************************************************************************************************
; *
; *	ui TO_INTEL( us MOTO_NUMBER )
; *	Converts a 16-bit value from the dreaded Motorola format to Intel
; *
; *	Given:
; *		MOTO_NUMBER = 2-byte number to convert
; *
; *	Returns:
; *		16-bit Intel repesentation of the sequence
; *
; *************************************************************************************************
to_intel proc, moto_number:word

	mov	ax, moto_number		; AX = number to convert
	xchg	ah, al
	ret

to_intel endp


; *************************************************************************************************
; *
; *	int ISBROADCAST( char FAR *ADDRESS )
; *	Determines the brodcast-ity of an address, eg is it all 0xFFs
; *
; *	Given:
; *		ADDRESS -> IPX/SPX 6-byte node address to check
; *
; *	Returns:
; *		0 if it is not, !0 if it is
; *
; *************************************************************************************************
isbroadcast proc, address:dword

	les	bx, address		; ES:BX -> node address to check
	mov	ax, es:[bx]		; AND all the words together
	and	ax, es:[bx+2]
	and	ax, es:[bx+4]
	add	ax, 1			; Sets CARRY only if AX == 0xFFFF
	sbb	ax, ax			; AX = 1 if CARRY set, otherwise it's 0
	ret

isbroadcast endp


; *************************************************************************************************
; *
; *	void __COPY_NODE( void FAR *DEST_NODE, void FAR *SRC_NODE )
; *	Copies the 6 bytes of node
; *
; *	Given:
; *		DEST_NODE -> destination node buffer
; *		SRC_NODE -> source node buffer
; *
; *	Returns:
; *		Node copied
; *
; *************************************************************************************************
__copy_node proc, dest_node:dword, src_node:dword

	les	bx, src_node		; ES:BX -> source node buffer
	mov	ax, es:[bx]
	mov	cx, es:[bx+2]
	mov	dx, es:[bx+4]
	les	bx, dest_node		; ES:BX -> destination node buffer
	mov	es:[bx], ax
	mov	es:[bx+2], cx
	mov	es:[bx+4], dx

	ret

__copy_node endp


; *************************************************************************************************
; *
; *	void __CMP_NODE( void FAR *DEST_NODE, void FAR *SRC_NODE )
; *	Compares the 6 bytes of two nodes
; *
; *	Given:
; *		DEST_NODE -> destination node buffer
; *		SRC_NODE -> source node buffer
; *
; *	Returns:
; *		0 if the nodes are equal
; *		!0 if they are different
; *
; *************************************************************************************************
__cmp_node proc, dest_node:dword, src_node:dword

	les	bx, src_node		; ES:BX -> source node buffer
	mov	ax, es:[bx]
	mov	cx, es:[bx+2]
	mov	dx, es:[bx+4]
	les	bx, dest_node		; ES:BX -> destination node buffer
	cmp	es:[bx+4], dx		; Statistically, this will be the greatest difference
	mov	dx, -1			; Assume bad comparison
	jne	short @F
	cmp	es:[bx], ax		; Next this with arcnet
	jne	short @F
	cmp	es:[bx+2], cx
	jne	short @F

	inc	dx			; DX = 0

@@:	mov	ax, dx			; AX = return value
	ret

__cmp_node endp


; *************************************************************************************************
; *
; *	void __CMP_ADDRESS( void FAR *DEST_NODE, void FAR *SRC_NODE )
; *	Compares the 10 bytes of two nodes and their networks
; *
; *	Given:
; *		DEST_NODE -> destination address buffer
; *		SRC_NODE -> source address buffer
; *
; *	Returns:
; *		0 if the addresses are equal
; *		!0 if they are different
; *
; *************************************************************************************************
__cmp_address proc uses ds si, dest_node:dword, src_node:dword

	les	bx, dest_node		; ES:BX -> destination address buffer
	lds	si, src_node		; DS:SI -> source address buffer
	or	ax, -1			; AX = assume bad return

	mov	cx, [si+8]		; Compare the points of greatest differentiation
	cmp	es:[bx+8], cx
	jne	short @F
	mov	cx, [si+4]
	cmp	es:[bx+4], cx
	jne	short @F
	mov	cx, [si+2]
	cmp	es:[bx+2], cx
	jne	short @F

	mov	cx, [si]
	cmp	es:[bx], cx
	jne	short @F
	mov	cx, [si+6]
	cmp	es:[bx+6], cx
	jne	short @F

	inc	ax			; AX = 0 from -1

@@:
	ret

__cmp_address endp


; *************************************************************************************************
; *
; *	void __COPY_ADDRESS( void FAR *DEST_NODE, void FAR *SRC_NODE )
; *	Copies the 10 bytes of internetwork address, not including the socket
; *
; *	Given:
; *		DEST_NODE -> destination node buffer
; *		SRC_NODE -> source node buffer
; *
; *	Returns:
; *		Node copied
; *
; *************************************************************************************************
__copy_address proc uses di si, dest_node:dword, src_node:dword

	les	bx, src_node		; ES:BX -> source node buffer
	mov	ax, es:[bx]
	mov	cx, es:[bx+2]
	mov	dx, es:[bx+4]
	mov	di, es:[bx+6]
	mov	si, es:[bx+8]
	les	bx, dest_node		; ES:BX -> destination node buffer
	mov	es:[bx], ax
	mov	es:[bx+2], cx
	mov	es:[bx+4], dx
	mov	es:[bx+6], di
	mov	es:[bx+8], si

	ret

__copy_address endp


; *************************************************************************************************
; *
; *	void __SET_BROADCAST( void FAR *DEST_NODE )
; *	Sets a node address to broadcast (all 0xFF's)
; *
; *	Given:
; *		DEST_NODE -> destination node buffer to set
; *
; *	Returns:
; *		Node set to -1's
; *
; *************************************************************************************************
__set_broadcast proc, dest_node:dword

	les	bx, dest_node		; ES:BX -> destination node buffer
	mov	word ptr es:[bx], -1
	mov	word ptr es:[bx+2], -1
	mov	word ptr es:[bx+4], -1

	ret

__set_broadcast endp

	end
