; *************************************************************************************************
; *
; *	Title:	IPXALLOC.ASM
; *	Copyright (c) November 1993, Ryu Consulting
; *	Written by Rahner James
; *
; *	Supports all memory allocation for ECB, IPX/SPX headers, and buffers
; *
; *************************************************************************************************

_IPXALLOC_ASM_	equ	1

ifdef LARGEMODEL
	.model	large,c
else
	.model	small,c
endif


MAX_IPX_PACKET	equ	546
MAX_SPX_PACKET	equ	534


.code
; *************************************************************************************************
; *
; *	GET_MEMORY
; *	Allocates memory from the DOS memory pool
; *
; *	Given:
; *		AX = number of bytes to allocate
; *
; *	Returns:
; *		ES:0 = segment of the buffer
; *		DI = 0
; *		CARRY set if memory could not be allocated and DX:AX
; *		Entire memory buffer cleared
; *		BX CX saved
; *
; *************************************************************************************************
get_memory proc near private uses bx cx

	mov	dx, ax			; DX = number of bytes to allocate
	add	ax, 15			; Round memory up
	mov	cl, 4
	shr	ax, cl
	mov	bx, ax			; BX = number of paragraphs
	mov	ah, 48h
	int	21h
	jc	short derr		; Quit if done

	mov	es, ax			; ES = memory segment
	xor	di, di
	mov	ax, di
	mov	cx, dx
	inc	cx
	shr	cx, 1
	cld
	rep	stosw
	xor	di, di			; DI = 0, and clears CARRY

done:	ret

derr:	xor	dx, dx
	mov	ax, dx
	jmp	done


get_memory endp


; *************************************************************************************************
; *
; *	void FAR *IPX_MALLOC( unsigned int BUFFER_SIZE )
; *	Allocates memory from the DOS memory pool
; *
; *	Given:
; *		BUFFER_SIZE = number of bytes to allocate
; *
; *	Returns:
; *		DX:AX -> memory block, or 0 if none available
; *
; *************************************************************************************************
ipx_malloc proc uses di, buffer_size:word

	mov	ax, buffer_size
	call	get_memory
	mov	ax, di
	mov	dx, es
	ret

ipx_malloc endp


; *************************************************************************************************
; *
; *	ECB_HEADER_T FAR *IPX_ALLOC_ECB( int FRAGMENTS )
; *	Allocates and initializes an ECB header with a number of fragments
; *
; *	Given:
; *		FRAGMENTS = number of fragments, if 0, allocates 1
; *
; *	Returns:
; *		-> ECB, or NULL if not enough memory
; *		ECB.fragment_count will be set
; *
; *************************************************************************************************
ipx_alloc_ecb proc uses di, fragments:word

; *
; * Allocate the space
; *
	mov	ax, 6
	mul	fragments
	add	ax, 36			; Add in the size of the ECB header + round off for a paragraph
	call	get_memory
	jc	done

	mov	ax, fragments		; Set the number of fragments
	mov	es:[di+34], ax

	mov	dx, es			; DX:AX -> buffer that was allocated
	xor	ax, ax

done:	ret

ipx_alloc_ecb endp


; *************************************************************************************************
; *
; *	ECB_HEADER_T FAR *IPX_ALLOC_ECBIPX( int FRAGMENTS )
; *	Allocates and initializes an ECB and IPX header with a number of fragments
; *
; *	Given:
; *		FRAGMENTS = number of fragments, if 0, allocates 1, must include the IPX header
; *
; *	Returns:
; *		-> ECB, or NULL if not enough memory
; *		ECB.fragment_count will be set
; *		First fragment buffer will point to the header
; *		Packet type set in the packet type field of the IPX header
; *
; *************************************************************************************************
ipx_alloc_ecbipx proc uses di, fragments:word

; *
; * Allocate the space
; *
	mov	ax, 6
	mul	fragments
	mov	bx, ax			; BX = size of the fragments
	add	ax, 30+36		; Add in the size of the ECB header + IPX header + round off for a paragraph
	call	get_memory
	jc	done

	add	bx, 36			; ES:BX -> IPX header
	mov	ax, fragments		; Set the number of fragments
	mov	es:[di+34], ax
	mov	es:[di+36], bx
	mov	es:[di+38], es
	mov	word ptr es:[di+40], 30
	mov	byte ptr es:[bx+5], 4	; Set the packet type

	mov	dx, es			; DX:AX -> buffer that was allocated
	xor	ax, ax

done:	ret

ipx_alloc_ecbipx endp


; *************************************************************************************************
; *
; *	ECB_HEADER_T FAR *IPX_ALLOC_ECBSPX( int FRAGMENTS )
; *	Allocates and initializes an ECB and SPX header with a number of fragments
; *
; *	Given:
; *		FRAGMENTS = number of fragments, if 0, allocates 1, must include the SPX header
; *
; *	Returns:
; *		-> ECB, or NULL if not enough memory
; *		ECB.fragment_count will be set
; *		First fragment buffer will point to the header
; *		Packet type set in the packet type field of the SPX header
; *
; *************************************************************************************************
ipx_alloc_ecbspx proc uses di, fragments:word

; *
; * Allocate the space
; *
	mov	ax, 6
	mul	fragments
	mov	bx, ax			; BX = size of the fragments
	add	ax, 42+36		; Add in the size of the ECB header + SPX header + round off for a paragraph
	call	get_memory
	jc	done

	add	bx, 36			; ES:BX -> SPX header
	mov	ax, fragments		; Set the number of fragments
	mov	es:[di+34], ax
	mov	es:[di+36], bx
	mov	es:[di+38], es
	mov	word ptr es:[di+40], 30
	mov	byte ptr es:[bx+5], 5	; Set the packet type

	mov	dx, es			; DX:AX -> buffer that was allocated
	xor	ax, ax

done:	ret

ipx_alloc_ecbspx endp


; *************************************************************************************************
; *
; *	ECB_HEADER_T FAR *IPX_ALLOC_IPX_BUFFER( int BUFFER_SIZE )
; *	Allocates and initializes an ECB, IPX header and a buffer with two fragment pointers
; *
; *	Given:
; *		BUFFER_SIZE = size of the buffer to allocate
; *
; *	Returns:
; *		-> ECB, or NULL if not enough memory
; *		ECB.fragment_count will be set
; *		First fragment buffer will point to the header
; *		Packet type set in the packet type field of the IPX header
; *
; *************************************************************************************************
ipx_alloc_ipx_buffer proc uses di, buffer_size:word

; *
; * Allocate the space
; *
	mov	ax, buffer_size
	or	ax, ax			; See if need the biggest
	jnz	@F
	mov	ax, MAX_IPX_PACKET
	mov	buffer_size, ax
@@:	add	ax, 30+36+12		; Add in the size of the ECB header + IPX header + round off for a paragraph
	call	get_memory
	jc	done

	mov	word ptr es:[di+34], 2	; Number of fragments
	lea	bx, [di+48]		; BX = offset of the IPX header
	mov	es:[di+36], bx
	mov	es:[di+38], es
	mov	word ptr es:[di+40], 30
	lea	ax, [di+48+30]		; AX = offset of the buffer
	mov	es:[di+42], ax
	mov	es:[di+44], es
	mov	ax, buffer_size
	mov	es:[di+46], ax

	mov	byte ptr es:[bx+5], 4	; Set the packet type

	mov	dx, es			; DX:AX -> buffer that was allocated
	xor	ax, ax

done:	ret

ipx_alloc_ipx_buffer endp


; *************************************************************************************************
; *
; *	ECB_HEADER_T FAR *IPX_ALLOC_SPX_BUFFER( int BUFFER_SIZE )
; *	Allocates and initializes an ECB, SPX header and a buffer with two fragment pointers
; *
; *	Given:
; *		BUFFER_SIZE = size of the buffer to allocate
; *
; *	Returns:
; *		-> ECB, or NULL if not enough memory
; *		ECB.fragment_count will be set
; *		First fragment buffer will point to the header
; *		Packet type set in the packet type field of the SPX header
; *
; *************************************************************************************************
ipx_alloc_spx_buffer proc uses di, buffer_size:word

; *
; * Allocate the space
; *
	mov	ax, buffer_size
	or	ax, ax			; See if need the biggest
	jnz	@F
	mov	ax, MAX_SPX_PACKET
	mov	buffer_size, ax
@@:	add	ax, 42+36+12		; Add in the size of the ECB header + SPX header + round off for a paragraph
	call	get_memory
	jc	done

	mov	word ptr es:[di+34], 2	; Number of fragments
	lea	bx, [di+48]		; BX = offset of the SPX header
	mov	es:[di+36], bx
	mov	es:[di+38], es
	mov	word ptr es:[di+40], 42
	lea	ax, [di+48+30]		; AX = offset of the buffer
	mov	es:[di+42], ax
	mov	es:[di+44], es
	mov	ax, buffer_size
	mov	es:[di+46], ax

	mov	byte ptr es:[bx+5], 5	; Set the packet type

	mov	dx, es			; DX:AX -> buffer that was allocated
	xor	ax, ax

done:	ret

ipx_alloc_spx_buffer endp


; *************************************************************************************************
; *
; *	void IPX_FREE_ECB( ECB_HEADER_T FAR *ECB )
; *	Frees an ECB or ECB/IPX/SPX header pair
; *
; *	Given:
; *		ECB -> ecb buffer to free
; *
; *	Returns:
; *		Buffer freed
; *
; *************************************************************************************************
ipx_free_ecb proc, ecb:dword

	mov	cx, word ptr ecb+2	; CX = ECB segment
	jcxz	done
	mov	es, cx
	mov	ah, 49h
	int	21h

done:	ret

ipx_free_ecb endp


; *************************************************************************************************
; *
; *	void IPX_FREE( void FAR *BUFFER )
; *	Frees an allocated memory block
; *
; *	Given:
; *		BUFFER -> buffer to free
; *
; *	Returns:
; *		Buffer freed
; *
; *************************************************************************************************
ipx_free proc, buffer:dword

	mov	es, word ptr buffer+2
	mov	ax, es
	or	ax, word ptr buffer
	jz	done

	mov	ah, 49h
	int	21h

done:	ret

ipx_free endp

	end
