; *************************************************************************************************
; *
; *	Title:	IPXCALL.ASM
; *	Copyright (c) November 1993, Ryu Consulting
; *	Written by Rahner James
; *
; *	Allows C level access to IPX/SPX calls
; *
; *************************************************************************************************

_IPXCALL_ASM_	equ	1

ifdef LARGEMODEL
	.model	large,c
else
	.model	small,c
endif


.data

	public	__Return_Regs

	even
IPX_Vector	dd	0		; -> IPX functions
__Return_Regs	label	word		; Return from previous call
AX_Reg		dw	0
BX_Reg		dw	0
CX_Reg		dw	0
DX_Reg		dw	0
DI_Reg		dw	0
SI_Reg		dw	0



.code
; *************************************************************************************************
; *
; *	int IPX_CALL( int CALL_NUMBER, void FAR *BUFFER, unsigned short REG_AX, unsigned short REG_DX )
; *	Generic IPX/SPX call function
; *
; *	Given:
; *		CALL_NUMBER = value to place in BX
; *		BUFFER = value to place in ES:SI
; *		REG_AX = value to place in AX
; *		REG_DX = value to place in DX
; *
; *	Returns:
; *		AX = return value from call, -1 == no IPX found
; *		Return_Regs filled in with rest
; *
; *************************************************************************************************
ipx_call proc uses ds di si, call_number:word, buffer:dword, reg_ax:word, reg_dx:word

	mov	ax, @Data
	mov	ds, ax

	cmp	word ptr IPX_Vector+2, 0	; See if we have gotten the IPX vecter yet
	jnz	short ipx10_call

	push	bp
	mov	ax, 7a00h
	int	2Fh
	pop	bp
	xor	ah, ah
	dec	ax				; AX = -1 if IPX not found
	js	short done			; Only if IPX not present

	mov	word ptr IPX_Vector, di
	mov	word ptr IPX_Vector+2, es

ipx10_call:
	mov	bx, call_number			; Load the appropriate registers
	les	si, buffer
	mov	ax, reg_ax
	mov	dx, reg_dx

	push	bp
	call	dword ptr IPX_Vector		; Call IPX driver
	mov	bp, @Data
	mov	ds, bp
	pop	bp

	mov	AX_Reg, ax			; Save the registers
	mov	BX_Reg, bx
	mov	CX_Reg, cx
	mov	DX_Reg, dx
	mov	DI_Reg, di
	mov	SI_Reg, si

	xor	ah, ah

done:	ret

ipx_call endp


; *************************************************************************************************
; *
; *	int IPX_GET_LOCAL( void FAR *LP, void FAR *NP )
; *	Gets the local target
; *
; *	Given:
; *		LP -> 6 byte buffer to fill
; *		NP -> full internetwork address to get the address of
; *
; *	Returns:
; *		AX = return value from call, -1 == no IPX found
; *
; *************************************************************************************************
ipx_get_local proc uses ds di si, lp:dword, np:dword
local	network[12]:byte
local	immediate[6]:byte

	mov	ax, @Data
	mov	ds, ax

	cmp	word ptr IPX_Vector+2, 0	; See if we have gotten the IPX vecter yet
	jnz	short get10_local

	push	bp
	mov	ax, 7a00h
	int	2Fh
	pop	bp
	xor	ah, ah
	dec	ax				; AX = -1 if IPX not found
	js	short done			; Only if IPX not present

	mov	word ptr IPX_Vector, di
	mov	word ptr IPX_Vector+2, es

; *
; * Move the internetwork address to the local buffer
; *
get10_local:
	lds	si, np				; DS:SI -> network address
	mov	ax, ds
	or	ax, si
	sub	ax, 1
	jc	short done			; Quit if it's a NULL pointer

	mov	ax, ss
	mov	es, ax
	lea	di, network
	mov	cx, 6
	cld
	rep	movsw

; *
; * Get the immediate address
; *
	mov	ax, @Data
	mov	ds, ax
	lea	si, network
	lea	di, immediate

	mov	bx, 2
	push	bp
	call	dword ptr IPX_Vector		; Call IPX driver
	mov	bp, @Data
	mov	ds, bp
	pop	bp

	and	ax, 0FFh
	jnz	done

	les	di, lp				; ES:DI -> destination address
	mov	bx, word ptr immediate
	mov	es:[di], bx
	mov	bx, word ptr immediate+2
	mov	es:[di+2], bx
	mov	bx, word ptr immediate+4
	mov	es:[di+4], bx

done:	ret

ipx_get_local endp

	end
