; *************************************************************************************************
; *
; *	Title:	LOADDRVR.ASM
; *	Copyright (c) March 1994, Ryu Consulting
; *	Written by Rahner James
; *
; *	This file contains the code to load all real mode drivers from disk files into low memory
; *
; *************************************************************************************************

	include	event.inc


.data
	public	_Real_Init_Int, _Event_Global


Filename		db	"*.DVR",0	; This will be used if get a NULL pointer
_Real_Init_Int		db	DEFAULT_INT_NUMBER,0

Previous_Int		dw	0,0		; Previous real-mode INT vector
_Event_Global		dd	0		; -> global variable structure for the event manager, IPX/SPX, COM and mouse
Event_Global_Selector	dw	0,0		; Selector used for the global event structure


.code
; *************************************************************************************************
; *
; *	int LOAD_REALMODE_DRIVERS( char *EAX )
; *	Loads the real mode drivers from the disk and places them in low memory
; *
; *	Given:
; *		EAX -> pathname to use for the find first
; *
; *	Returns:
; *		Number of drivers loaded
; *		ERR_NO_MEMORY (-86) if not enough low memory for the operation
; *		-100 if file is too big (>64K) or too small, <10 bytes
; *		< 0 if file error
; *
; *	Note:
; *		Each driver is loaded into its own low memory buffer.  The drivers must be
; *		in TINY memory model and must start with "RYU1", followed by a 32-bit real-mode
; *		far pointer variable that the loader will point to the next driver in the line.
; *		The drivers will be called once by the initialization routine (INT-18h).
; *		All other communication done with the drivers will occur through the event manager.
; *		The dword at offset 8 must be the offset that will be called to initialize the
; *		driver.  The driver must be ORG 0.
; *
; *************************************************************************************************
load_realmode_drivers_ proc uses ebx ecx edx edi esi
local	file_count:dword
local	dta:dword
local	handle:dword			; Handle for the open file
local	buffer:dword			; Low memory buffer of current driver
local	selector:dword			; Selector of the allocate memory buffer

	mov	file_count, 0		; Initialize our number of files found

	or	eax, eax		; Make sure we have something here
	jnz	short @F		; Skip if we do

	mov	eax, offset Filename	; -> default file type to search for
; *
; * Find the first file match
; *
@@:	mov	edx, eax		; EDX -> file to search for
	xor	ecx, ecx		; Any old file is OK
	mov	ah, 4eh			; AH = DOS Find First command
	int	21h			; Make the DOS call
	jc	done			; Quit if an error

; *
; * Get a pointer to the DTA
; *
	mov	ah, 2fh			; AH = DOS Get DTA Pointer command
	push	es			; Save ES
	int	21h
	push	ebx
	mov	bx, es
	mov	ax, 6			; Get the linear address for the selector
	int	31h

	and	edx, 0FFFFh		; EDX = linear address
	shl	ecx, 16
	or	edx, ecx
	pop	ebx
	add	edx, ebx		; Add in the offset
	mov	dta, edx		; Save it for later
	pop	es			; Restore ES

; *
; * Allocate low memory
; *
load10_realmode:
	mov	ebx, dta			; EBX -> DTA
	mov	ebx, [ebx].DTA_S.file_size	; EBX = size of the file in bytes
	cmp	ebx, 64*1024			; See if too big
	jnc	derr_filesize
	cmp	ebx, 10				; See if too small
	jc	derr_filesize

	mov	esi, ebx			; Save the file size
	add	ebx, 15
	shr	ebx, 4				; BX = number of paragraphs to allocate
	mov	eax, 100h			; AX = DPMI Allocate DOS Memory command
	int	31h
	jc	derr_allocate			; Quit if can't allocate the memory
	and	eax, 0FFFFh			; Clear MSW of EAX
	shl	eax, 4				; EAX = offset of the allocated buffer
	mov	buffer, eax
	mov	selector, edx

; *
; * Open the file
; *
	mov	eax, 3d40h			; AX = Open File, Read Only, Deny None
	mov	edx, dta			; EDX -> DTA
	lea	edx, [edx].DTA_S.file_name	; EDX -> file name
	int	21h
	jc	derr				; Quit with an error
	mov	handle, eax

; *
; * Read the file
; *
	mov	ebx, eax			; EBX = file handle
	mov	ah, 3fh				; AH = DOS Read File command
	mov	ecx, esi			; ECX = file size
	mov	edx, buffer			; EDX -> buffer to load the driver into
	int	21h
	jc	derr_close

; *
; * Close the file
; *
	mov	ah, 3eh
	mov	ebx, handle
	int	21h

; *
; * Validate the file
; *
	mov	ecx, buffer				; ECX -> driver just loaded
	cmp	[ecx].DRIVER_S.signature, "RYU1"	; Check the signature
	jne	skip_file				; Skip this file if not a good signature
	cmp	dword ptr [ecx].DRIVER_S.next_driver, 0	; Pointer must be NULL at first
	jnz	skip_file
	cmp	[ecx].DRIVER_S.code_offset, 11		; Code must start after this header, based 0
	jc	skip_file
	cmp	[ecx].DRIVER_S.code_offset, 65532	; Code jump must be less than 64K less some change
	jnc	skip_file

; *
; * Lock the memory region for the driver
; *
	mov	ebx, ecx			; BX:CX -> linear address of the area
	shr	ebx, 16
	mov	edi, dta			; SI:DI = size of driver, will be < 64K
	mov	edi, [edi].DTA_S.file_size
	xor	esi, esi
	mov	eax, 600h			; DPMI Lock Memory Region command
	int	31h

; *
; * See if we need to fill in the INT vector and allocate the global storage structure
; *
	cmp	dword ptr Previous_Int, 0 	; See if we have store the previous INT
	jnz	@F

	mov	ax, 200h			; Get the current value for INT
	mov	bl, _Real_Init_Int		; BL = int number to check
	int	31h
	mov	Previous_Int, cx		; Store the previous INT
	mov	Previous_Int+2, dx

	mov	ax, 100h			; AX = DPMI Allocate DOS Memory command
	mov	ebx, ((size ET_S)+15)/16
	int	31h
	jc	derr_allocate			; Quit if can't allocate the memory
	mov	Event_Global_Selector, dx	; Save the selector

	and	eax, 0FFFFh
	shl	eax, 4				; EAX -> event manager's global storage
	mov	_Event_Global, eax

	mov	edi, eax			; EDI -> structure to clear
	mov	ecx, (size ET_S)/4
	xor	eax, eax
	rep	stosd				; Clear the structure to 0

	mov	ecx, _Event_Global			; ECX -> etable
	mov	[ecx].ET_S.iret_instruction, 0CFh	; Store the IRET instruction
	mov	[ecx].ET_S.set_carry, 0F8h		; Store the STC instruction
	mov	[ecx].ET_S.far_return, 0CBh		; Store the RETF instruction
	mov	eax, ecx				; EAX -> etable
	shl	eax, 12					; EAX -> etable far ptr
	mov	ax, ET_S.set_carry
	mov	[ecx].ET_S.vector_get_packet, eax	; Set this to point to the STC/RETF sequence during startup

; *
; * Lock down the global storage area
; *
	mov	ebx, ecx			; BX:CX -> linear address of the area
	shr	ebx, 16
	mov	edi, size ET_S			; SI:DI = size of area, will be < 64K
	xor	esi, esi
	mov	ax, 600h			; DPMI Lock Memory Region command
	int	31h

; *
; * Set the INT to point to this structure's IRET
; *
	mov	ax, 201h			; DPMI Set Real-mode Interrupt command
	mov	bl, _Real_Init_Int		; BL = int number to check
	mov	ecx, _Event_Global
	lea	edx, [ecx].ET_S.iret_instruction
	shr	ecx, 4				; CX:DX -> IRET
	and	edx, 0Fh
	int	31h

; *
; * Set the previous driver to point to this one
; *
@@:	mov	ax, 200h			; Get the current value for INT
	mov	bl, _Real_Init_Int		; BL = int number to check
	int	31h				; CX:DX -> old real-mode interrupt (eg, previous driver)
	mov	eax, buffer			; EAX -> this driver
	mov	[eax].DRIVER_S.next_driver, dx	; Setup the next driver real-mode vector
	mov	[eax].DRIVER_S.next_driver+2, cx
	mov	edx, _Event_Global
	shl	edx, 12				; Move the linear address into a segment:offset position
	mov	dword ptr [eax].DRIVER_S.global_data_ptr, edx

	mov	edx, [eax].DRIVER_S.code_offset	; EDX = offset of code area
	mov	ecx, eax
	shr	ecx, 4				; ECX = driver segment, so that the driver is called based at 0
	mov	eax, 201h			; DPMI Set Real-mode Interrupt command
	mov	bl, _Real_Init_Int		; BL = int number to check
	int	31h

	inc	file_count			; One more driver installed

get_next_file:
	mov	ah, 4fh				; AH = DOS Find Next command
	int	21h
	jnc	load10_realmode
	jmp	done

; *
; * Skip the file and deallocate its memory
; * 	selector = selector of buffer to free
; *
skip_file:
	mov	eax, 101h			; DPMI Free DOS Memory command
	mov	edx, selector
	int	31h
	jmp	get_next_file


derr_allocate:
	mov	eax, -86			; Allocation error
	jmp	short derr

derr_filesize:
	mov	eax, -100			; Bad file size
	jmp	short derr

derr_close:
	push	eax
	mov	ah, 3eh
	mov	ebx, handle
	int	21h
	pop	eax

derr:	cwd
	mov	file_count, eax
done:	mov	eax, file_count		; EAX = number of files found or error code
	ret

load_realmode_drivers_ endp


; *************************************************************************************************
; *
; *	int STOP_REALMODE_DRIVERS( void )
; *	Returns the real-mode INT vector to its original value
; *
; *	Given:
; *		Previous_Int = original real-mode vector
; *
; *	Returns:
; *		0 if all OK
; *
; *************************************************************************************************
stop_realmode_drivers_ proc uses ebx ecx edx
local	regs:REG_S

	xor	eax, eax
	cmp	dword ptr Previous_Int, eax	; See if it was set
	jz	short done

; *
; * Clear the calling structure
; *
	mov	ecx, (size REG_S)/4
	lea	edi, regs
	rep	stosd
	mov	regs.eax32, 1			; Tell them to disengage

; *
; * Call the drivers with EAX = 1 to stop all activity
; *
	mov	ah, 3				; AX = DPMI Simulate Real-mode Interrupt command (300h)
	movzx	ebx, _Real_Init_Int
	lea	edi, regs
	int	31h

; *
; * Restore the access interrupt to its original condition
; *
	mov	eax, 201h			; DPMI Set Real-mode Interrupt command
	mov	dx, Previous_Int
	mov	cx, Previous_Int+2
	mov	bl, _Real_Init_Int		; BL = int number to restore
	int	31h

done:	xor	eax, eax
	ret

stop_realmode_drivers_ endp


; *************************************************************************************************
; *
; *	int INIT_REALMODE_DRIVERS( void )
; *	Initializes the real-mode drivers
; *
; *	Given:
; *		_Real_Init_Int = INT number to start the initialization
; *
; *	Returns:
; *		0 if all OK
; *
; *************************************************************************************************
init_realmode_drivers_ proc uses ebx ecx edx edi
local	regs:REG_S

	xor	eax, eax
	cmp	dword ptr Previous_Int, eax	; See if it was set
	jz	short done

; *
; * Clear the calling structure
; *
	mov	ecx, (size REG_S)/4
	lea	edi, regs
	rep	stosd

; *
; * Call the drivers with EAX = 0 for initialization
; *
	mov	ah, 3				; AX = DPMI Simulate Real-mode Interrupt command (300h)
	movzx	ebx, _Real_Init_Int
	lea	edi, regs
	int	31h

done:	xor	eax, eax
	ret

init_realmode_drivers_ endp


	end
