/**************************************************************************************************
*
*		Title:	MOUSE.H
*		Copyright (c) January 1993, Ryu Consulting
*		Written by Rahner James
*
*		This is the header file that contains all the information and structures for mouse access
*
**************************************************************************************************/

#define	_MOUSE_H_

#ifndef FAR
	#ifdef __386__
		#define	HUGE
		#define	FAR
	#else
		#define	HUGE	huge
		#define	FAR		far
	#endif
#endif
#ifdef __WATCOMC__
	#ifdef __SMALL__
		#pragma library ("wcmous_s.lib");
	#else
		#pragma library ("wcmous_l.lib");
	#endif

	#ifndef	_MS_C_
		#define	_MS_C_
		#pragma	aux MASM	"_*";
	#endif
#endif
#pragma	pack(1)


/**************************************************************************************************
*
*										Definitions
*
**************************************************************************************************/

#define	MOUSE_LEFT			1			// Mask for the left button
#define	MOUSE_RIGHT			2			// Mask for the right button
#define	MOUSE_MIDDLE		4			// Middle button on a Logitech

#define	EVT_NOEVENTS		0
#define	EVT_MOUSEMOVE		1
#define	EVT_LPRESS			2
#define	EVT_LRELEASE		4
#define	EVT_RPRESS			8
#define	EVT_RRELEASE		0x10
#define	EVT_MPRESS			0x20
#define	EVT_MRELEASE		0x40
#define	EVT_SERIAL			0x2000		// Event triggered by the COM port
#define	EVT_TIMER			0x4000		// Event triggered by the timer
#define	EVT_KEYBOARD		0x8000		// Event triggered by the keyboard


/**************************************************************************************************
*
*										Structures
*
**************************************************************************************************/

typedef struct MOUSE_STATUS_S
{
	int		status;						// Current button status
	int		press_count;				// Number of presses
	int		press_x, press_y;			// X,Y position of the last press
	int		release_count;				// Number of releases
	int		release_x, release_y;		// X,Y position of the last release
} MOUSE_STATUS_T;

typedef struct EVENT_S
{
	union
	{
		unsigned short	mask;			// Event mask
		struct
		{
			unsigned short mmove : 1;
			unsigned short mlpress : 1;
			unsigned short mlrelease : 1;
			unsigned short mrpress : 1;

			unsigned short mrrelease : 1;
			unsigned short mmpress : 1;
			unsigned short mmrelease : 1;
			unsigned short reserved1 : 8;
			unsigned short keyboard : 1;
		};
	};
	unsigned char	ascii;				// ASCII character from keyboard
	unsigned char	scan;				// Scancode from the keyboard
	union
	{
		unsigned short	state;			// Keyboard flags, mouse button status
		struct
		{
			unsigned short rshift : 1;
			unsigned short lshift : 1;
			unsigned short ctl : 1;
			unsigned short alt : 1;
			unsigned short scroll_lock : 1;
			unsigned short num_lock : 1;
			unsigned short caps_lock : 1;
			unsigned short insert : 1;
			unsigned short left : 1;
			unsigned short right : 1;
			unsigned short middle : 1;
		};
	};
	int 			x;
	int				y;
	unsigned long	time;				// System tick time of the event
} EVENT_T;


/**************************************************************************************************
*
*										Prototypes
*
**************************************************************************************************/

extern	int	_Mouse_Button_Count;					// Number of mouse buttons, must be !0 if mouse functions will work
extern	int	_Mouse_Motion_X, _Mouse_Motion_Y;
extern	int	_Mouse_X, _Mouse_Y;						// Current location of the mouse, as of the last event
extern	char _Mouse_Button;							// Current state of the mouse buttons

extern	int cdecl clear_events( void );
extern	void cdecl disable_help( void );
extern	void cdecl enable_help( void );
extern	int cdecl get_event( EVENT_T far * );
extern	void cdecl init_help( void (FAR *)(void), unsigned short );
extern	void cdecl mouse_arrow( void );
extern	void cdecl mouse_cross( void );
extern	void cdecl mouse_finger( void );
extern	int cdecl mouse_get_motion( void );
extern	void FAR * cdecl mouse_get_pointer( int FAR *, int FAR * );
extern	int cdecl mouse_get_status( MOUSE_STATUS_T FAR *, int );
extern	int cdecl mouse_getxy( int *, int * );
extern	int cdecl mouse_gotoxy( int, int );
extern	int cdecl mouse_hide( void );
extern	void cdecl mouse_hourglass( void );
extern	int cdecl mouse_init( void );
extern	void cdecl mouse_pointing_hand( void );
extern	int cdecl mouse_reset();
extern	int cdecl mouse_set_bounds( int, int, int, int );
extern	int cdecl mouse_set_gcursor( unsigned short FAR *, int, int );
extern	int cdecl mouse_set_ratio( int, int );
extern	int cdecl mouse_set_speed( int );
extern	int cdecl mouse_set_tcursor( int, int, int );
extern	int cdecl mouse_set_vector( void (FAR *)(), int );
extern	int cdecl mouse_show( void );
extern	void cdecl mouse_stop_hand( void );
extern	void cdecl mouse_up_arrow( void );
extern	int cdecl mouse_update_region( int, int, int, int );
extern	int cdecl mouse_wait( int, int );				// Mask, result
extern	int cdecl put_event( EVENT_T far * );
extern	int cdecl start_events( int );
extern	void cdecl stop_events( void );
extern	int cdecl wait_event( EVENT_T far * );


/**************************************************************************************************
*
*										Pragmas
*
**************************************************************************************************/

