; *************************************************************************************************
; *
; *	Title:	MOUSE.INC
; *	Copyright (c) January 1993, Ryu Consulting
; *	Written by Rahner James
; *
; *	This file contains all the include code for the mouse access library
; *
; *************************************************************************************************

ifdef	LARGEMODEL
	.model	large,c
else
	.model	small,c
endif

NULL_CHECK	equ	1		; Set if functions should do NULL checking


; *************************************************************************************************
; *
; *					Timer Equates
; *
; *************************************************************************************************

TMR_RETRIGGER		equ	(1 shl 0)
TMR_DISABLED		equ	(1 shl 1)
TMR_DYNAMIC		equ	(1 shl 2)
TMR_SIGNAL_STOP	equ	(1 shl 3)
TMR_SET_BYTE		equ	(1 shl 4)	; Sets the byte pointed to by the buffer pointer when triggered
TMR_CALL_FUNCTION	equ	(1 shl 5)	; Calls the function when triggered
TMR_ACTIVE		equ	(1 shl 15)


; *************************************************************************************************
; *
; *					IPX/SPX Equates
; *
; *************************************************************************************************

CHECK_BROADCAST	equ	1

MAX_CONNECTIONS	equ	20
ECB_SIZE		equ	48
SPX_SIZE		equ	42
IPX_SIZE		equ	30

FLAG_SPX		equ	(1 shl 0)	; SPX packet type, 0 if IPX
FLAG_ESTABLISH		equ	(1 shl 1)	; Connection was the connection establisher
FLAG_CONNECTED		equ	(1 shl 2)	; Connection has been established
FLAG_STOPPED		equ	(1 shl 3)	; Connection has been stopped or was determined to be bad
FLAG_BROADCAST		equ	(1 shl 4)	; Broadcast type connection
FLAG_VALID		equ	(1 shl 15)	; Connection is valid


; *************************************************************************************************
; *
; *					IPX/SPX Structures
; *
; *************************************************************************************************

NODE_S struct
	network		dd	?
	node		db	6 dup(?)
	socket		dw	?
NODE_S ends


CONN_S struct
	next		dw	0		; -> offset of next connection
	index		dw	0		; Index into the Connection_Table for this entry
	flags		dw	0		; B0:0=IPX/1=SPX
						; B1:0=listen SPX/1=establish SPX
						; B2:1=connection established
						; B3:1=connection is bad or was stopped
						; B4:NCP type connection
						; B5:server, set if NCP server type
						; B8-B15: packet type for transmissions
	network		dd	0
	node		db	6 dup(0)
	tsocket		dw	0		; Socket used for transmissions
	lsocket		dw	0		; Socket used for receptions
	immediate	db	6 dup(0)
	connection	dw	0		; Connection number if this is an SPX connection, can be used by an IPX connection
	transport_time	dw	0
	listener_memory	dw	0		; Segment used to store the listeners
	listener_count	dw	0		; Number of listeners
CONN_S ends


ECB_S struct
	linkAddress		dd	?	; Used as a linked list pointer while ECB is in the IPX/SPX domain
	ESRAddress		dd	?	; -> function to call when this ECB has completed, AL=identifier,ESI->ECB
	inUseFlag		db	?	; State of this ECB
	completionCode		db	?	; Completion code this ECB, only valid when inUseFLag == 0
	socketNumber		dw	?	; Socket number of the source or destination

	reserved		db	16 dup(?)

	immediateAddress	db	6 dup(?); Physical destination of this ECB
	fragmentCount		dw	?	; Number of fragments to follow

	buffer1			dd	?	; -> SPX header
	buffer1_size		dw	?	; Size of SPX header, 30 for IPX, 42 for SPX
	buffer2			dd	?	; -> buffer
	buffer2_size		dw	?	; Size of buffer

	completion_ptr		dd	?	; -> word to fill with the inUseFlag and completionCode

	checkSum		dw	?	; Checksum number, unused by this driver, stored high-low
	spx_length		dw	?	; Length of the complete IPX packet, set by the IPX driver, stored high-low
	transportControl	db	?	; Number of bridges that this packet has passed, set to 0 by the IPX driver
	packetType		db	?	; Packet type number, SPX driver sets this to 5, application sets IPX to 0 or 4
	destination		db	12 dup(?);Internetwork address of the destination for the packet, filled by the application
	source			db	12 dup(?);Internetwork address of the destination for the packet, filled by the driver

	connectionControl	db	?	; Flags used by SPX, see SYSTEM_PACKET, ACKNOWLEDGEMENT_REQUIRED, END_OF_MESSAGE
	dataStreamType		db	?	; Can be used by the IPX application, SPX uses 0xFE and 0xFF
	sourceConnectionID	dw	?	; Connection ID of the source of the packet, stored high-low
	destConnectionID	dw	?	; Connection ID of the destination of the packet set upon xmit by app., stored high-low
	sequenceNumber		dw	?	; Sequence number of the packet, used by SPX, stored high-low
	acknowledgeNumber	dw	?	; Sequence number of the next packet that SPX expects to receive, stored high-low
	allocationNumber	dw	?	; Number of listening packets available at the receiver + acknowledgeNumber, stored high-low

ECB_S ends


; *************************************************************************************************
; *
; *					Mouse Structures
; *
; *************************************************************************************************

MOUSE_STATUS_S struct
	status		dw	?	; Current button status
	press_count	dw	?	; Number of presses
	press_x		dw	?	; X,Y position of the last press
	press_y		dw	?
	release_count	dw	?	; Number of releases
	release_x	dw	?	; X,Y position of the last release
	release_y	dw	?
MOUSE_STATUS_S ends


MOUSE_EVT struct
	etype		db	0
	emask		db	0
	time		dd	0	; Time at which the event occurred

	state		dw	0
	ascii		db	0	; ASCII, SCAN and STATE also double as a callback vector
	scan		db	0

	x		dw	0	; Doubles for extended pointer offset
	y		dw	0	; Doubles for extended pointer segment
MOUSE_EVT ends

TMR_EVT struct
	etype		db	0
	emask		db	0
	time		dd	0	; Time at which the event occurred

	timer		dd	0	; -> structure that triggered this event
	vector		dd	0	; -> data buffer associated with the timer
	value		dw	0	; Value associated with the timer event
TMR_EVT ends

SERIAL_EVT struct
	etype		db	0
	emask		db	0
	time		dd	0	; Time at which the event occurred

	status		dw	0	; Status at the time of the event
	com_number	db	0	; COM number of the event

SERIAL_EVT ends

NETWORK_EVT struct
	etype		db	0
	emask		db	0
	time		dd	0	; Time at which the event occurred

	connection	dw	0
	ecb		dd	0	; -> ECB that triggered the event

	completionCode	db	0	; Flags from the network operation

NETWORK_EVT ends

CALLBACK_EVT struct
	etype		db	0
	emask		db	0
	time		dd	0	; Time at which the event occurred

	function	dd	0	; -> function to call when the event is encountered
	vector		dd	0	; -> data buffer associated with the event
	value		dw	0	; Value associated with the event
CALLBACK_EVT ends


EVT_TIMER struct
	next		dd	0	; This field must be the first
	count		dd	0
	flags		dw	0
	counter_reload	dd	0

	function	dd	0	; -> function associated with the timer
	vector		dd	0	; -> data buffer associated with the timer
	value		dw	0	; Value associated with the timer event
EVT_TIMER ends


MOUSE	macro	command
	mov	ax, command
	int	33h
	endm


	.286

.data

ifndef _MINIT_ASM_
	extern	_Mouse_Button_Count:word
endif
ifndef _EVENT_ASM_
	extern	_Mouse_X:word, _Mouse_Y:word, _Mouse_Button:byte
endif
