; *************************************************************************************************
; *
; *	Title:	IPXEVENT.ASM
; *
; *	Takes care of any network events
; *
; *	March 18, 1994:	Corrected problem where ipx_open_connection was messing up the .next
; *			ptr for CONN_Ss openned
; *
; *************************************************************************************************

	include	mouse.inc

.data
	public	SPX_Retry_Count, Socket_Life, Connection_Table
	public	_Our_Address, _Our_Node

	even
_Our_Address label dword
_Our_Network		dd	0
_Our_Node		db	6 dup(0)

First_Talk_ECB		dd	0	; -> to the first ECB that is available
Talk_ECB_Count		dw	0	; Number of free talking ECBs
Talk_Buffer		dd	0	; -> buffer to use for talking ECBs
Talk_Total		dw	0	; Number of talkers allocated

Connection_Table	CONN_S	MAX_CONNECTIONS dup(<>)
First_Open_Conn	dw	0	; Offset of first open
First_Closed_Conn	dw	0	; Offset of first closed
Conn_Available		dw	MAX_CONNECTIONS

Max_Packet_Size	dw	576	; Maximum size of a data buffer with the IPX/SPX header included

SPX_Retry_Count	db	0	; Use default retry, these two must be paired as a word
Watchdog_Flag		db	1	; Enable watchdog

Socket_Life		db	0	; Socket life

.code

if @Codesize
	extern	put_network_event:far
else
	extern	put_network_event:near
endif

	even
IPX_Vector label dword
	dw	offset ipx_stub
	dw	@Code
ipx_stub:
	mov	ax, -2
	retf

fatal_error:
	pop	bp			; Put the return address in BP
	int	3
	hlt				; This ought to get 'em


; *************************************************************************************************
; *
; *	FIND_CONNECTION_MATCH
; *	Searches the open connection table for an SPX connection number
; *
; *	Given:
; *		AX = connection to match
; *		Interrupts disabled
; *
; *	Returns:
; *		DI = connection table index number
; *		AX destroyed
; *		DS set to our data segment
; *		All other registers preserved
; *		CARRY set if no match found
; *
; *	Note:
; *		This function will probably only be called from the ESRs
; *
; *************************************************************************************************
find_connection_match proc near private

	mov	di, @Data			; DS = our data segment
	mov	ds, di

	mov	di, First_Open_Conn		; DI -> first open connection
find10_match:
	or	di, di				; See if end of the line
	jz	short no_match
	cmp	[di].CONN_S.connection, ax	; See if we have a match
	je	short found_match		; Quit if we did
	mov	di, [di].CONN_S.next
	jmp	find10_match			; Loop until we reach the end of the list

found_match:
	mov	di, [di].CONN_S.index		; DI = index value

done:	ret

no_match:
	stc
	jmp	done

find_connection_match endp


; *************************************************************************************************
; *
; *	void far IPX_LISTEN_ESR
; *	Queues up an IPX listening packet
; *
; *	Given:
; *		ES:SI -> ECB to queue
; *		AL = -1
; *		Interrupts disabled
; *
; *	Returns:
; *		ECB queued
; *
; *************************************************************************************************
ipx_listen_esr proc far private uses ds

	mov	ax, @Data
	mov	ds, ax

ifdef CHECK_BROADCAST
; *
; * Check if this is broadcast from ourself
; *
	mov	ax, word ptr _Our_Node+4
	cmp	word ptr es:[si].ECB_S.immediateAddress+4, ax
	je	short check_id

listen10_esr:
endif
; *
; * If it's a bogus packet, chuck it
; *
	cmp	es:[si].ECB_S.completionCode, 0		; See if a good reception
	jnz	short return_the_packet			; Return it to the listening post if it's bogus

; *
; * Make the packet length not include the header and be in Intel format
; *
	mov	ax, es:[si].ECB_S.spx_length		; Get the Moto length + header
	xchg	ah, al					; Make it Intel
	sub	ax, es:[si].ECB_S.buffer1_size		; Just the packet and not the header
	mov	es:[si].ECB_S.spx_length, ax		; Store it for the foreground process to deal with

; *
; * Put into the event queue
; *
	call	put_network_event			; Get an event envelope, and fill it with defaults
	jc	short return_the_packet			; Send it back to listening if not enqueued
	mov	ds, dx					; DS:BX -> event envelope
	mov	[bx].NETWORK_EVT.emask, 1 shl 0		; Just need to set the receive bit

done_listen:
	ret

ifdef CHECK_BROADCAST
check_id:
	mov	ax, word ptr _Our_Node+2
	cmp	word ptr es:[si].ECB_S.immediateAddress+2, ax
	jne	listen10_esr
	mov	ax, word ptr _Our_Node
	cmp	word ptr es:[si].ECB_S.immediateAddress, ax
	jne	listen10_esr
endif

; *
; * Send ECB back to listening
; *
return_the_packet:
	mov	bx, 4					; IPX listen for packet
	push	bp
	call	[IPX_Vector]
	pop	bp
	jmp	done_listen

ipx_listen_esr endp


; *************************************************************************************************
; *
; *	void far SPX_LISTEN_ESR
; *	Queues up an SPX listening packet
; *
; *	Given:
; *		ES:SI -> ECB to queue
; *		AL = -1
; *		Interrupts disabled
; *
; *	Returns:
; *		ECB queued
; *
; *************************************************************************************************
spx_listen_esr proc far private uses ds

	cmp	es:[si].ECB_S.completionCode, 0EDh	; See if a watchdog trigger
	jne	short @F				; Skip if not
	mov	ax, word ptr es:[si].ECB_S.reserved	; AX = failed connection number
	mov	es:[si].ECB_S.destConnectionID, ax	; SPX connection # of the destination = AX
@@:	mov	ax, es:[si].ECB_S.destConnectionID	; AX = SPX connection # of the destination
	call	find_connection_match			; Finds a matching SPX connection in the CONN table, DI = CONN id
	jc	short return_the_packet			; Ignore the packet if there is no match

; *
; * Make the packet length not include the header and be in Intel format
; *
	mov	ax, es:[si].ECB_S.spx_length		
	xchg	ah, al
	sub	ax, es:[si].ECB_S.buffer1_size
	mov	es:[si].ECB_S.spx_length, ax

; *
; * Put into the event queue
; *
	call	put_network_event
	jc	short jump_fatal_error			; See if this ever triggers
	mov	ds, dx
	mov	[bx].NETWORK_EVT.emask, 1 shl 0		; Signal a received packet
	mov	[bx].NETWORK_EVT.connection, di		; Save the Connection_Table index

	cmp	es:[si].ECB_S.completionCode, 0		; See if a good reception
	jnz	short bad_code
	cmp	es:[si].ECB_S.dataStreamType, 0FEh	; See if a special packet, FE and FF are disconnects
	jz	short process_spx_disconnect		; Might be a control packet

done_listen:
	ret

jump_fatal_error:
	call	fatal_error

; *
; * Got a bad completion code
; * DS:BX -> event packet
; *
bad_code:
	mov	[bx].NETWORK_EVT.emask, 90h		; Signal an error with a disconnection
	jmp	short process10_spx_disconnect

process_spx_disconnect:
	mov	[bx].NETWORK_EVT.emask, 10h		; Disconnect
process10_spx_disconnect:
	mov	word ptr [bx].NETWORK_EVT.ecb, 0	; Make sure that the foreground does not return this packet again
	mov	word ptr [bx].NETWORK_EVT.ecb+2, 0

	call	close_disconnection			; Close the connection associated with the connection, returns with DS = data segment

; *
; * Send ECB back to listening
; *
return_the_packet:
	mov	bx, 17h					; SPX listen for sequenced packet
	push	bp
	call	[IPX_Vector]
	pop	bp
	jmp	done_listen

spx_listen_esr endp


; *************************************************************************************************
; *
; *	void far IPX_TALK_ESR
; *	Queues up a talking packet
; *
; *	Given:
; *		ES:SI -> ECB to queue
; *		AL = -1
; *		Interrupts disabled
; *
; *	Returns:
; *		ECB queued
; *
; *************************************************************************************************
ipx_talk_esr proc far private uses ds

	mov	ax, @Data
	mov	ds, ax

	inc	Talk_ECB_Count
	mov	ax, word ptr First_Talk_ECB
	mov	word ptr es:[si], ax
	mov	ax, word ptr First_Talk_ECB+2
	mov	word ptr es:[si+2], ax

	mov	word ptr First_Talk_ECB, si
	mov	word ptr First_Talk_ECB+2, es

	xor	bx, bx
	xchg	bx, word ptr es:[si].ECB_S.inUseFlag	; BX = completion set
	les	si, es:[si].ECB_S.completion_ptr
	mov	ax, es
	or	ax, si
	jz	@F
	mov	es:[si], bx

@@:	ret

ipx_talk_esr endp


; *************************************************************************************************
; *
; *	void far SPX_TALK_ESR
; *	Queues up a talking packet
; *
; *	Given:
; *		ES:SI -> ECB to queue
; *		AL = -1
; *		Interrupts disabled
; *
; *	Returns:
; *		ECB queued
; *
; *************************************************************************************************
spx_talk_esr proc far private uses ds

	cmp	es:[si].ECB_S.completionCode, 0		; See if it finished OK
	jnz	short spx_error

	mov	ax, @Data				; DS = our data segment
	mov	ds, ax

spx10_talk:
	inc	Talk_ECB_Count
	mov	ax, word ptr First_Talk_ECB
	mov	word ptr es:[si], ax
	mov	ax, word ptr First_Talk_ECB+2
	mov	word ptr es:[si+2], ax

	mov	word ptr First_Talk_ECB, si
	mov	word ptr First_Talk_ECB+2, es

	xor	bx, bx
	xchg	bx, word ptr es:[si].ECB_S.inUseFlag	; BX = completion set
	les	si, es:[si].ECB_S.completion_ptr
	mov	ax, es
	or	ax, si
	jz	@F
	mov	es:[si], bx

@@:	ret

; *
; * Here if there was an error in transmission
; *
spx_error:
	cmp	es:[si].ECB_S.completionCode, 0EDh	; See if a watchdog trigger
	jne	short @F				; Skip if not
	mov	ax, word ptr es:[si].ECB_S.reserved	; AX = failed connection number
	mov	es:[si].ECB_S.sourceConnectionID, ax	; SPX connection # of the source = AX
@@:	mov	ax, es:[si].ECB_S.sourceConnectionID	; AX = SPX connection # of the source
	call	find_connection_match			; Finds a matching SPX connection in the CONN table, DI = CONN id, DS = data segment
	jc	short spx10_talk

	call	put_network_event			; Get a packet to signal the foreground process
	jc	short spx10_talk			; Ah, well, I tried to tell them, they just weren't listening

	mov	ds, dx

; *
; * Got a bad completion code
; * DS:BX -> event packet
; *
	mov	[bx].NETWORK_EVT.connection, di		; Save the Connection_Table index
	mov	[bx].NETWORK_EVT.emask, 90h		; Signal an error with a disconnection
	mov	word ptr [bx].NETWORK_EVT.ecb, 0	; Make sure that the foreground does not return this packet again
	mov	word ptr [bx].NETWORK_EVT.ecb+2, 0

	call	close_disconnection			; Close the connection, returns with DS = data segment

	jmp	spx10_talk

spx_talk_esr endp


; *************************************************************************************************
; *
; *	void far SPX_CONNECTION_ESR
; *	Called by an ECB that just caught a connection by listening
; *
; *	Given:
; *		ES:SI -> ECB that listened for the connection
; *		AL = -1
; *		Interrupts disabled
; *
; *	Returns:
; *		ECB queued
; *
; *************************************************************************************************
spx_connection_esr proc far private uses ds

	mov	ax, @Data
	mov	ds, ax

	cmp	es:[si].ECB_S.completionCode, 0		; See if it was established OK
	jz	short good_connection

; *
; * An error, so what
; *
done:	mov	ax, word ptr SPX_Retry_Count
	mov	bx, 12h
	push	bp
	call	[IPX_Vector]
	pop	bp
	ret

abort_connection:
	mov	dx, word ptr es:[si].ECB_S.reserved	; DX = connection number
	mov	bx, 14h					; BX = SPX Abort Connection command
	push	si
	push	es
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	es
	pop	si
	jmp	done

; *
; * Create a connection entry for this ECB
; *
good_connection:
	call	put_network_event			; DX:BX -> event packet placed
	jc	abort_connection			; Abort if we can't get an event packet

	mov	di, bx					; DX:DI -> event packet
	mov	cx, es
	mov	es, dx					; ES:DI -> event packet
	mov	es:[di].NETWORK_EVT.etype, 0		; Make it a NULL event if we get an error in the next call
	call	get_new_connection			; Get a connection entry to put this in, DX:DI & CX saved
	jc	abort_connection			; Quit if there ain't one

	mov	es:[di].NETWORK_EVT.etype, 5		; Return the packet's network status
	mov	es:[di].NETWORK_EVT.emask, 1 shl 3	; Tell 'em the listen was filled
	mov	word ptr es:[di].NETWORK_EVT.ecb, 0
	mov	word ptr es:[di].NETWORK_EVT.ecb+2, 0
	mov	ax, [bx].CONN_S.index
	mov	es:[di].NETWORK_EVT.connection, ax

	mov	es, cx					; ES:SI -> connection ECB

	mov	[bx].CONN_S.flags, 8005h		; SPX connection, established, used

	mov	ax, word ptr es:[si].ECB_S.reserved+4	; Move the address over to the connection table
	mov	word ptr [bx].CONN_S.network, ax
	mov	ax, word ptr es:[si].ECB_S.reserved+6
	mov	word ptr [bx].CONN_S.network+2, ax
	mov	ax, word ptr es:[si].ECB_S.reserved+8
	mov	word ptr [bx].CONN_S.node, ax
	mov	ax, word ptr es:[si].ECB_S.reserved+10
	mov	word ptr [bx].CONN_S.node+2, ax
	mov	ax, word ptr es:[si].ECB_S.reserved+12
	mov	word ptr [bx].CONN_S.node+4, ax
	mov	ax, word ptr es:[si].ECB_S.reserved+14
	mov	[bx].CONN_S.tsocket, ax
	mov	ax, es:[si].ECB_S.socketNumber
	mov	[bx].CONN_S.lsocket, ax

	mov	ax, word ptr es:[si].ECB_S.immediateAddress	; Move the immediate address over to the connection table
	mov	word ptr [bx].CONN_S.immediate, ax
	mov	ax, word ptr es:[si].ECB_S.immediateAddress+2
	mov	word ptr [bx].CONN_S.immediate+2, ax
	mov	ax, word ptr es:[si].ECB_S.immediateAddress+4
	mov	word ptr [bx].CONN_S.immediate+4, ax

	mov	ax, word ptr es:[si].ECB_S.reserved		; AX = connection number
	mov	[bx].CONN_S.connection, ax

	mov	[bx].CONN_S.transport_time, 2
	mov	word ptr [bx].CONN_S.listener_memory, 0
	mov	word ptr [bx].CONN_S.listener_count, 0

	jmp	done

spx_connection_esr endp


; *************************************************************************************************
; *
; *	__GET_TALKER
; *	Gets the next talking ECB from the list
; *
; *	Given:
; *		DS = our data segment
; *		BX = connection pointer
; *
; *	Returns:
; *		CARRY set if there are none
; *		ES:SI -> ECB to use
; *		Uses AX
; *		Interrupts are enabled
; *		Talker count is decremented
; *
; *************************************************************************************************
__get_talker proc near private

	cli
	cmp	Talk_ECB_Count, 0		; See if there is an ECB to use
	jz	no_talkers

	les	si, First_Talk_ECB		; ES:SI -> ECB to connect with
	mov	ax, word ptr es:[si].ECB_S.linkAddress
	mov	word ptr First_Talk_ECB, ax
	mov	ax, word ptr es:[si].ECB_S.linkAddress+2
	mov	word ptr First_Talk_ECB+2, ax
	dec	Talk_ECB_Count
	sti

; *
; * Setup the talker ECB
; *
	mov	word ptr es:[si].ECB_S.ESRAddress+2, cs
	mov	es:[si].ECB_S.fragmentCount, 1
	lea	ax, es:[si].ECB_S.checksum
	mov	word ptr es:[si].ECB_S.buffer1, ax
	mov	word ptr es:[si].ECB_S.buffer1+2, es
	mov	es:[si].ECB_S.buffer1_size, SPX_SIZE

	mov	word ptr es:[si].ECB_S.completion_ptr, 0
	mov	word ptr es:[si].ECB_S.completion_ptr+2, 0

	clc
	ret

no_talkers:
	sti
	stc
	ret

__get_talker endp


; *************************************************************************************************
; *
; *	GET_NEW_CONNECTION
; *	Gets the next available connection and initializes it
; *
; *	Given:
; *		DS = this data segment
; *
; *	Returns:
; *		BX -> connection
; *		AX used, all other registers preserved
; *
; *************************************************************************************************
get_new_connection proc private

	cli
	cmp	Conn_Available, 0		; See if any are available
	jz	short derr

	mov	bx, First_Closed_Conn		; DS:BX -> first connection entry
	mov	ax, First_Open_Conn
	xchg	ax, [bx].CONN_S.next
	mov	First_Closed_Conn, ax
	mov	First_Open_Conn, bx
	dec	Conn_Available

	or	ax, ax
done:	sti
	ret

derr:	stc
	jmp	done

get_new_connection endp


; *************************************************************************************************
; *
; *	RETURN_CONNECTION
; *	Returns a connection to the free pool
; *
; *	Given:
; *		DS:BX -> connection to free
; *
; *	Returns:
; *		Connection returned
; *
; *************************************************************************************************
return_connection proc private

	cli
; *
; * Get us out of the open list
; *
	cmp	First_Open_Conn, bx	; See if we are first
	jne	short return10		; Skip if we aren't

	mov	ax, [bx].CONN_S.next
	mov	First_Open_Conn, ax
	jmp	short return30

return10:
	mov	ax, bx			; AX = this connection
	mov	bx, First_Open_Conn	; BX -> open list
return20:
	cmp	[bx].CONN_S.next, 0	; See if we are at the end
	jz	short done		; Quit if so
	cmp	[bx].CONN_S.next, ax	; See if we are next
	je	short @F		; Skip if we are
	mov	bx, [bx].CONN_S.next
	jmp	return20

@@:	push	di
	mov	di, bx			; DI -> the previous link
	mov	bx, ax
	mov	ax, [bx].CONN_S.next
	mov	[di].CONN_S.next, ax
	pop	di

return30:
	mov	ax, First_Closed_Conn	; DS:AX -> first connection entry
	mov	[bx].CONN_S.next, ax
	mov	First_Closed_Conn, bx
	mov	[bx].CONN_S.flags, 0

	inc	Conn_Available
done:
	sti
	ret

return_connection endp


; *************************************************************************************************
; *
; *	CLOSE_DISCONNECTION
; *	Returns a connection associated with a network event to the free pool
; *
; *	Given:
; *		DS:BX -> event that contains the conection index
; *
; *	Returns:
; *		Connection returned
; *		DS = data segment
; *		All other registers preserved
; *
; *************************************************************************************************
close_disconnection proc private uses ax bx

	imul	bx, [bx].NETWORK_EVT.connection, size CONN_S	; BX = offset of connection entry within the table
	add	bx, offset Connection_Table
	mov	ax, @Data
	mov	ds, ax

; *
; * Get us out of the open list
; *
	cmp	First_Open_Conn, bx	; See if we are first
	jne	short return10		; Skip if we aren't

	mov	ax, [bx].CONN_S.next
	mov	First_Open_Conn, ax
	jmp	short return30

return10:
	mov	ax, bx			; AX = this connection
	mov	bx, First_Open_Conn	; BX -> open list
return20:
	cmp	[bx].CONN_S.next, 0	; See if we are at the end
	jz	short done		; Quit if so
	cmp	[bx].CONN_S.next, ax	; See if we are next
	je	short @F		; Skip if we are
	mov	bx, [bx].CONN_S.next
	jmp	return20

@@:	push	di
	mov	di, bx			; DI -> the previous link
	mov	bx, ax
	mov	ax, [bx].CONN_S.next
	mov	[di].CONN_S.next, ax
	pop	di

return30:
	mov	ax, First_Closed_Conn	; DS:AX -> first connection entry
	mov	[bx].CONN_S.next, ax
	mov	First_Closed_Conn, bx
	mov	[bx].CONN_S.flags, 0

	inc	Conn_Available
done:
	ret

close_disconnection endp


; *************************************************************************************************
; *
; *	int IPX_FIND_HANDLE( NODE_T FAR *NP )
; *	Gets a handle for the connection
; *
; *	Given:
; *		NP -> node address of the connection to look for
; *
; *	Returns:
; *		Handle of connection or -1 if none found
; *
; *************************************************************************************************
ipx_find_handle proc uses di si ds, np:dword

	les	di, np			; ES:DI -> node address to search for
if NULL_CHECK
	mov	ax, es
	or	ax, di
	jz	short derr
endif
	mov	ax, seg Connection_Table
	mov	ds, ax
	mov	si, First_Open_Conn	; DS:SI -> first open connection

	cld
ipx10_find:
	or	si, si			; See if NULL ptr
	jz	short derr		; Quit with error if it is

	mov	bx, si			; Save the offset in BX
	add	si, CONN_S.network	; SI -> network address
	mov	di, word ptr np
	mov	cx, (size NODE_S)/2
	repe	cmpsw
	je	short done

	mov	si, [bx].CONN_S.next	; SI -> next in line
	jmp	ipx10_find

; *
; * Found the match
; * BX -> the match
; *
done:	mov	ax, [bx].CONN_S.index	; AX = connection index value
done10:
	ret

derr:	or	ax, -1
	jmp	done10

ipx_find_handle endp


; *************************************************************************************************
; *
; *	CONN_T FAR *IPX_FIND_POINTER( NODE_T FAR *NP )
; *	Gets a pointer for the connection
; *
; *	Given:
; *		NP -> node address of the connection to look for
; *
; *	Returns:
; *		Pointer to connection or NULL if no match found
; *
; *************************************************************************************************
ipx_find_pointer proc uses di si ds, np:dword

	les	di, np			; ES:DI -> node address to search for
if NULL_CHECK
	mov	ax, es
	or	ax, di
	jz	short derr
endif
	mov	ax, seg Connection_Table
	mov	ds, ax
	mov	si, First_Open_Conn	; DS:SI -> first open connection

	cld
ipx10_find:
	or	si, si			; See if NULL ptr
	jz	short derr		; Quit with error if it is

	mov	bx, si			; Save the offset in BX
	add	si, CONN_S.network	; SI -> network address
	mov	di, word ptr np
	mov	cx, (size NODE_S)/2
	repe	cmpsw
	je	short done

	mov	si, [bx].CONN_S.next	; SI -> next in line
	jmp	ipx10_find

; *
; * Found the match
; * BX -> the match
; *
done:	mov	dx, ds
	mov	ax, bx
done10:
	ret

derr:	xor	ax, ax
	mov	dx, ax
	jmp	done10

ipx_find_pointer endp


; *************************************************************************************************
; *
; *	int IPX_INIT( int TALK_COUNT )
; *	Initializes the IPX interface
; *
; *	Given:
; *		TALK_COUNT = number of talkers to allocate
; *				this allocates the ECBs and a SPX header
; *
; *	Returns:
; *		0 if all was done OK
; *		-1 if there was not enough memory
; *		-2 if IPX was not present
; *		-3 if all was already initialized
; *
; *************************************************************************************************
ipx_init proc uses ds di si, talk_count:word
local	rv:word
local	buffer_size:word

	mov	ax, @Data
	mov	ds, ax

	mov	rv, -3			; Assume already installed
	cmp	Talk_Total, 0		; See if this was already done
	jnz	done_ipx_init		; Quit if so

	mov	rv, -2			; Assume IPX is not there
	mov	ax, 7a00h		; See if IPX is present
	int	2fh
	inc	al			; See if it's not installed
	jz	@F			; Skip if it's OK
ipx10_init:
	jmp	done_ipx_init		; Quit if it's not

@@:	mov	word ptr IPX_Vector, di
	mov	word ptr IPX_Vector+2, es

; *
; * Get out internetwork address
; *
	mov	ax, @Data
	mov	es, ax
	mov	si, offset _Our_Network
	mov	bx, 9
	push	bp
	call	[IPX_Vector]
	pop	bp

; *
; * Allocate the memory we need to use for talkers
; *
	mov	rv, -1			; Assume not enough memory
	mov	cx, talk_count		; Calculate the number of paragraphs to allocate
	jcxz	ipx10_init
	mov	ax, size ECB_S		; AX = ECB + SPX header
	mul	cx
	or	dx, dx
	jnz	done_ipx_init		; Quit if it needs more than 64K

	mov	buffer_size, ax		; Save the size for later

	add	ax, 15			; Convert to paragraphs
	adc	dx, 0
	mov	bx, 16
	div	bx

	mov	bx, ax			; BX = number of paragraphs
	mov	ah, 48h			; AH = DOS Allocate Memory Blocks command
	int	21h
	jc	done_ipx_init		; Quit if nothing to allocate
	mov	rv, 0			; All good from here on

	mov	word ptr Talk_Buffer, 0
	mov	word ptr Talk_Buffer+2, ax
	mov	word ptr First_Talk_ECB, 0
	mov	word ptr First_Talk_ECB+2, ax

; *
; * Initialize the talking buffer
; *
	les	di, Talk_Buffer
	xor	ax, ax
	mov	cx, buffer_size
	shr	cx, 1
	cld
	rep	stosw

	les	di, Talk_Buffer		; Make the linked list
	mov	cx, talk_count
	mov	Talk_Total, cx
	mov	Talk_ECB_Count, cx
	dec	cx			; No need to link the last one

@@:	lea	ax, [di+(size ECB_S)]	; AX = DI + ECB + SPX header size
	mov	word ptr es:[di], ax
	mov	word ptr es:[di+2], es
	mov	di, ax
	loop	@B

; *
; * Initialize the connection buffer
; *
	mov	ax, @Data
	mov	es, ax
	mov	di, offset Connection_Table
	mov	First_Closed_Conn, di
	xor	ax, ax
	mov	cx, (MAX_CONNECTIONS * (size CONN_S))/2
	rep	stosw

	mov	di, offset Connection_Table
	mov	cx, MAX_CONNECTIONS
	dec	cx			; No need to link the last one
	xor	bx, bx			; BX = index number

@@:	lea	ax, [di+(size CONN_S)]	; AX = DI + connection entry size
	mov	[di].CONN_S.next, ax	; -> next available
	mov	[di].CONN_S.index, bx	; Fill in the index number for this entry so we don't have to DIVide
	mov	di, ax
	inc	bx			; BX = next index number
	loop	@B
	mov	[di].CONN_S.index, bx	; Fill in the last index number

	mov	First_Open_Conn, 0
	mov	Conn_Available, MAX_CONNECTIONS

done_ipx_init:
	mov	ax, rv
	ret

ipx_init endp


; *************************************************************************************************
; *
; *	int IPX_OPEN_CONNECTION( char FAR *NODE_ID, int LISTENER_COUNT, unsigned int LSOCKET, unsigned int FLAGS )
; *	Opens a communication channel
; *
; *	Given:
; *		NODE_ID -> full internetwork address, including the network, node, and socket
; *				the socket will be opened, local target gotten
; *		LISTENER_COUNT = number of listeners to attach
; *		LSOCKET = socket used for listening
; *		FLAGS = ; B0:0=IPX/1=SPX, B1:0=listen SPX/1=establish SPX
; *			; B2:1=connection established
; *			; B3:1=connection is bad or was stopped
; *			; B8-B15:if non-0, will be used for the packet type
; *
; *	Returns:
; *		Connection handle if all went well
; *		< 0 if an error occurred
; *		-1 if not enough memory
; *		-2 if IPX not initialized
; *		-3 if parameter sent was bad
; *		-4 if socket error
; *		-5 if no connections are available
; *		-6 if no local target
; *		-9 if no talking sockets were available for the SPX connection
; *		-10 if no remote SPX connection
; *
; *************************************************************************************************
ipx_open_connection proc uses ds di si, node_id:dword, listener_count:word, lsocket:word, flags:word
local	rv:word
local	ep:word			; Segment of the memory block allocated for this connection
local	buffer_size:word
local	packet_type:byte	; Packet type to use for the packets
local	conn_ptr:word		; -> connection entry being filled

	mov	ax, @Data
	mov	ds, ax

	cld
	mov	ep, 0

	mov	rv, -2
	cmp	word ptr IPX_Vector+2, @Code
	je	done_open

	mov	rv, -3			; Assume bad parameter
	mov	ax, word ptr node_id
	or	ax, word ptr node_id+2
	jz	done_open
	mov	cx, listener_count	; See if we need to allocate space for the listeners
	jcxz	no_listeners

; *
; * Allocate space for the listeners
; *
	mov	rv, -1			; Assume not enough memory

	mov	ax, Max_Packet_Size
	add	ax, (size ECB_S) - SPX_SIZE
	mul	cx			; DX:AX = number of bytes needed
	mov	buffer_size, ax
	or	dx, dx			; See if above 64K
	jnz	done_open		; Quit if so
	add	ax, 15
	jc	done_open		; Quit if too much
	mov	cx, 16
	div	cx			; AX = number of paragraphs

	mov	bx, ax
	mov	ah, 48h			; AH = DOS Allocate Memory command
	int	21h
	jc	done_open		; Quit if error in allocating

	mov	ep, ax			; Clear the buffer
	mov	es, ax
	xor	di, di
	mov	ax, di
	mov	cx, buffer_size
	shr	cx, 1
	rep	stosw

; *
; * Make sure that there is a connection available
; *
no_listeners:
	mov	rv, -5
	mov	conn_ptr, -1
	call	get_new_connection
	jc	short pre_free_done_open	; Quit if there aren't
	mov	conn_ptr, bx

; *
; * Open the talking socket
; *
	les	di, node_id			; ES:DI -> node ID

	mov	rv, -4				; Assume socket error
	mov	bx, 0				; BX = IPX Open Socket command
	mov	al, Socket_Life
	mov	dx, es:[di].NODE_S.socket	; DX = socket number to use for transmissions

	push	bp
	call	[IPX_Vector]
	pop	bp
	cmp	al, 0FEh			; See if the socket table is full
	je	pre_free_done_open		; Quit if it is

	les	di, node_id			; ES:DI -> node ID
	or	al, al				; Any other error, skip the fill in
	jnz	short @F
	mov	es:[di].NODE_S.socket, dx

@@:	cmp	lsocket, 0			; See if the listener socket needs to be filled with the transmitter socket number
	jnz	short open03_conn

	mov	ax, es:[di].NODE_S.socket
	mov	lsocket, ax
	jmp	short open05_conn

pre_free_done_open:
	cmp	conn_ptr, -1			; See if we need to return the connection pointer
	je	@F
	mov	bx, conn_ptr
	call	return_connection
@@:	jmp	free_done_open

; *
; * Open the listening socket
; *
open03_conn:
	mov	bx, 0				; BX = IPX Open Socket command
	mov	al, Socket_Life
	mov	dx, lsocket			; DX = socket number to use for receptions

	push	bp
	call	[IPX_Vector]
	pop	bp
	cmp	al, 0FEh			; See if the socket table is full
	je	short pre_free_done_open	; Quit if it is

; *
; * Fill in the connection table entry
; *
open05_conn:
	mov	bx, conn_ptr			; BX -> new connection

	les	di, node_id
	mov	ax, word ptr es:[di].NODE_S.network
	mov	dx, ax
	mov	word ptr [bx].CONN_S.network, ax
	mov	ax, word ptr es:[di].NODE_S.network+2
	or	dx, ax
	mov	word ptr [bx].CONN_S.network+2, ax
	mov	ax, word ptr es:[di].NODE_S.node
	or	dx, ax
	mov	word ptr [bx].CONN_S.node, ax
	mov	ax, word ptr es:[di].NODE_S.node+2
	or	dx, ax
	mov	word ptr [bx].CONN_S.node+2, ax
	mov	ax, word ptr es:[di].NODE_S.node+4
	or	dx, ax
	mov	word ptr [bx].CONN_S.node+4, ax

	mov	ax, word ptr es:[di].NODE_S.socket
	mov	[bx].CONN_S.tsocket, ax
	mov	ax, lsocket
	mov	[bx].CONN_S.lsocket, ax

	mov	ax, flags
	and	ax, FLAG_SPX+FLAG_ESTABLISH+FLAG_BROADCAST+07f00h	; Clear all but the first two bits
	or	ah, 1 shl 7
	mov	[bx].CONN_S.flags, ax
	mov	[bx].CONN_S.connection, -1
	mov	ax, ep
	mov	[bx].CONN_S.listener_memory, ax
	mov	ax, listener_count
	mov	[bx].CONN_S.listener_count, ax

; *
; * Get the immediate address of the internetwork address
; *
	mov	[bx].CONN_S.transport_time, dx
	or	dx, dx				; If NULL address, don't look for immediate address
	jz	short @F

	mov	ax, @Data
	mov	es, ax
	lea	si, [bx].CONN_S.network
	lea	di, [bx].CONN_S.immediate
	push	bx
	mov	bx, 2
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	bx
	mov	[bx].CONN_S.transport_time, cx

	or	al, al			; See if we got an error here
	jz	@F			; Skip if it's OK

	mov	rv, -6			; No local target
	call	return_connection	; Return the connection to the closed pool
	jmp	free_done_open

; *
; * Send the listeners out to the listening post, if there some
; *
@@:	mov	ax, [bx].CONN_S.index		; AX = connection handle
	mov	rv, ax

	mov	cx, ep			; CX = listener memory segment
	jcxz	short open20_conn	; Quit if we didn't

	mov	es, cx
	mov	cx, listener_count
	xor	si, si

	mov	dx, lsocket			; DX = listening socket number
	push	bx				; Save the connection pointer

	push	offset ipx_listen_esr
	mov	bx, 4				; BX = IPX command
	mov	di, IPX_SIZE			; DI = size of the header
	test	flags, FLAG_SPX			; See if SPX packet
	jz	short open10_conn		; Skip if it's IPX

	pop	ax
	push	offset spx_listen_esr
	mov	bx, 17h				; BX = SPX command
	mov	di, SPX_SIZE			; DI = size of the header

open10_conn:
	mov	es:[si].ECB_S.socketNumber, dx
	pop	word ptr es:[si].ECB_S.ESRAddress
	push	word ptr es:[si].ECB_S.ESRAddress
	mov	word ptr es:[si].ECB_S.ESRAddress+2, cs
	mov	word ptr es:[si].ECB_S.completion_ptr, 0
	mov	word ptr es:[si].ECB_S.completion_ptr+2, 0
	mov	es:[si].ECB_S.fragmentCount, 2
	lea	ax, es:[si].ECB_S.checksum
	mov	word ptr es:[si].ECB_S.buffer1, ax
	mov	word ptr es:[si].ECB_S.buffer1+2, es
	mov	es:[si].ECB_S.buffer1_size, di
	add	ax, di
	mov	word ptr es:[si].ECB_S.buffer2, ax
	mov	word ptr es:[si].ECB_S.buffer2+2, es
	mov	ax, Max_Packet_Size
	sub	ax, di
	mov	es:[si].ECB_S.buffer2_size, ax

	pusha
	push	es
	call	[IPX_Vector]
	pop	es
	popa
	add	si, (size ECB_S) - SPX_SIZE
	add	si, Max_Packet_Size
	loop	open10_conn

	pop	ax			; Clear the ESR offset
	pop	bx			; BX -> connection entry

open20_conn:
; *
; * See if we need to take care of the SPX stuff
; *
	test	flags, FLAG_SPX				; See if SPX connection
	jnz	process_spx_connection

	or	[bx].CONN_S.flags, FLAG_CONNECTED	; Set this bit to inform the connection was made

done_open:
	sti
	mov	ax, rv
	ret

free_done_open:
	cmp	ep, 0			; See if we need to free the memory
	jz	done_open
	mov	es, ep
	mov	ah, 49h
	int	21h
	jmp	done_open

; *
; * Start any SPX connections
; * DS:BX -> connection structure
; *
process_spx_connection:
	and	[bx].CONN_S.flags, 80ffh	; Clear the packet type bits
	or	[bx].CONN_S.flags, 500h		; Force the packet type to be an SPX type

	call	__get_talker			; ES:SI -> next available talker
	jnc	@F				; CARRY set if there are none

	push	rv
	call	ipx_close_connection		; Close the connection
	add	sp, 2
	mov	rv, -9				; Return appropriate error code
	jmp	done_open

@@:	mov	ax, [bx].CONN_S.tsocket
	mov	es:[si].ECB_S.socketNumber, ax

	test	flags, FLAG_ESTABLISH		; See if bwana wants to establish the connection
	jz	listen_for_connection		; Jump if just a listener

; *
; * Setup the talker ECB
; *
	mov	word ptr es:[si].ECB_S.ESRAddress, 0		; offset establish_esr
	mov	word ptr es:[si].ECB_S.ESRAddress+2, 0
	mov	ax, word ptr [bx].CONN_S.immediate
	mov	word ptr es:[si].ECB_S.immediateAddress, ax
	mov	ax, word ptr [bx].CONN_S.immediate+2
	mov	word ptr es:[si].ECB_S.immediateAddress+2, ax
	mov	ax, word ptr [bx].CONN_S.immediate+4
	mov	word ptr es:[si].ECB_S.immediateAddress+4, ax
	mov	es:[si].ECB_S.packetType, 5

	mov	ax, word ptr [bx].CONN_S.network
	mov	word ptr es:[si].ECB_S.destination, ax
	mov	ax, word ptr [bx].CONN_S.network+2
	mov	word ptr es:[si].ECB_S.destination+2, ax
	mov	ax, word ptr [bx].CONN_S.network+4
	mov	word ptr es:[si].ECB_S.destination+4, ax
	mov	ax, word ptr [bx].CONN_S.network+6
	mov	word ptr es:[si].ECB_S.destination+6, ax
	mov	ax, word ptr [bx].CONN_S.network+8
	mov	word ptr es:[si].ECB_S.destination+8, ax
	mov	ax, word ptr [bx].CONN_S.network+10
	mov	word ptr es:[si].ECB_S.destination+10, ax

; *
; * Establish the connection
; *
	push	es
	push	si
	push	bx
	mov	bx, 11h				; BX = SPX Establish Connection command
	mov	ax, word ptr SPX_Retry_Count
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	bx
	pop	si
	pop	es

	or	al, al			; See if we got an immediate error from the SPX Establish
	jz	@F

close_spx_error_connection:
	push	rv
	call	ipx_close_connection	; Close the connection
	add	sp, 2
	mov	rv, -10			; Return appropriate error code
	jmp	done_open

; *
; * Wait for the packet to pass muster
; *
@@:	cmp	es:[si].ECB_S.inUseFlag, 0	; See if still OK
	jnz	@B				; Loop 'til we drop

	mov	[bx].CONN_S.connection, dx	; Save the connection ID

	cli
	inc	Talk_ECB_Count
	mov	ax, word ptr First_Talk_ECB
	mov	word ptr es:[si], ax
	mov	ax, word ptr First_Talk_ECB+2
	mov	word ptr es:[si+2], ax

	mov	word ptr First_Talk_ECB, si
	mov	word ptr First_Talk_ECB+2, es
	sti

	cmp	es:[si].ECB_S.completionCode, 0	; See if we connected OK
	jnz	close_spx_error_connection

	or	[bx].CONN_S.flags, 7		; Show that we are connected
	jmp	done_open

; *
; * Here to set up the listen for connection
; * DS:BX -> connection structure
; * ES:SI -> ECB to use for the talking
; *
listen_for_connection:
	mov	word ptr es:[si].ECB_S.ESRAddress, offset spx_connection_esr

; *
; * Listen for the connection
; *
	push	bx
	mov	bx, 12h				; BX = SPX Listen for Connection command
	mov	ax, word ptr SPX_Retry_Count
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	bx

	jmp	done_open

ipx_open_connection endp


; *************************************************************************************************
; *
; *	int IPX_WRITE_PACKET( int HANDLE, void FAR *BUFFER, int COUNT, unsigned short FAR *CC_PTR )
; *	Writes a packet through a channel
; *
; *	Given:
; *		HANDLE = channel to write the packet out
; *		BUFFER -> data to write
; *		COUNT = number of bytes to write
; *		CC_PTR -> word ptr filled in by the ESR
; *
; *	Returns:
; *		0 if all went well
; *		-2 if IPX not initialized
; *		-3 if parameter sent was bad
; *		-5 if bad connection number
; *		-7 if no talking ECB is available
; *		-10 if no connection made yet
; *
; *************************************************************************************************
ipx_write_packet proc uses ds di si, handle:word, buffer:dword, count:word, cc_ptr:dword
local	rv:word

	mov	rv, -5
	mov	ax, handle
	cmp	ax, MAX_CONNECTIONS
	jnb	done_write

; *
; * Get a pointer to the connection entry
; *
	mov	bx, @Data
	mov	ds, bx
	mov	bx, size CONN_S
	mul	bx
	mov	bx, ax			; DS:BX -> connection entry
	add	bx, offset Connection_Table

	test	[bx].CONN_S.flags, FLAG_VALID
	jz	done_write
	mov	rv, -10
	test	[bx].CONN_S.flags, FLAG_CONNECTED	; Set this bit to inform the connection was made
	jz	done_write

	mov	rv, -3
	mov	ax, word ptr buffer
	or	ax, word ptr buffer+2
	jz	done_write

	mov	rv, -7
	call	__get_talker		; ES:SI -> next available talking packet
	jc	done_write

	mov	rv, 0			; Assume the rest of the function will be OK

; *
; * Setup the talker ECB
; *
	mov	ax, [bx].CONN_S.tsocket
	mov	es:[si].ECB_S.socketNumber, ax
	mov	ax, word ptr [bx].CONN_S.immediate
	mov	word ptr es:[si].ECB_S.immediateAddress, ax
	mov	ax, word ptr [bx].CONN_S.immediate+2
	mov	word ptr es:[si].ECB_S.immediateAddress+2, ax
	mov	ax, word ptr [bx].CONN_S.immediate+4
	mov	word ptr es:[si].ECB_S.immediateAddress+4, ax
	mov	es:[si].ECB_S.fragmentCount, 2
	mov	ax, word ptr buffer
	mov	word ptr es:[si].ECB_S.buffer2, ax
	mov	ax, word ptr buffer+2
	mov	word ptr es:[si].ECB_S.buffer2+2, ax
	mov	ax, count
	mov	es:[si].ECB_S.buffer2_size, ax
	mov	ax, word ptr [bx].CONN_S.network
	mov	word ptr es:[si].ECB_S.destination, ax
	mov	ax, word ptr [bx].CONN_S.network+2
	mov	word ptr es:[si].ECB_S.destination+2, ax
	mov	ax, word ptr [bx].CONN_S.network+4
	mov	word ptr es:[si].ECB_S.destination+4, ax
	mov	ax, word ptr [bx].CONN_S.network+6
	mov	word ptr es:[si].ECB_S.destination+6, ax
	mov	ax, word ptr [bx].CONN_S.network+8
	mov	word ptr es:[si].ECB_S.destination+8, ax
	mov	ax, word ptr [bx].CONN_S.network+10
	mov	word ptr es:[si].ECB_S.destination+10, ax
	mov	ax, word ptr [bx].CONN_S.lsocket
	mov	word ptr es:[si].ECB_S.source+10, ax

	push	ds
	lds	di, cc_ptr
	mov	word ptr es:[si].ECB_S.completion_ptr, di
	mov	word ptr es:[si].ECB_S.completion_ptr+2, ds
	mov	ax, ds
	or	ax, di
	jz	@F
	mov	word ptr [di], -1
@@:	pop	ds

	mov	dx, [bx].CONN_S.connection
	test	[bx].CONN_S.flags, FLAG_SPX	; See if it's SPX or IPX
	jnz	write_spx

; *
; * Setup and send the IPX packet
; *
	mov	es:[si].ECB_S.buffer1_size, IPX_SIZE
	mov	al, byte ptr [bx].CONN_S.flags+1
	and	al, 7fh					; Clear all but the packet type
	mov	es:[si].ECB_S.packetType, al		; Set the packet type
	mov	bx, 3
	mov	word ptr es:[si].ECB_S.ESRAddress, offset ipx_talk_esr
	jmp	short @F

; *
; * Setup and send the SPX packet
; *
write_spx:
	mov	es:[si].ECB_S.buffer1_size, SPX_SIZE
	mov	es:[si].ECB_S.packetType, 5
	mov	bx, 16h
	mov	word ptr es:[si].ECB_S.ESRAddress, offset spx_talk_esr

@@:	push	es
	push	si
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	si					; Restore ES:SI
	pop	es

; *
; * See if the packet has finished before it began
; *
	cmp	es:[si].ECB_S.inUseFlag, 0		; See if already done
	jnz	short done_write			; Skip if not yet
	cmp	es:[si].ECB_S.completionCode, 0		; See if it was done with an error, this would be cleared by the ESR
	jz	short done_write

	cli
	call	es:[si].ECB_S.ESRAddress		; Call the ESR to process the goodies
	sti

done_write:
	sti
	mov	ax, word ptr rv
	ret

ipx_write_packet endp


; *************************************************************************************************
; *
; *	int IPX_LISTEN_PACKET( ECB FAR *EP )
; *	Returns a listening ECB back to the listening queue
; *
; *	Given:
; *		EP -> ECB to send back to the listening post
; *
; *	Returns:
; *		0 if the packet was returned OK
; *		-2 if IPX not initialized
; *		-3 if parameter sent was bad
; *		-5 if bad connection number
; *		-7 if no talking ECB is available
; *
; *************************************************************************************************
ipx_listen_packet proc uses di si ds, ep:dword
	
	les	si, ep			; ES:SI -> ECB
	mov	ax, es
	or	ax, si
	jz	short done_listen

	mov	bx, 17h				; BX = SPX listen command
	cmp	es:[si].ECB_S.packetType, 5	; See if an SPX packet type
	je	short @F

	mov	bx, 4				; BX = IPX command

@@:	push	bp
	call	[IPX_Vector]
	pop	bp
	cbw

done_listen:
	ret

ipx_listen_packet endp


; *************************************************************************************************
; *
; *	CONN_S FAR *IPX_GET_CONNECTION( int HANDLE )
; *	Gets a pointer to the connection structure that is defined by the handle
; *
; *	Given:
; *		HANDLE = connection to check
; *
; *	Returns:
; *		-> connection structure
; *
; *************************************************************************************************
ipx_get_connection proc uses ds, handle:word

	mov	dx, @Data
	mov	ds, dx

	cmp	word ptr IPX_Vector+2, @Code
	je	derr

	mov	ax, handle
	cmp	ax, MAX_CONNECTIONS
	jnb	derr

; *
; * Get a pointer to the connection entry
; *
	mov	bx, size CONN_S
	mul	bx
	add	ax, offset Connection_Table
	mov	bx, ax			; DS:BX -> connection entry

	test	[bx].CONN_S.flags, FLAG_VALID
	jz	derr

	mov	dx, ds

done:	ret

derr:	xor	dx, dx
	mov	ax, dx
	jmp	done

ipx_get_connection endp


; *************************************************************************************************
; *
; *	int IPX_CLOSE_SOCKET( unsigned short SOCKET )
; *	Closes a socket
; *
; *	Given:
; *		SOCKET = socket number to close
; *
; *	Returns:
; *		Always 0
; *
; *************************************************************************************************
ipx_close_socket proc uses di si, socket:word

	mov	dx, socket
	mov	bx, 1
	push	bp
	call	[IPX_Vector]
	pop	bp

	xor	ax, ax
	ret

ipx_close_socket endp


; *************************************************************************************************
; *
; *	int IPX_STOP( void )
; *	Closes down all sockets and allocated memory structures
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		0 if all connections were closed OK
; *		-2 if IPX not initialized
; *
; *************************************************************************************************
ipx_stop proc uses di si ds
local	rv:word

	mov	ax, @Data
	mov	ds, ax

	mov	rv, -2			; Assume IPX was not initialized
	cmp	word ptr IPX_Vector+2, @Code
	je	done_stop

; *
; * Close all open sockets
; *
	mov	rv, 0
	mov	si, First_Open_Conn	; DS:SI -> first open connection
ipx10_stop:
	or	si, si			; See if we reached the end of the list
	jz	ipx20_stop		; Quit if we did

	mov	dx, [si].CONN_S.tsocket
	or	dx, dx
	jz	@F
	mov	bx, 1			; BX - IPX Close Socket command
	push	si
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	si

@@:	mov	dx, [si].CONN_S.lsocket
	or	dx, dx
	jz	@F
	mov	bx, 1			; BX - IPX Close Socket command
	push	si
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	si

@@:	mov	si, [si].CONN_S.next	; SI -> next connection entry in line
	jmp	ipx10_stop

; *
; * Free all memory used for listeners
; *
ipx20_stop:
	mov	si, First_Open_Conn	; DS:SI -> first open connection
ipx30_stop:
	or	si, si			; See if we reached the end of the list
	jz	ipx40_stop		; Quit if we did

	mov	es, [si].CONN_S.listener_memory
	mov	cx, es
	jcxz	@F
	mov	ah, 49h			; AH = DOS Free Memory command
	int	21h

@@:	mov	si, [si].CONN_S.next	; SI -> next connection entry in line
	jmp	ipx30_stop

; *
; * Free all memory used for talkers
; *
ipx40_stop:
	mov	es, word ptr Talk_Buffer+2
	mov	cx, es
	jcxz	@F
	mov	ah, 49h			; AH = DOS Free Memory command
	int	21h

@@:	mov	word ptr First_Talk_ECB, 0	; -> to the first ECB that is available
	mov	word ptr First_Talk_ECB+2, 0
	mov	Talk_ECB_Count, 0		; Number of free talking ECBs
	mov	word ptr Talk_Buffer, 0		; -> buffer to use for talking ECBs
	mov	word ptr Talk_Buffer+2, 0
	mov	Talk_Total, 0			; Number of talkers allocated
	mov	First_Open_Conn, 0		; Offset of first open
	mov	First_Closed_Conn, 0		; Offset of first closed
	mov	Conn_Available, MAX_CONNECTIONS

	mov	word ptr IPX_Vector, offset ipx_stub
	mov	word ptr IPX_Vector+2, @Code

done_stop:
	mov	ax, rv
	ret

ipx_stop endp


; *************************************************************************************************
; *
; *	int IPX_CLOSE_CONNECTION( int HANDLE )
; *	Closes down all allocated memory structures for a connection handle
; *
; *	Given:
; *		HANDLE = handle of the connection to close
; *
; *	Returns:
; *		0 if the connection was closed OK
; *		-2 if IPX not initialized
; *
; *************************************************************************************************
ipx_close_connection proc uses di si ds, handle:word
local	rv:word

	mov	ax, @Data
	mov	ds, ax

	mov	rv, -2			; Assume IPX was not initialized
	cmp	word ptr IPX_Vector+2, @Code
	je	done_close

	mov	rv, -5
	mov	ax, handle
	cmp	ax, MAX_CONNECTIONS
	jnb	done_close

; *
; * Get a pointer to the connection entry
; *
	mov	bx, size CONN_S
	mul	bx
	mov	bx, ax			; DS:BX -> connection entry
	add	bx, offset Connection_Table

	test	[bx].CONN_S.flags, FLAG_VALID
	jz	done_close

; *
; * Terminate the connection if it's an SPX connection
; *
	test	[bx].CONN_S.flags, FLAG_SPX
	jz	cancel_ecbs
	call	__get_talker
	jc	cancel_ecbs

	mov	word ptr es:[si].ECB_S.ESRAddress, offset ipx_talk_esr	; This is so the connection won't be closed again by the ESR

; *
; * Terminate the connection
; *
	mov	dx, [bx].CONN_S.connection	; DX = SPX connection number
	push	bx
	mov	bx, 13h				; BX = SPX Terminate Connection command
	push	bp
	call	[IPX_Vector]
	pop	bp
	pop	bx

; *
; * Cancel all ECBs
; *
cancel_ecbs:
	sti
	mov	es, [bx].CONN_S.listener_memory	; ES:SI -> first listening ECB
	xor	si, si
	mov	rv, si

	mov	cx, es
	jcxz	ipx20_close
	mov	cx, [bx].CONN_S.listener_count
	jcxz	ipx20_close

	push	bx
ipx10_close:
	push	cx
	push	si
	push	es

	mov	bx, 6
	push	bp
	call	[IPX_Vector]
	pop	bp

	pop	es
	pop	si
	pop	cx
	add	si, (size ECB_S) - SPX_SIZE
	add	si, Max_Packet_Size
	loop	ipx10_close

	mov	ah, 49h			; Free the memory
	int	21h
	pop	bx

; *
; * Free the connection entry
; *
ipx20_close:
	cli
	inc	Conn_Available
	mov	ax, First_Closed_Conn		; AX -> offset of first closed
	mov	First_Closed_Conn, bx
	xchg	ax, [bx].CONN_S.next		; AX -> next open connection

	mov	si, First_Open_Conn
	cmp	si, bx				; See if this is the first
	jne	ipx30_close

	mov	First_Open_Conn, ax		; -> the next in line
	jmp	short ipx40_close

ipx30_close:
	or	si, si
	jz	ipx40_close
	cmp	[si].CONN_S.next, bx		; See if we were the next
	je	@F				; Jump if we are
	mov	si, [si].CONN_S.next
	jmp	ipx30_close

@@:	mov	[si].CONN_S.next, ax		; Make it's next our old next

ipx40_close:
	sti
	mov	[bx].CONN_S.flags, 0

done_close:
	mov	ax, rv
	ret

ipx_close_connection endp

	end
