; *************************************************************************************************
; *
; *	Title:		PROCESS.ASM
; *	Version:	0.01
; *	Copyright (c) November 1993, Ryu Consulting
; *	Written by Rahner James
; *
; *	This file contains the continuos processing loop for the event manager
; *
; *************************************************************************************************

	include	mouse.inc

MAX_EVENT_VALUE	equ	6
CHECK_COM_FLAG		equ	1


.data
	public	_Event_Vector_Table, _Callback_Event_Vector, _Callback_Table_Size, _Serial_Input_Vector


ifdef CHECK_COM_FLAG
	extern	_Com_Event_Threshold:word
	extern	Cn:word
endif


_Event_Vector_Table label dword
	dd	__null_function			; 0 - no event
	dd	__null_function			; 1 - mouse event
	dd	__null_function			; 2 - keyboard event
	dd	__null_function			; 3 - timer event
	dd	__null_function			; 4 - serial event
	dd	__null_function			; 5 - network event
	dd	__process_callbacks		; 6 - callback


if @Codesize
_Serial_Input_Vector	dw	__null_function, @Code
else
_Serial_Input_Vector	dw	__null_function
endif

_Callback_Event_Vector		dd	0	; Used for a vectored callback value
_Callback_Table_Size		dw	0	; Number of entries in the callback table


Event	MOUSE_EVT	<>			; Event packet used by this function
	dw	0,0

.code
if @Codesize
	extern	get_event:far
else
	extern	get_event:near
endif

ifdef CHECK_COM_FLAG
	if @Codesize
		extern	com_qin:far
	else
		extern	com_qin:near
	endif
endif

; *************************************************************************************************
; *
; *	int __NULL_FUNCTION( MOUSE_EVT FAR *EP )
; *	Used to fill in unused event table entries
; *
; *	Given:
; *		EP -> event packet, ignored
; *
; *	Returns:
; *		Always 0
; *
; *************************************************************************************************
__null_function proc far private

	xor	ax, ax
	ret

__null_function endp


; *************************************************************************************************
; *
; *	int __NULL_NFUNCTION( MOUSE_EVT FAR *EP )
; *	A near NULL function
; *
; *	Given:
; *		EP -> event packet, ignored
; *
; *	Returns:
; *		Always 0
; *
; *************************************************************************************************
__null_nfunction proc near private

	xor	ax, ax
	ret

__null_nfunction endp


; *************************************************************************************************
; *
; *	int __PROCESS_CALLBACKS( CALLBACK_EVT FAR *EP )
; *	Processes a callback event
; *
; *	Given:
; *		EP -> callback event packet
; *
; *	Returns:
; *		Whatever the function does
; *
; *************************************************************************************************
__process_callbacks proc far private, ep:dword

	les	bx, ep			; ES:BX -> event packet
ifdef NULL_CHECK
	mov	ax, es
	or	ax, bx
	jz	short done		; Quit if NULL pointer
endif

; *
; * Check the direct vector
; *
	mov	ax, word ptr es:[bx].CALLBACK_EVT.function
	or	ax, word ptr es:[bx].CALLBACK_EVT.function+2
	jz	short @F

	push	es
	push	bx
	mov	dx, es
	mov	ax, bx
	call	es:[bx].CALLBACK_EVT.function
	add	sp, 4
	jmp	short done

; *
; * Check the indirect vector
; *
@@:
if @Datasize
	mov	ax, ds
	mov	ds, ax
endif
	mov	ax, es:[bx].CALLBACK_EVT.value	; AX = indirect pointer
	cmp	_Callback_Table_Size, ax	; See if we are too big here, this also checks 0 condition
	jbe	short dood			; Quit if too big

	push	es
	push	bx
	les	bx, _Callback_Event_Vector	; ES:BX -> start of the vector table
	mov	cx, es
	or	cx, bx
	jz	short pre_dood			; Quit if NULL pointer
	shl	ax, 2				; AX = offset of the vector we want
	add	bx, ax				; ES:BX -> vector entry we want to go to

if NULL_CHECK
	mov	ax, es:[bx]
	or	ax, es:[bx+2]
	jz	short pre_dood
endif
	call	dword ptr es:[bx]

pre_dood:
	add	sp, 4
dood:	xor	ax, ax
done:	ret

__process_callbacks endp


; *************************************************************************************************
; *
; *	int PROCESS_EVENTS( void )
; *	Checks for an event and processes it
; *
; *	Given:
; *		nothing
; *
; *	Returns:
; *		0 if event was processed OK
; *			value returned from event process otherwise
; *
; *************************************************************************************************
process_events proc

process10_event:
ifdef CHECK_COM_FLAG
if @Datasize
	mov	ax, SEG _Com_Event_Threshold
	mov	es, ax
	cmp	es:_Com_Event_Threshold, 0	; See if we need to process the COM port
	jnz	short @F

	mov	ax, SEG Cn
	mov	es, ax
	mov	ax, es:Cn
	push	ax
else
	cmp	_Com_Event_Threshold, 0		; See if we need to process the COM port
	jnz	short @F
	mov	ax, Cn
	push	ax
endif
	call	com_qin
	add	sp, 2
	or	ax, ax
	jz	short @F
	call	[_Serial_Input_Vector]
	or	ax, ax
	jnz	short done

@@:
endif
	mov	dx, @Data
	mov	ax, offset Event
	push	dx
	push	ax
	call	get_event
	add	sp, 4			; Remove the values
	or	ax, ax			; See if we got a value back
	jnz	short process10_event	; Loop forever

; *
; * Process the event
; *
	mov	bl, Event.etype		; BX = event number
	cmp	bl, MAX_EVENT_VALUE	; See if it's over the top
	ja	short process10_event	; Skip the call if it is
	xor	bh, bh
	shl	bx, 2			; BX *= 4

	mov	dx, @Data
	mov	ax, offset Event
	push	dx
	push	ax
	call	dword ptr _Event_Vector_Table[bx]
	add	sp, 4

	or	ax, ax
	jz	short process10_event

done:	ret

process_events endp

	end
