/**************************************************************************************************
*
*		Title:	QUERY.C
*		Version: 1.00
*		Copyright (c) November 1993, Ryu Consulting, 916/722-1939
*		Written by Rahner James
*
*		This file contains all the structure and header information for the Novell network
*		programming library.
*
*		This code can be compiled with either Microsoft 6.00 or Watcom C 8.5 compilers
*
**************************************************************************************************/

#define	_QUERY_C_


#include	<rjuser.h>
#include	"event.h"
#define		SAP_LISTENERS		5


static	SAP_T FAR		*Server_List;
static	volatile int	Server_Count;
static	int				Max_Servers;
static	unsigned short	Server_Type;


/**************************************************************************************************
*
*	void INCOMING_SAP_ESR( ECB_HEADER_T FAR *EP )
*	Result from a SAP query
*
*	Given:
*		EP -> ECB just filled
*
*	Returns:
*		ECB processed
*
**************************************************************************************************/
#ifdef __WATCOMC__
static void FAR _loadds incoming_sap_esr( ECB_HEADER_T FAR *ep )
#pragma aux incoming_sap_esr parm [es si];
{
	IPX_T FAR *	ipx;
	SAP_T FAR *	sap_ptr;
	int			i, j;

#else
static void FAR _loadds incoming_sap_esr( void )
{
	IPX_T FAR *			ipx;
	SAP_T FAR *			sap_ptr;
	ECB_HEADER_T FAR *	ep;
	int	i, j;

	_asm
	{
		mov		[bp-6], es			// Puts ES:SI in EP
		mov		[bp-8], si
	}
#endif

/*
** See what we have here
*/
	_enable();
	ipx = ep->frag[0].ptr;

	if ( ((i=to_intel(ipx->length)-sizeof(IPX_T)-2) >= sizeof(SAP_T)) && (ep->completionCode == 0) )
	{
/*
** Loop through the server list, looking for ones that are our type
*/
		sap_ptr = (SAP_T FAR *)(((unsigned char FAR *)ep->frag[1].ptr)+2);
		_disable();
		for ( i/=sizeof(SAP_T) ; (i>0) && (Server_Count<Max_Servers) ; --i,++sap_ptr )
		{
			if ( sap_ptr->type == Server_Type )
			{
				if ( Server_Count )
				{
					for ( j=0 ; j<Server_Count ; ++j )
					{
						if ( farcmpw(&sap_ptr->network,&Server_List[j].network,12) == 0 )
							break;
					}
					if ( j < Server_Count )
						continue;
				}

				Server_List[Server_Count].type = sap_ptr->type;
				farmove( Server_List[Server_Count].name, sap_ptr->name, 48 );
				__copy_address( &Server_List[Server_Count].network, &sap_ptr->network );
				Server_List[Server_Count++].socket = sap_ptr->socket;
				_enable();
			}
		}
		_enable();
	}

	ipx_call( 4, ep, 0, 0 );				// IPX Listen for Packet command
}


/**************************************************************************************************
*
*	int SAP_FIND_SERVER( unsigned short SERVER_TYPE, SAP_T FAR *SERVER_LIST, int SERVERS )
*	Builds a server list
*
*	Given:
*		SERVER_TYPE = server type number, Motorola format
*		SERVER_LIST -> array to store the list of servers that match
*		SERVERS = number of server to find
*
*	Returns:
*		Number of servers found
*
**************************************************************************************************/
int sap_find_server( unsigned short server_type, SAP_T FAR *server_list, int servers )
{
	int	i, retry, j;
	ECB_HEADER_T FAR *	listen[SAP_LISTENERS];
	ECB_HEADER_T FAR *	lecb;
	IPX_T FAR *			ipx;
	ECB_HEADER_T FAR *	tecb;
	SAP_T FAR *			sap_ptr;
	struct
	{
		unsigned short	query_type;
		unsigned short	server_type;
	} FAR *query_ptr;
	unsigned short		sap_response;

/*
** Get the buffers we will use
*/
	if ( (tecb=ipx_alloc_ipx_buffer(sizeof(*query_ptr))) == NULL )		// Get the talking buffer
		return 0;
	for ( i=0 ; i<SAP_LISTENERS ; ++i )
	{
		if ( (listen[i]=ipx_alloc_ipx_buffer(0)) == NULL )
		{
			while ( i-- )
				ipx_free_ecb( listen[i] );
			ipx_free_ecb( tecb );
			return 0;
		}
	}

/*
** Open the SAP socket
*/
	sap_response = ipx_call( 0, 0, 0, 0x5204 );						// Open the SAP socket
	if ( (sap_response == 0xFFFF) || (sap_response == 0xFE) )
		goto done_release_memory;

/*
** Fill in the global statics
*/
	Server_List = server_list;
	Server_Count = 0;
	Max_Servers = servers;
	Server_Type = server_type;

/*
** Setup the listeners
*/
	for ( i=0 ; i<SAP_LISTENERS ; ++i )
	{
		listen[i]->socketNumber = 0x5204;
		listen[i]->ESRAddress = incoming_sap_esr;
		sap_ptr = (SAP_T FAR *)((char FAR *)listen[i]->frag[1].ptr+2);
		for ( j=0 ; j<7 ; ++j )
			sap_ptr[j].type = 0;
		ipx_call( 4, listen[i], 0,0 );
	}

/*
** Setup the SAP query packet
*/
	tecb->socketNumber = 0x5204;
	__set_broadcast( tecb->immediateAddress );
	ipx = tecb->frag[0].ptr;
	ipx_call( 9, &ipx->destination, 0,0 );		// Get the local address
	ipx->destination.socket = 0x5204;
	__set_broadcast( ipx->destination.node );
	query_ptr = (void FAR *)(tecb->frag[1].ptr);
	query_ptr->query_type = servers == 1 ? 0x300 : 0x100;	// One server == get nearset, > 1 == get all
	query_ptr->server_type = server_type;

/*
** Loop through the retries, hunting for packets
*/
	for ( retry=0 ; (retry<3) && (Server_Count<servers) ; ++retry )
	{
		ipx_call( 3, tecb, 0,0 );					// Send the query out
		while ( tecb->inUseFlag )					// Wait for it to finish sending
			;
		ipx_call( 5, tecb, 6, 0 );					// Schedule it to retrigger after a few system ticks
		while ( tecb->inUseFlag && (Server_Count<servers) )
			;
	}

/*
** Clean up and exit
*/
	for ( i=0 ; i<SAP_LISTENERS ; ++i )
		ipx_call( 6, listen[i], 0,0 );
	if ( sap_response == 0 )
		sap_response = ipx_call( 1, 0, 0, 0x5204 );					// Close the SAP socket

done_release_memory:
	for ( i=0 ; i<SAP_LISTENERS ; ++i )
		ipx_free_ecb( listen[i] );
	ipx_free_ecb( tecb );

	return Server_Count;
}
