/**************************************************************************************************
*
*		Title:	SAP.C
*		Version: 1.00
*		Copyright (c) November 1993, Ryu Consulting, 916/722-1939
*		Written by Rahner James
*
*		This file contains all the structure and header information for the Novell network
*		programming library.
*
*		This code can be compiled with either Microsoft 6.00 or Watcom C 8.5 compilers
*
**************************************************************************************************/

#define	_SAP_C_
#include	<rjuser.h>
#include	"event.h"

#define	MAX_SERVERS		7		// Maximum number of file servers that can be advertised

static struct SAPPER_S
{
	ECB_HEADER_T	ecb;				// ECB used for SAP transmissions
	struct
	{
		void FAR *		ptr;
		unsigned short	size;
	} frag[MAX_SERVERS];				// -> all the servers
	IPX_T			ipx;				// IPX header to use with the SAP
	unsigned short	response_type;		// Type of SAP response, 0x200 for periodic, 0x400 for nearest
	SAP_T			server[MAX_SERVERS];// Storage of each of the servers
} Sap;

static struct SAP_LISTENER_S
{
	ECB_HEADER_T	ecb;
	IPX_T			ipx;
	unsigned short	query_type;		// Type of SAP query
	unsigned short	server_type;	// Type of server they are hunting for
	SAP_T			server[MAX_SERVERS];// Storage of each of the servers, here because of overwrite problem
} Listener;

static	ECB_HEADER_T	Aes;
static	int	Server_Count = 0;	// Number of servers currently being represented


/**************************************************************************************************
*
*	static void FAR _loadds AES_ESR( void )
*	ESR for the SAP AES
*
*	Given:
*		Aes = AES trigger
*
*	Returns:
*		Aes processed
*
**************************************************************************************************/
static void FAR _loadds aes_esr( void )
{
	if ( Server_Count )
	{
		if ( Sap.ecb.completionCode == 0 )
		{
			if ( Sap.ecb.inUseFlag )			// See if already being used
			{
				_enable();
				ipx_call( 5, &Aes, 18, 0 );	// Schedule it for a second later
			}
			else
			{
				Sap.ecb.fragmentCount = Server_Count + 1;
				_enable();
				__set_broadcast( Sap.ecb.immediateAddress );
				__set_broadcast( Sap.ipx.destination.node );
				Sap.ipx.destination.network = Sap.ipx.source.network;
				Sap.ipx.destination.socket = 0x5204;

				ipx_call( 3, &Sap.ecb, 0, 0 );
				ipx_call( 5, &Aes, 18*60, 0 );
			}
		}
	}
}


/**************************************************************************************************
*
*	static void FAR _loadds INCOMING_SAP_ESR( void )
*	ECB filled by a remote query
*
*	Given:
*		Listener = ECB just filled
*
*	Returns:
*		Listener ECB processed
*
**************************************************************************************************/
static void FAR _loadds incoming_sap_esr( void )
{
	int	i;

	_enable();
	if ( Listener.ecb.completionCode == 0 )
	{
/*
** See if they are looking for one of ours
*/
		if ( Server_Count && __cmp_address(&Listener.ipx.source,&Sap.ipx.source) )
		{
			if ( to_intel(Listener.ipx.length) == sizeof(IPX_T)+4 )
			{
				i = 0;
				if ( Listener.server_type != 0xFFFF )
				{
					for ( ; i<Server_Count ; ++i )
					{
						if ( Sap.server[i].type == Listener.server_type )
							break;
					}
				}
				if ( i < Server_Count )
				{
					_disable();
					if ( Sap.ecb.inUseFlag == 0 )
					{
						__copy_node( Sap.ecb.immediateAddress, Listener.ecb.immediateAddress );
						farmove( &Sap.ipx.destination, &Listener.ipx.source, sizeof(NODE_T) );
//					Sap.ecb.fragmentCount = Server_Count + 1;
						Sap.ecb.inUseFlag = 0xFF;
						_enable();
						ipx_call( 3, &Sap.ecb, 0, 0 );
					}
					else
					{
						_enable();
					}
				} // if (i<
			} // if (to_
		} // if (Serv
	}
	else
	{
/*
** Any error except a malformed packet, quit operating
*/
		if ( Listener.ecb.completionCode != 0xFD )
			return;
	}

	ipx_call( 4, &Listener, 0, 0 );				// IPX Listen for Packet command
}


/**************************************************************************************************
*
*	int SAP_ADVERTISE_SERVER( unsigned short SERVER_TYPE, char FAR *SERVER_NAME, unsigned short SOCKET )
*	Starts serving a server
*
*	Given:
*		SERVER_TYPE = server type number, Motorola format
*		SERVER_NAME -> ASCIIZ name of server to advertise
*		SOCKET = socket number that server expects request from
*
*	Returns:
*		0 if all went OK
*		-1 if IPX not present
*		-2 if not enough room for the new server
*		-11 if bad passed parameter
*
**************************************************************************************************/
int sap_advertise_server( unsigned short server_type, char FAR *server_name, unsigned short socket )
{
	int	i;

/*
** See if the server has been started yet
*/
	if ( Server_Count == 0 )
	{
/*
** Initialize the listener
*/
		i = ipx_call( 0, NULL, 0, 0x5204 );
		if ( i && (i != 0xFF) )
			return -1;
		if ( ipx_call(9,&Sap.ipx.source,0,0) )					// May need the address before the first listener sent out
			return -1;

		Listener.ecb.ESRAddress = incoming_sap_esr;
		Listener.ecb.inUseFlag = 0;
		Listener.ecb.socketNumber = 0x5204;
		Listener.ecb.fragmentCount = 1;
		Listener.ecb.frag[0].ptr = &Listener.ipx;
		Listener.ecb.frag[0].size = sizeof( IPX_T ) + 4;		// The IPX header plus the two entries
		if ( ipx_call(4,&Listener.ecb,0,0) )
			return -1;

/*
** Fill in the SAP ECB
*/
		Sap.ecb.ESRAddress = NULL;
		Sap.ecb.inUseFlag = 0;
		Sap.ecb.socketNumber = 0x5204;
		__set_broadcast( Sap.ecb.immediateAddress );
		Sap.ecb.fragmentCount = 1;
		Sap.ecb.frag[0].ptr = &Sap.ipx;
		Sap.ecb.frag[0].size = sizeof( IPX_T ) + 2;		// The IPX header plus the response type entry

/*
** Fill in the IPX header
*/
		Sap.ipx.packetType = 0;
		Sap.ipx.destination.socket = 0x5204;
		__set_broadcast( Sap.ipx.destination.node );
		Sap.ipx.destination.network = Sap.ipx.source.network;

/*
** Fill in the AES
*/
		Aes.ESRAddress = aes_esr;
		Aes.socketNumber = 0x5204;
		Aes.inUseFlag = 0;

/*
** Fill out the rest of the SAP entries
*/
		Sap.response_type = 0x200;

		for ( i=0 ; i<MAX_SERVERS ; ++i )
		{
			Sap.frag[i].ptr = &Sap.server[i];
			Sap.frag[i].size = sizeof( SAP_T );
		}

	}

/*
** Add in the new server if we can
*/
	if ( (server_type == 0) || (server_type == 0xFFFF) || (socket == 0) || (socket == 0x5204) )
		return -11;
	if ( Server_Count >= MAX_SERVERS )
		return -2;

	for ( i=0 ; i<Server_Count ; ++i )
	{
		if ( (Sap.server[i].type == server_type) && (farcmpw(server_name,Sap.server[i].name,48) == 0) )
			break;
	}

/*
** Add it in if it's a new one
*/
	if ( i == Server_Count )
	{
		Sap.server[i].type = server_type;
		farset( Sap.server[i].name, 0, 48 );
		farmove( Sap.server[i].name, server_name, farnlen(server_name,47) );
		__copy_address( &Sap.server[i].network, &Sap.ipx.source );
		Sap.server[i].socket = socket;
		Sap.server[i].intermediate_networks = 0x100;

		++Server_Count;
		++Sap.ecb.fragmentCount;
	}

/*
** Send out the SAP if we can
*/
	if ( Sap.ecb.inUseFlag == 0 )
	{
		ipx_call( 3, &Sap.ecb, 0, 0 );
		ipx_call( 5, &Aes, 60*18, 0 );
	}

	return 0;
}


/**************************************************************************************************
*
*	int SAP_STOP_SERVER( char FAR *SERVER_NAME )
*	Stops serving a server
*
*	Given:
*		SERVER_NAME -> ASCIIZ name of server to stop, if NULL all will be stopped
*
*	Returns:
*		0 if all went OK
*		-1 if IPX not present
*		-2 if not enough room for the new server
*		-11 if bad passed parameter
*
**************************************************************************************************/
int sap_stop_server( char FAR *server_name )
{
	int	i, j;

	if ( server_name == FNULL )
	{
/*
** Do them all
*/
		Server_Count = 0;			// This disables all the server retransmissions
	}
	else
	{
/*
** Find a server that matches
*/
		for ( i=0 ; i<Server_Count ; ++i )
		{
			if ( farcmpw(server_name,Sap.server[i].name,48) == 0 )
				break;
		}
		if ( i >= Server_Count )
			return 0;				// Quit if we couldn't find it

/*
** Remove the server that we have found
*/
		_disable();
		if ( --Server_Count > i )
			farmove( &Sap.server[i], &Sap.server[i+1], sizeof(SAP_T)*(Server_Count-i) );
		_enable();
	}

/*
** See if we need to stop the whole thing
*/
	if ( Server_Count == 0 )
	{
		ipx_call( 6, &Listener, 0,0 );	// Stop the listener, do first cuz this can trigger the rest
		ipx_call( 6, &Aes, 0,0 );		// Stop the timer
		ipx_call( 6, &Sap, 0,0 );		// Stop the SAP
	}

	return 0;
}
