; *************************************************************************************************
; *
; *	Title:	SEMAPHOR.ASM
; *	Copyright (c) December 1991, Ryu Consulting, 916/722-1939
; *	Written by Rahner James
; *
; *	This file contains all the functions to support semaphores on a Novell network
; *
; *************************************************************************************************


	include mouse.inc

.code
; *************************************************************************************************
; *
; *	int OPEN_SEMAPHORE( char FAR *SEMAPHORE, char INITIAL, long FAR *HANDLE )
; *	Opens a semaphore with a particular name
; *
; *	Given:
; *		SEMAPHORE -> name of the semaphore to open
; *		INITIAL = initial count
; *		HANDLE -> filled with the open semaphore handle
; *
; *	Returns:
; *		Current count of the processes that have the semaphore open
; *		 0 = bad parameter passed
; *		-1 = invalid name length
; *		-2 = invalid name
; *
; *************************************************************************************************
open_semaphore proc uses di si ds, semaphore:dword, initial:byte, handle:dword
local	request[128]:byte

	mov	ax, word ptr handle
	or	ax, word ptr handle+2
	jz	short done

; *
; * Setup the call
; *
	cld
	les	di, semaphore		; ES:DI -> semaphore name
	mov	si, di			; DS:SI ->    "
	mov	ax, es
	mov	ds, ax

; *
; * Look for the length of the semaphore name
; *
	xor	al, al			; AL = 0
	mov	cx, 127
	mov	bx, cx
	repne	scasb
	cmc
	sbb	bx, cx			; BX = string length or 127
	jz	short done		; Quit if "" string
	mov	request, bl

; *
; * Move the semaphore name
; *
	lea	di, request+1		; ES:DI -> request buffer
	mov	ax, ss
	mov	es, ax
	mov	cx, bx			; CX = string length
	shr	cx, 1
	rep	movsw			; Words may not matter here cuz we are probably not gunna be on a word boundary
	adc	cx, cx
	rep	movsb

; *
; * Make the call
; *
	mov	ax, 0c500h
	mov	dx, ss
	mov	ds, dx
	lea	dx, request
	mov	cl, initial
	int	21h
	or	al, al			; See if error return
	jnz	short done

	mov	al, bl			; AL = number of opens done to this semaphore
	lds	si, handle
	mov	[si], dx
	mov	[si+2], cx

done:	cbw
	ret
open_semaphore endp


; *************************************************************************************************
; *
; *	int close_semaphore( long HANDLE )
; *	Closes a semaphore associated with a handle
; *
; *	Given:
; *		HANDLE = handle of the semaphore to close
; *
; *	Returns:
; *		000h if semaphore is open a ready for business
; *		0FFFFh if invalid handle
; *
; *************************************************************************************************
close_semaphore proc, handle:dword

	mov	ax, 0c504h		; Close semaphore command
	mov	dx, word ptr handle
	mov	cx, word ptr handle+2
	int	21h
	cbw
	ret

close_semaphore endp

	end

